package FI2.Dictionary;
/**{p}
 * -- FI2.Dictionary --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Dizionari
 */

/**{f}
 * InspectableLocatorOrderedDictionary.java
 * interfaccia dizionario read-only 
 * con ordinamento totale e supporto riferimenti
 * (Locator)
 *
 * @version 1.00 00/09/25
 * @author M.Moro DEI UPD
 */

import FI2.Set.Locator;
import FI2.Exceptions.InvalidKeyException;

/**{i}
 * dizionario con metodi di sola lettura
 * e con ordinamento totale e supporto ai
 * riferimenti (Locator) -
 * puo` contenere anche piu` elementi con la stessa
 * chiave
 * @see InspectableLocatorDictionary
 * @see InspectableOrderedDictionary
 */
public interface InspectableLocatorOrderedDictionary 
  extends InspectableLocatorDictionary,
  InspectableOrderedDictionary
{
    /**[m][a]
     * ricerca la maggior chiave <= di quella fornita,
     * associata ad un elemento nel dizionario
     * e ne ritorna un riferimento
     * @param key  Locator all'elemento con chiave di ricerca
     * @return un Locator alla maggior chiave <= key 
     *         se un item con tale chiave e` presente,
     *         NO_SUCH_KEY altrimenti
     * @exception InvalidKeyException
     * chiave illegale per il dizionario
     */
    public Locator before (Locator key) 
      throws InvalidKeyException;
  
    /**[m][a]
     * ricerca la minor chiave >= di quella fornita,
     * associata ad un elemento nel dizionario
     * e ne ritorna un riferimento
     * @param key  Locator all'elemento con chiave di ricerca
     * @return un Locator alla minor chiave >= key 
     *         se un item con tale chiave e` presente,
     *         NO_SUCH_KEY altrimenti
     * @exception InvalidKeyException
     * chiave illegale per il dizionario
     */
    public Locator after (Locator key) 
      throws InvalidKeyException;
  
    /**[m][a]
     * ricerca la maggior chiave <= di quella fornita,
     * associata ad un elemento nel dizionario
     * e ne ritorna un riferimento
     * @param key  chiave di ricerca
     * @return la maggior chiave <= key 
     *         se un item con tale chiave e` presente,
     *         NO_SUCH_KEY altrimenti
     * @exception InvalidKeyException
     * chiave illegale per il dizionario
     */
    public Locator closestBefore (Object key) 
      throws InvalidKeyException;
  
    /**[m][a]
     * ricerca la minor chiave >= di quella fornita,
     * associata ad un elemento nel dizionario
     * e ne ritorna un riferimento
     * @param key  chiave di ricerca
     * @return la minor chiave >= key 
     *         se un item con tale chiave e` presente,
     *         NO_SUCH_KEY altrimenti
     * @exception InvalidKeyException
     * chiave illegale per il dizionario
     */
    public Locator closestAfter (Object key) 
      throws InvalidKeyException;
  
    /**[m][a]
     * ritorna un Locator all'elemento
     * con la chiave piu` piccola
     * @return l'elemento associato alla minor chiave,
     *         NO_SUCH_KEY se dizionario vuoto
     */
    public Locator first();
  
    /**[m][a]
     * ritorna un Locator all'elemento
     * con la chiave piu` grande
     * @return l'elemento associato alla maggior chiave,
     *         NO_SUCH_KEY se dizionario vuoto
     */
    public Locator last();
  
} //{i} InspectableLocatorOrderedDictionary
