package FI2.Dictionary;
/**{p}
 * -- FI2.Dictionary --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Dizionari
 */

/**{f}
 * SeparateChainingHashTable.java
 * dizionario realizzato in forma di tabella hash
 * con bucket array -
 * derivata da HashTable della v 1.x 
 *
 * @version 2.00 00/09/27
 * @author M.Moro DEI UPD
 */

import java.io.*;
import java.util.NoSuchElementException;
import FI2.Set.ObjectIterator;
import FI2.Set.Item;
import FI2.Set.Position;
import FI2.Set.Container;
import FI2.Set.InspectableContainer;
import FI2.Set.InspectableKeyBasedContainer;
import FI2.Util.Equality;
import FI2.Util.ObjectEqual;
import FI2.Util.VectorIter;
import FI2.Util.FI2Sys;
import FI2.Exceptions.InvalidKeyException;

/**{c}
 * tabella hash - gli elementi del bucket array 
 * sono Dictionary realizzati come sequenze la cui 
 * implementazione viene decisa all'atto della creazione 
 * della tabella passando un parametro
 * di tipo Dictionary ed utilizando di questo il metodo 
 * generico newContainer() - 
 * si assume disponibile una funzione di hash associata 
 * agli elementi (tipicamente Item) che ritorna un int
 * @see Dictionary
 */
public class SeparateChainingHashTable 
  implements Dictionary  
{
    public static final int DICTCAPACITY = 101;
      // capacita` del dizionario

    private Equality E;  // comparatore
    private Dictionary T[];  // bucket array
    private int capacity;  // capacita`
    private int size;  // numero totale elementi inseriti

    /**[c]
     * costruttore base, le sequenze sono vuote
     * @param e  comparatore per i confronti di chiave
     * @param d  dizionario per gli elementi 
     *           di bucket array
     */
    public SeparateChainingHashTable(Equality e, Dictionary d)  
    {
        this(e, d, DICTCAPACITY);
    } //[c] SeparateChainingHashTable

    /**[c]
     * costruttore con capacita`
     * @param e  comparatore per i confronti di chiave
     * @param d  dizionario per gli elementi 
     *           di bucket array
     * @param cap  capacita` (dovrebbe essere un numero primo)
     * @exception IllegalArgumentException  se comparatore
     * nullo
     */
    public SeparateChainingHashTable(Equality e, Dictionary d, 
      int cap)  
    {
        if ((E = e) == null)
          throw new IllegalArgumentException(
          "Comparatore indicato nullo!");
        T = new Dictionary[capacity=cap];
        size = 0;
        for (int i=0; i<cap; 
          T[i++] = (Dictionary)d.newContainer());
          // gli elementi del bucket array sono sequenze
          // realizzate dalla classe di d
    } //[c] SeparateChainingHashTable

    /**[c] !!
     * costruttore con una copia dei riferimenti contenuti
     * nell'array fornito -
     * effettua l'ordinamento degli elementi -
     * gli oggetti non vengono clonati 
     * @param key  array di chiavi da copiare
     * @param val  array di dati da copiare
     * @param e  comparatore per i confronti di chiave
     * @param s  sequenza per gli elementi di bucket array
     * @param cap  capacita` (dovrebbe essere un numero primo)
     */
    public SeparateChainingHashTable(Object key[], Object val[], 
      Equality e, Dictionary d, int cap)
    {
        this(e, d, cap);
        for (int i=0; i<key.length; i++)
            insertItem(key[i], val[i]);
    }  //[c] SeparateChainingHashTable


    // metodi ausiliari

    /**[m]
     * chiave associata ad una posizione
     * @param position  posizionatore
     * @return la chiave 'posizionata'
     */
    protected Object key(Position position)  
    {
        return ((Item) position.element()).key();
    } //[m] key

    /**[m]
     * elemento associato ad una posizione
     * @param position  posizionatore
     * @return l'elemento 'posizionato'
     */
    protected Object element(Position position)  
    { 
        return ((Item) position.element()).element(); 
    } //[m] element

    /**[m]
     * verifica se la chiave e` legale
     * @param key  chiave da verificare
     * @exception InvalidKeyException
     * se chiave illegale
     */
    protected void checkKey(Object key) 
      throws InvalidKeyException 
    {
        if (!E.isComparable(key))
          throw new InvalidKeyException(
          "La chiave <"+key+"> non e` confrontabile");
    } //[m] checkKey

    /**[m]
     * calcolo della chiave hash ridotta
     * @param key  chiave
     * @return chiave hash ridotta
     */
    protected int reducedHashCode(Object key)  
    { 
        return (key.hashCode() & 0x7fffffff) % capacity; 
    } //[m] reducedHashCode

    // metodi dell'interfaccia Dictionary

    /**[m]
     * @see InspectableContainer
     */
    public int size()  { return size; }

    /**[m]
     * @see InspectableContainer
     */
    public boolean isEmpty() { return size==0; }

    /**[m]
     * @see InspectableContainer
     */
    public ObjectIterator elements()  
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per scorrere i valori -
         * si appoggia sull'iteratore elements() delle
         * sequenze-dizionario
         */
        class ElemsIter implements ObjectIterator
        {
            private ObjectIterator it;
              // iteratore sulla sequenza
            private int curIdx;
              // indice in bucket array

            /**[c]
             * costruttore di default
             */
            ElemsIter()
            {
                for (curIdx = 0, it = T[curIdx].elements(); 
                  !it.hasNext() && 
                  curIdx <= capacity-2;
                  it = T[++curIdx].elements());
                  // cerca la prima sequenza non vuota;
                  // si ferma in ogni caso all'ultima
                  // sequenza (anche se vuota)
            } //[c] ElemsIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                boolean someElem;

                // cerca una sequenza non vuota;
                // si ferma in ogni caso all'ultima
                // sequenza (anche se vuota)
                for ( ; 
                  ! (someElem = it.hasNext()) && 
                  curIdx <= capacity-2;
                  it = T[++curIdx].elements());
                return someElem;
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                if (!it.hasNext())
                    throw new NoSuchElementException(
"SeparateChainingHashTable.ElemsIter.nextObject(): fine elementi"
                  );
                return it.nextObject();
            } //[m] nextObject

        } //{c}{l} ElemsIter

        return new ElemsIter();
    } //[m] elements

   /**[m]
     * @see Container
     */
    public Container newContainer()
    {
        Equality e = new ObjectEqual();
        return new SeparateChainingHashTable(e, 
          new UnOrderedSLList(e), 
          DICTCAPACITY);
    } //[m] newContainer

    /**[m]
     * @see InspectableKeyBasedContainer
     */
    public ObjectIterator keys()  
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per scorrere le chiavi -
         * si appoggia sull'enumeratore keys() delle
         * sequenze-dizionario
         */
        class KeysIter implements ObjectIterator
        {
            private ObjectIterator it;
              // iteratore sulla sequenza
            private int curIdx;
              // indice in bucket array

            /**[c]
             * costruttore di default
             */
            KeysIter()
            {
                for (curIdx = 0, it = T[curIdx].keys(); 
                  !it.hasNext() && 
                  curIdx <= capacity-2;
                  it = T[++curIdx].keys());
                  // cerca la prima sequenza non vuota;
                  // si ferma in ogni caso all'ultima
                  // sequenza (anche se vuota)
            } //[c] KeysIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                boolean someKey;

                // cerca una sequenza non vuota;
                // si ferma in ogni caso all'ultima
                // sequenza (anche se vuota)
                for ( ; 
                  ! (someKey = it.hasNext()) && 
                  curIdx <= capacity-2;
                  it = T[++curIdx].keys());
                return someKey;
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                if (!hasNext())
                    throw new NoSuchElementException(
"SeparateChainingHashTable.KeysIter.nextObject(): fine elementi"
                  );
                return it.nextObject();
            } //[m] nextObject

        } //{c}{l} KeysIter

        return new KeysIter();
    } //[m] keys

    /**[m]
     * @see InspectableDictionary
     */
    public Object findElement(Object key) 
      throws InvalidKeyException 
    {
        return T[reducedHashCode(key)].findElement(key);
    } //[m] findElement

    /**[m]
     * @see InspectableDictionary
     */
    public ObjectIterator findAllElements(Object key) 
      throws InvalidKeyException  
    {
        return T[reducedHashCode(key)].findAllElements(key);
    } //[m] findAllElements

    /**[m]
     * @see Dictionary
     */
    public void insertItem(Object key, Object element)
      throws InvalidKeyException  
    {
        T[reducedHashCode(key)].insertItem(key, element);
        size++;
    } //[m] insertItem

    /**[m]
     * @see Dictionary
     */
    public Object replaceElement(Object key, Object element) 
      throws InvalidKeyException  
    {
        return T[reducedHashCode(key)].replaceElement(
          key, element);
    } //[m] replaceElement

    /**[m]
     * @see Dictionary
     */
    public Object removeElement(Object key) 
      throws InvalidKeyException  
    {
        Object toReturn =
          T[reducedHashCode(key)].removeElement(key);
        if (toReturn != NO_SUCH_KEY)
            size--;
        return toReturn;
    } //[m] removeElement

    /**[m]
     * @see Dictionary
     */
    public ObjectIterator removeAllElements(Object key) 
      throws InvalidKeyException  
    {
        ObjectIterator elems = findAllElements(key);
        while (elems.hasNext())
        {
            size--;
            elems.nextObject();
        }
        return T[reducedHashCode(key)].removeAllElements(key);
    } //[m] removeAllElements

    /**[m]
     * conversione a stringa -
     * produce il contenuto dell'albero
     * @return stringa contenuto del dizionario 
     */
    public String toString()
    {
        String str = "Dizionario con HastTable\r\n";
        for (int i=0; i<capacity; i++)
            if (!T[i].isEmpty())
                str += 
                "Sequenza con redHashCode=" + i + "\r\n" + 
                T[i].toString();
        return str;
    } //[m] toString

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
 "Dizionario: Battere codice 6 cifre + stringa (fine=Ctrl-Z):"
              );
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        BufferedReader re = new BufferedReader(new
          InputStreamReader(in));

        Equality eq = new AKeyComp();
        SeparateChainingHashTable myDictio = 
          new SeparateChainingHashTable(eq, 
          new UnOrderedSLList(eq));

        try
        {
            String str;
            while((str = re.readLine()) != null)
                myDictio.insertItem(str.substring(0,6), 
                  str.substring(6));
                  // la chiave sono i primi 6 caratteri
                  // il resto i dati
        }
        catch (IOException e)
        {
            // termina il ciclo
        } //catch(EOF)

        FI2Sys.out.println(myDictio);

        FI2Sys.out.println("numEl=" + myDictio.size());
        FI2Sys.out.println("I primi 3 elementi sono:");
        ObjectIterator it = myDictio.elements();

        int pos1 = 1;
        while (pos1++ <= 3 && it.hasNext())
            FI2Sys.out.println(
              (String)it.nextObject());

        Object tmp;
        FI2Sys.out.println("La stringa con chiave 123456 " + (
          (tmp = myDictio.findElement("123456")) == 
           NO_SUCH_KEY ?
          "non e` presente" :
          ("e` presente e vale ->" + tmp + "<-")
          )
          );

        it = myDictio.findAllElements("123456");
        FI2Sys.out.println(
          "Le stringhe con chiave 123456 sono:");
        while (it.hasNext())
            FI2Sys.out.println(
              (String)it.nextObject());
        ObjectIterator itk = myDictio.keys();
        FI2Sys.out.println("Dizionario completo:");
        VectorIter vi = new VectorIter();
        while (itk.hasNext())
        {
            Object key = itk.nextObject();
            it = myDictio.findAllElements(key);

            // la prima viene copiata
            if (it.hasNext())
            {
                Object val;
                vi.insertLast(
                  new Item(key, val = it.nextObject()));
                FI2Sys.out.println("key=" + key + 
                  " val=" + val);
            }
            while (it.hasNext())
                FI2Sys.out.println("key=" + key + 
                  " val=" + it.nextObject());
        } // while itk

        Object tmpkey[] = new
          Object[vi.size()];
        Object tmpval[] = new
          Object[vi.size()];
        int i = 0;
        while(vi.hasNext())
        {
            Item ie = (Item)vi.nextObject();
            tmpkey[i] = ie.key();
            tmpval[i++] = ie.element();
        }
        
        Dictionary sndDictio = 
          new SeparateChainingHashTable(tmpkey, tmpval,
          eq, new UnOrderedSLList(eq), DICTCAPACITY);
        sndDictio.removeElement("654321");
        sndDictio.replaceElement("123456", 
          "------- NUOVO -------");
        FI2Sys.out.println(
  "Il secondo senza la chiave 654321 e con 123456 aggiornato:"
          );
        FI2Sys.out.println(sndDictio);
        FI2Sys.out.println("numEl=" + sndDictio.size());
    } //[m][s] main
    
} //{c} SeparateChainingHashTable
