package FI2.Dictionary;
/**{p}
 * -- FI2.Dictionary --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Dizionari
 */

/**{f}
 * SimpleBinarySearchTree.java
 * dizionario realizzato in forma di albero binario
 * di ricerca (non bilanciato)
 *
 * @version 2.00 00/09/27
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Util.FI2Sys;
import FI2.Util.Comparator;
import FI2.Util.VectorIter;
import FI2.Set.Position;
import FI2.Set.PositionIterator;
import FI2.Set.ObjectIterator;
import FI2.Set.Container;
import FI2.Set.InspectableContainer;
import FI2.Set.InspectableKeyBasedContainer;
import FI2.Set.Item;
import FI2.Tree.ExpandBinaryTree;
import FI2.Exceptions.InvalidKeyException;
import FI2.Tree.LinkedBinaryTree;
import FI2.Tree.BinTreeBasicAlgo;

/**{c}
 * albero binario di ricerca - si appoggia su un albero
 * binario in forma concatenata e presenta autobilanciamento -
 * gli elementi dell'albero sono Item chiave-oggetto -
 * la comparazione delle chiavi avviene 
 * mediante un comparatore fornito all'atto della costruzione
 * @see OrderedDictionary
 */
public class SimpleBinarySearchTree 
  implements OrderedDictionary  
{
    private static final double BALANCEFACTOR = 2.5;
      // fattore di bilanciamento

    private Comparator C;  // comparatore
    private ExpandBinaryTree T;  // albero binario
    protected Position actionPos; 
      // posizione di inserimento oppure
      // padre della posizione di rimozione

    /**[c]
     * costruttore base, albero vuoto
     * @param c  comparatore per i confronti di chiave
     */
    public SimpleBinarySearchTree(Comparator c)  
    {
        C = c;
        T = (ExpandBinaryTree) new LinkedBinaryTree();
    } //[c] SimpleBinarySearchTree

    /**[c] !!
     * costruttore con una copia dei riferimenti contenuti
     * nell'array fornito -
     * effettua l'ordinamento degli elementi -
     * gli oggetti non vengono clonati 
     * @param key  array di chiavi da copiare
     * @param val  array di dati da copiare
     * @param c  comparatore per i confronti di chiave
     */
    public SimpleBinarySearchTree(Object key[], Object val[], 
      Comparator c)
    {
        C = c;
        T = (ExpandBinaryTree) new LinkedBinaryTree();
        for (int i=0; i<key.length; i++)
            insertItem(key[i], val[i]);
    }  //[c] SimpleBinarySearchTree


    // metodi ausiliari

    /**[m]
     * chiave associata ad una posizione
     * @param position  posizionatore
     * @return la chiave 'posizionata'
     */
    protected Object key(Position position)  
    {
        return ((Item) position.element()).key();
    } //[m] key

    /**[m]
     * elemento associato ad una posizione
     * @param position  posizionatore
     * @return l'elemento 'posizionato'
     */
    protected Object element(Position position)  
    { 
        return ((Item) position.element()).element(); 
    } //[m] element

    /**[m]
     * verifica se la chiave e` legale
     * @param key  chiave da verificare
     * @exception InvalidKeyException
     * se chiave illegale
     */
    protected void checkKey(Object key) 
      throws InvalidKeyException 
    {
        if (!C.isComparable(key))
          throw new InvalidKeyException(
          "La chiave <"+key+"> non e` confrontabile");
    } //[m] checkKey

    /**[m]
     * ricerca ricorsiva di una chiave
     * @param key  chiave da ricercare
     * @param pos  posizione di partenza
     * @return la posizione trovata, 
     *         altrimenti la posizione di un nodo foglia
     */
    protected Position findPosition(Object key, Position pos) 
    {
        if (T.isExternal(pos))
            return pos; 
            // chiave non trovata e raggiunto un nodo foglia
        else 
        {
            Object curKey = key(pos);
            if(C.isLessThan(key, curKey))
                // minore, cerca a sinistra
                return findPosition(key, T.leftChild(pos));
            else if(C.isGreaterThan(key, curKey))
                // maggiore, cerca a destra
                return findPosition(key, T.rightChild(pos));
            else
                // trovata
                return pos;
        } // else (isExternal)
    } //[m] findPosition

    /**[m]
     * costruzione ricorsiva di un iteratore degli
     * elementi con la chiave fornita
     * @param key  chiave da ricercare
     * @param pos  posizione di partenza
     * @return l'iteratore costruito
     */
    protected VectorIter
      findAllVectorIter(Object key, Position pos) 
    {
        VectorIter result = new VectorIter();
          // iteratore vuoto
        Position curPos = findPosition(key, pos);
        if (T.isInternal(curPos)) {
            // il primo elemento trovato potrebbe essere 
            // seguito da altri con la stessa chiave
            VectorIter leftResult = new VectorIter();
            VectorIter rightResult = new VectorIter();
              // due enumeratori per i sottoalberi figlio
              // inizialmente vuoti
            if(T.isInternal(T.leftChild(curPos)))
                // il sottoalbero figlio di sinistra ha chiavi
                leftResult = findAllVectorIter(key, 
                  T.leftChild(curPos));
            if(T.isInternal(T.rightChild(curPos)))
                // il sottoalbero figlio di destra ha chiavi
                rightResult = findAllVectorIter(key, 
                  T.rightChild(curPos));
            // costruzione dell'enumeratore, come
            // una visita in ordine
            result.insertLast(element(curPos));
            for (int i = 0; i < leftResult.size(); i++)
                result.insertLast(leftResult.elemAtRank(i));
            for (int i = 0; i < rightResult.size(); i++)
                result.insertLast(rightResult.elemAtRank(i));
        } //if isInternal
        return result;
    } //[m] findAllVectorIter


    // metodi dell'interfaccia OrderedDictionary

    /**[m]
     * @see InspectableContainer
     */
    public int size()  { return (T.size()-1)/2; }

    /**[m]
     * @see InspectableContainer
     */
    public boolean isEmpty() { return T.size() == 1; }

    /**[m]
     * @see InspectableContainer
     */
    public ObjectIterator elements()  
    {
        VectorIter validElemIter = new VectorIter();
        PositionIterator posIter;
          // usa l'iteratore sull'albero
        Position curPos;
        
        for(posIter = T.positions(); 
          posIter.hasNext();)  
        {
            curPos = posIter.nextPosition();
            if(T.isInternal(curPos))
                validElemIter.insertLast(element(curPos));
        }

        return validElemIter;
    } //[m] elements

   /**[m]
     * @see Container
     */
    public Container newContainer()
    {
        return new SimpleBinarySearchTree(C);
          // stesso comparatore
    } //[m] newContainer

    /**[m]
     * @see InspectableDictionary
     */
    public Object findElement(Object key) 
      throws InvalidKeyException 
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        Position curPos = findPosition(key, T.root());
        actionPos = curPos;
        if (T.isInternal(curPos))
            // trovata la chiave
            return element(curPos);
        else
            // non trovata la chiave
            return NO_SUCH_KEY;
    } //[m] findElement

    /**[m]
     * @see InspectableDictionary
     */
    public ObjectIterator findAllElements(Object key) 
      throws InvalidKeyException  
    {
        checkKey(key); 
          // puo` sollevare InvalidKeyException
        return findAllVectorIter(key, T.root());
    } //[m] findAllElements

    /**[m]
     * @see InspectableKeyBasedContainer
     */
    public ObjectIterator keys()  
    {
        Position curPos = T.root();
        VectorIter keys = new VectorIter();
        
        if (size()==0)
            // vuoto
            return keys;
        while(!T.isExternal(curPos))
            // cerca primo nodo a sinistra
            curPos = T.leftChild(curPos);
        curPos = T.parent(curPos);
        _lastKey = key(curPos);
        keys.insertLast(_lastKey);
        _recursive_keys(T.root(), keys);
        return keys;
    } //[m] keys
    
    /**[m]
     * @see Dictionary
     */
    public void insertItem(Object key, Object element)
      throws InvalidKeyException  
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        Position insPos = T.root();
          // posizione di inserimento
        // se la chiave e` gia` presente, l'elemento viene
        // inserito a destra di quello preesistente
        do {
            insPos = findPosition(key, insPos);
            if (T.isExternal(insPos))
                break;
            else
                insPos = T.rightChild(insPos);
        } while (true);
        T.expandExternal(insPos);
        Item newItem = new Item(key, element);
        T.replaceElement(insPos, newItem);
        actionPos = insPos;
    } //[m] insertItem

    /**[m]
     * @see Dictionary
     */
    public Object replaceElement(Object key, Object element) 
      throws InvalidKeyException  
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        Position pos = findPosition(key, T.root());
          // posizione di rimozione
        actionPos = pos;
        if(T.isExternal(pos)) 
            // chiave non trovata
            return NO_SUCH_KEY;
        Object toReturn;
        toReturn = element(pos); 
          // elemento da restituire
        Item newItem = new Item(key, element);
        T.replaceElement(pos, newItem);
        return toReturn;
    } //[m] replaceElement

    /**[m]
     * @see Dictionary
     */
    public Object removeElement(Object key) 
      throws InvalidKeyException  
    {
        Object toReturn;
        checkKey(key);
          // puo` sollevare InvalidKeyException
        Position remPos = findPosition(key, T.root());
          // posizione di rimozione
        if(T.isExternal(remPos)) 
        {
            // chiave non trovata
            actionPos = remPos;
            return NO_SUCH_KEY;
        }
        else 
        {
            toReturn = element(remPos); 
              // elemento da restituire
            if (T.isExternal(T.leftChild(remPos)))
                remPos = T.leftChild(remPos);
            else if (T.isExternal(T.rightChild(remPos)))
                remPos = T.rightChild(remPos);
            else 
            { 
                // il nodo ha entrambi i figli interni
                // cerca il nodo 'successore' con cui
                // scambiarsi
                Position swapPos = remPos;
                remPos = T.rightChild(swapPos);
                do
                    remPos = T.leftChild(remPos);
                while (T.isInternal(remPos));
                swap(swapPos, T.parent(remPos));
            } //else both internal
        actionPos = T.sibling(remPos);
        T.removeAboveExternal(remPos);
        return toReturn;
        }//else isExternal
    } //[m] removeElement

    /**[m]
     * @see Dictionary
     */
    public ObjectIterator removeAllElements(Object key) 
      throws InvalidKeyException  
    {
        VectorIter elems = new VectorIter();
        Object curElement;
        
        while((curElement = removeElement(key)) != NO_SUCH_KEY)
            elems.insertLast(curElement);
        
        return elems;
    } //[m] removeAllElements

    private Object _lastKey;
      // variabile interna
    
    /**[m]
     * costruzione ricorsiva di un enumeratore delle chiavi
     * con una visita in ordine interno
     * @param curPos  posizione di partenza
     * @param keys  enumeratore
     */
    private void _recursive_keys(Position curPos, 
      VectorIter keys)  
    {
        if(T.isExternal(curPos))
            // foglia, terminato
            return;
        
        _recursive_keys(T.leftChild(curPos), keys);
        
        if(!C.isEqualTo(key(curPos), _lastKey))  
        {
            keys.insertLast(key(curPos));
            _lastKey = key(curPos);
        } //if isEqualTo
        
        _recursive_keys(T.rightChild(curPos), keys);
    } //[m] _recursive_keys

    /**[m]
     * @see InspectableOrderedDictionary
     */
    public Object closestKeyBefore(Object key) 
      throws InvalidKeyException 
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        Position curPos = findPosition(key, T.root());
        actionPos = curPos;
        if (T.isInternal(curPos))
            // trovata la chiave
            return key;
        else
        {
            // non trovata la chiave
            // ricerca in su un ramo destro
            Position newPos = null;
            actionPos = T.root();
            while(!T.isRoot(curPos) && 
              curPos == 
              T.leftChild(newPos = T.parent(curPos)))
                curPos = newPos;
            if (T.isRoot(curPos))
                // key e` minore del minimo
                return NO_SUCH_KEY;
            actionPos = newPos;
            return key(newPos);
         } //else isInternal
    } //[m] closestKeyBefore

    /**[m]
     * @see InspectableOrderedDictionary
     */
    public Object closestElemBefore(Object key) 
      throws InvalidKeyException 
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        Position curPos = findPosition(key, T.root());
        actionPos = curPos;
        if (T.isInternal(curPos))
            // trovata la chiave
            return curPos.element();
        else
        {
            // non trovata la chiave
            // ricerca in su un ramo destro
            Position newPos = null;
            actionPos = T.root();
            while(!T.isRoot(curPos) && 
              curPos == 
              T.leftChild(newPos = T.parent(curPos)))
                curPos = newPos;
            if (T.isRoot(curPos))
                // key e` minore del minimo
                return NO_SUCH_KEY;
            actionPos = newPos;
            return element(newPos);
         } //else isInternal
    } //[m] closestElemBefore

    /**[m]
     * @see InspectableOrderedDictionary
     */
    public Object closestKeyAfter(Object key) 
      throws InvalidKeyException 
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        Position curPos = findPosition(key, T.root());
        actionPos = curPos;
        if (T.isInternal(curPos))
            // trovata la chiave
            return key;
        else
        {
            // non trovata la chiave
            // ricerca in su un ramo destro
            Position newPos = null;
            actionPos = T.root();
            while(!T.isRoot(curPos) && 
              curPos == 
              T.rightChild(newPos = T.parent(curPos)))
                curPos = newPos;
            if (T.isRoot(curPos))
                // key e` maggiore del massimo
                return NO_SUCH_KEY;
            actionPos = newPos;
            return key(newPos);
         } //else isInternal
    } //[m] closestKeyAfter

    /**[m]
     * @see InspectableOrderedDictionary
     */
    public Object closestElemAfter(Object key) 
      throws InvalidKeyException 
    {
        checkKey(key);
          // puo` sollevare InvalidKeyException
        Position curPos = findPosition(key, T.root());
        actionPos = curPos;
        if (T.isInternal(curPos))
            // trovata la chiave
            return curPos.element();
        else
        {
            // non trovata la chiave
            // ricerca in su un ramo sinistro
            Position newPos = null;
            actionPos = T.root();
            while(!T.isRoot(curPos) && 
              curPos == 
              T.rightChild(newPos = T.parent(curPos)))
                curPos = newPos;
            if (T.isRoot(curPos))
                // key e` maggiore del massimo
                return NO_SUCH_KEY;
            actionPos = newPos;
            return element(newPos);
         } //else isInternal
    } //[m] closestElemAfter

    /**[m]
     * scambio per rimozione
     * @param swapPos  posizione di scambio
     * @param remPos  posizione di rimozione
     */
    protected void swap(Position swapPos, Position remPos)
    {
        T.replaceElement(swapPos, remPos.element());
    } //[m] swap

    // variabili di supporto per il bilanciamento
    private LinkedBinaryTree newT;
      // nuovo albero bilanciato
    private PositionIterator pt;
      // iteratore di posizioni sull'albero di partenza

    /**[m] !!
     * verifica ed eventuale ribilanciamento -
     * se l'albero risulta sbilanciato
     * (height / log2 numElem > BALANCEFACTOR)
     * l'albero viene ricostruito in forma
     * bilanciata mantenendo lo stesso ordine
     * degli oggetti - puo` essere forzato
     * dal parametro
     * @param forced  se true, bilanciamento 
     *                forzato     
     * @return this se albero non sbilanciato;
     *         il nuovo albero se ribilanciato
     */
    public void checkBalance(boolean forced)
    {
        int n = T.size();
            // altezza, numero elementi
        if (!forced && (n<=1 || 
        ((double)BinTreeBasicAlgo.height2(T, T.root()) /
        (Math.log((double)n) / Math.log(2))
        ) <= BALANCEFACTOR))
            // vuoto o abbastanza bilanciato
            return;
        // ribilanciamento
        newT = new LinkedBinaryTree();
        pt = T.positions();
        balance(newT.root(), size());
          // la suddivisione avviene in base ai nodi nonnulli
        T.cut(T.root());
          // rimuove il vecchio albero
        T = newT;
    } //[m] checkBalance

    /**[m] !!
     * bilanciamento -
     * dimezza il numero totale di nodi e li assegna al
     * sottoalbero di sinistra del nuovo albero (newT), 
     * poi genera la radice e assegna il resto 
     * al sottoalbero di destra del nuovo albero
     * @param v  posizione di espansione nel nuovo albero:
     *           si assume che v sia compatibile con newT
     * @param nodes  numero nodi da bilanciare
     */
    private void balance(Position v, int nodes)
    {
        newT.expandExternal(v);
        if (nodes > 1)
        {
            int leftSize = nodes>>1;
              // se del caso, ne mette di piu` a sinistra
              // perche' i nodi ancora da sistemare sono
              // nodes-1. Ad esempio con nodes=4, a sinistra
              // ne mette 2 e a destra 1
            if (leftSize > 0)
                balance(newT.leftChild(v), leftSize);

            Position p;
            while (T.isExternal(p = 
              pt.nextPosition()));
              // cerca il successivo nodo non foglia
              // non e` necessario il controllo hasMoreElement
              // perche' e` noto passo passo il numero di nodi
            newT.replaceElement(v, p.element());
            int rightSize = nodes-1-leftSize;
            if (rightSize > 0)
                 balance(newT.rightChild(v), rightSize);
        } //if
        else
        {
            Position p;
            while (T.isExternal(p = 
              pt.nextPosition()));
              // cerca il successivo nodo non foglia
              // non e` necessario il controllo hasMoreElement
              // perche' e` noto passo passo il numero di nodi
            newT.replaceElement(v, p.element());
        } //else nodes
     } //[m] balance

    /**[m]
     * conversione a stringa -
     * produce il contenuto dell'albero
     * @return stringa contenuto del dizionario 
     */
    public String toString()
    {
        return "Dizionario con LinkedBinaryTree\r\n" +
            T.toString();
    } //[m] toString

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
"Dizionario: Battere codice 6 cifre + stringa (fine=Ctrl-Z):"
              );
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        BufferedReader re = new BufferedReader(new
          InputStreamReader(in));

        SimpleBinarySearchTree myDictio = 
          new SimpleBinarySearchTree(new AKeyComp());

        try
        {
            String str;
            while((str = re.readLine()) != null)
                myDictio.insertItem(str.substring(0,6), 
                  str.substring(6));
                  // la chiave sono i primi 6 caratteri
                  // il resto i dati
        }
        catch (IOException e)
        {
            // termina il ciclo
        } //catch(EOF)

        FI2Sys.out.println(myDictio);

        FI2Sys.out.println("numEl=" + myDictio.size());
        FI2Sys.out.println("I primi 3 elementi sono:");
        ObjectIterator it = myDictio.elements();

        int pos1 = 1;
        while (pos1++ <= 3 && it.hasNext())
            FI2Sys.out.println(
              (String)it.nextObject());

        Object tmp;
        FI2Sys.out.println("La stringa con chiave 123456 " + (
          (tmp = myDictio.findElement("123456BOH")) == 
           NO_SUCH_KEY ?
          "non e` presente" :
          ("e` presente e vale ->" + tmp + "<-")
          )
          );

        it = myDictio.findAllElements("123456BOH");
        FI2Sys.out.println(
          "Le stringhe con chiave 123456 sono:");
        while (it.hasNext())
            FI2Sys.out.println(
              (String)it.nextObject());
        ObjectIterator itk = myDictio.keys();
        FI2Sys.out.println("Dizionario completo:");
        VectorIter vi = new VectorIter();
        while (itk.hasNext())
        {
            Object key = itk.nextObject();
            it = myDictio.findAllElements(key);
            
            // la prima viene copiata
            if (it.hasNext())
            {
                Object val;
                vi.insertLast(
                  new Item(key, val = it.nextObject()));
                FI2Sys.out.println("key=" + key + 
                  " val=" + val);
            }
            while (it.hasNext())
                FI2Sys.out.println("key=" + key + 
                  " val=" + it.nextObject());
        } // while itk

        Object tmpkey[] = new
          Object[vi.size()];
        Object tmpval[] = new
          Object[vi.size()];
        int i = 0;
        while(vi.hasNext())
        {
            Item ie = (Item)vi.nextObject();
            tmpkey[i] = ie.key();
            tmpval[i++] = ie.element();
        }
              
        FI2Sys.out.println(
  "La maggiore chiave <= di 666666 e` " + 
          myDictio.closestKeyBefore("666666") +
          " di valore " +
          myDictio.closestElemBefore("666666"));
        FI2Sys.out.println(
  "La minor chiave >= di 666666 e` " + 
          myDictio.closestKeyAfter("666666") +
          " di valore " +
          myDictio.closestElemAfter("666666"));

        FI2Sys.out.println(
  "La maggiore chiave <= di 555555 e` " + 
          myDictio.closestKeyBefore("555555") +
          " di valore " +
          myDictio.closestElemBefore("555555"));
        FI2Sys.out.println(
  "La minor chiave >= di 555555 e` " + 
          myDictio.closestKeyAfter("555555") +
          " di valore " +
          myDictio.closestElemAfter("555555"));

        Dictionary sndDictio = 
          new SimpleBinarySearchTree(tmpkey, tmpval,
          new AKeyComp());
        sndDictio.removeElement("654321");
        sndDictio.replaceElement("123456", 
          "------- NUOVO -------");
        FI2Sys.out.println(
  "Il secondo senza la chiave 654321 e con 123456 aggiornato:"
          );
        FI2Sys.out.println(sndDictio);

        FI2Sys.out.println("numEl=" + sndDictio.size());
        FI2Sys.out.println("\nAltezza primo albero=" + 
          BinTreeBasicAlgo.height2(myDictio.T, 
          myDictio.T.root()));
        FI2Sys.out.println(
  "Il primo ribilanciato:");
        myDictio.checkBalance(true);
        FI2Sys.out.println(myDictio);
        FI2Sys.out.println("Dopo il bilanciamento numEl=" + 
          myDictio.size() +
          " altezza=" + 
          BinTreeBasicAlgo.height2(myDictio.T, 
            myDictio.T.root()));
    } //[m][s] main

} //{c} SimpleBinarySearchTree
