package FI2.Examples;
/**{p}
 * -- FI2.Examples --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Esempi vari
 */

/**{f}
 * DirList.java
 * Esercizio 8.2 -
 * Rappresentazione ad albero di un directory
 *
 * @version 1.00 99/11/28
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Tree.ExpandBinaryTree;
import FI2.Tree.LinkedBinaryTree;
import FI2.Tree.BinTreeBasicAlgo;
import FI2.Set.Position;
import FI2.Util.FI2Sys;

/**{c}
 * Utilizza la tecnica dell'albero figlio-fratello
 * per rappresentare un directory con i
 * suoi subdirectory -
 * un nodo directory ha un figlio a sinistra, un
 * nodo file no -
 * i nodi foglia effettivi non hanno figli e non
 * hanno dati associati (sono nodi sentinella)
 */
public class DirList 
{
    private ExpandBinaryTree dt;
      // albero del directory
    
    /**[c]
     * costruttore base
     * @param path  pathname del directory
     */
    public DirList(String path)
    {
        File dir = new File(path);
        if (! dir.isDirectory())
            throw new IllegalArgumentException(
              "File "+path+" non directory!");
        dt = new LinkedBinaryTree();
        build(dt.root(), path, dir);
    } //[c] DirList

    /**[m]
     * routine di costruzione ricorsiva
     * @param ron  nodo radice
     * @param rop  pathname radice
     * @param rof  file radice
     */
    private void build(Position ron, String rop, File rof)
    {
        String[] rootDirList = rof.list();
          // assunto directory
        Position curr = ron;  // nodo corrente
        dt.expandExternal(ron);  // espande radice
        dt.replaceElement(ron, rof);  // imposta radice
        for (int i=0; i<rootDirList.length; i++)
        {
            String fs = rootDirList[i];
            File ff = new File(rop, fs);
            if (i == 0)
              // il primo figlio
                curr = dt.leftChild(curr);
            else
                curr = dt.rightChild(curr);
            if (ff.isDirectory())
                build(curr, fs, ff);  // subdirectory
            else
            {
                dt.expandExternal(curr);  // espande figlio
                dt.replaceElement(curr, ff);  // imposta figlio
            }
        } // for
    } //[m] build

    /**[m]
     * visualizza contenuto directory
     * @param v  nodo directory
     */
    public void showDir(Position v)
    {
        if (dt.leftChild(v).element() != null)
        {
            // e' effettivamente un directory
            FI2Sys.out.println("Directory " + v.element());
            v = dt.leftChild(v);
            do
                FI2Sys.out.println(v.element() + 
                ((dt.leftChild(v).element() != null) ? " directory" : " file"));
            while ((v = dt.rightChild(v)).element() != null);
        }
        else
            FI2Sys.out.println("File " + v.element());
    } //[m] showDir

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  nome file (opzionale)
     */
    public static void main(String[] args)
    {
        DirList dl;

        if (args.length > 0)
        {
            dl = new DirList(args[0]);
            FI2Sys.out.println("\nVisita parentetica");
            BinTreeBasicAlgo.parentheticRepresentation(dl.dt,
              dl.dt.root());
                // notare che visualizza pathname relativi
            dl.showDir(dl.dt.root());
        } // if
    } //[m][s] main

} //{c} DirList

