package FI2.Examples;
/**{p}
 * -- FI2.Examples --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Esempi vari
 */

/**{f}
 * LisArrayStack.java
 * Esercizio 5.4 -
 * Estende ArrayStack per implementare Container e
 * impostare un attivatore su un oggetto di
 * classe ContainerHelper
 *
 * @version 2.00 00/09/29
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Set.ObjectIterator;
import FI2.Set.Container;
import FI2.Util.FI2Sys;
import FI2.Linear.ArrayStack;

/**{c}
 * Aggiunge ad ArrayStack l'inizializzazione di
 * un oggetto di classe ContainerHelper ove
 * viene registrato un oggetto che implementa
 * l'interfaccia ContainerActivator in forma
 * di classe anonima
 * @see ArrayStack
 * @see Container
 */
public class LisArrayStack extends ArrayStack
  implements Container
{
    private ContainerHelper hlp;
      // helper associato
    private Object vElem[];
    private int pos;
      // supporto elements()

    /**[c]
     * costruttore di default
     */
    public LisArrayStack()
    {
        this(ArrayStack.CAPACITY);
    } //[c] LisArrayStack

    /**[c]
     * costruttore base
     * @param cap  capacita` massima dell'array
     */
    public LisArrayStack(int cap)
    {
        super(cap);
          // alloca l'array di supporto
        // ora definisce l'oggetto helper:
        // questo sara' un oggetto di classe anonima
        // implicitamente associato a this 
        // (this e` di classe LisArrayStack)
        // e avra` accesso ai campi di quest'ultimo
        hlp = new ContainerHelper(new ContainerActivator()

        /**{c}
         * classe anonima: implementa per stack
         * un ContainerActivator
         * @see ContainerActivator
         */
        {
            /**[m]
             * rimuove tutti gli elementi
             * @return il medesimo oggetto stack
             */
            public Container setEmpty()
            { 
                // non ha accesso diretto ai campi
                // in ArrayStack
                FI2Sys.out.println("Metodo setEmpty");
                while (!isEmpty())
                    FI2Sys.out.println("Rimosso: " +
                      pop());
                return LisArrayStack.this;
                  // riferimento all'oggetto di classe 
                  // ArrayStack associato a this (che
                  // e` di classe anonima): si utilizza
                  // la speciale sintassi per riferirsi
                  // all'oggetto di classe includente
            } //[m] setEmpty

            /**[m]
             * rimuove gli elementi dallo stack originario
             * e li copia in uno nuovo di dimensione
             * doppia
             * @return un nuovo oggetto LisArrayStack
             *         con un array di supporto doppio
             */
            public Container setDouble()
            { 
                FI2Sys.out.println("Metodo setDouble");
                LisArrayStack newSt = new LisArrayStack(
                  capacity()<<1);
                  // array di dimensione doppia
                int sz = size();
                  // numero elementi attuale
                Object tmp[] = new Object[sz];
                for (int i=sz-1; i>=0; i--)
                    // rimuove e riordina gli elementi
                    tmp[i] = pop();
                for (int i=0; i<sz; i++)
                {
                    // copia elementi
                    FI2Sys.out.println("Copiato: " + tmp[i]);
                    newSt.push(tmp[i]);
                } // for
                return newSt;
                // non avendo accesso diretto ai campi di ArrayStack
                // occorre creare un nuovo stack con supporto doppio
                // e copiarvi gli elementi
            } //[m] setDouble

        } //{c} anonima ContainerActivator()
        );

    } //[c] LisArrayStack

    /**[m]
     * @see Container
     */
    public ObjectIterator elements()
    {
        int sz;

        vElem = new Object[sz = size()];
        for (int i=sz-1; i>=0; i--)
            // rimuove e riordina gli elementi
            vElem[i] = pop();
        for (int i=0; i<sz; i++)
            // rimette gli elementi
            push(vElem[i]);
        pos = 0;
        return new ObjectIterator()

        /**{c}
         * classe anonima: implementa ObjectIterator
         * @see ObjectIterator
         */
        {
            /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {  return pos < vElem.length;  }

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {  return vElem[pos++];  }

         } //{c} anonima ObjectIterator()
         ;
    } //[m] elements

    /**[m]
     * @see Container
     */
    public Container newContainer() 
//      throws InvalidContainerException
    {
        return new LisArrayStack(capacity());
    } //[m] newContainer

    /**[m]
     * lo helper associato
     * @return helper
     */
    public ContainerHelper getHelper()
    {  return hlp; }

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  nome file (opzionale)
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;
        String str;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "Stack espandibile con helper, battere una stringa per riga (? per svuotare)");
        }
        else
            in = new FileInputStream(args[0]);

        BufferedReader re = new BufferedReader(new
          InputStreamReader(in));
        LisArrayStack st = new LisArrayStack(2);
        ContainerHelper hlp = st.getHelper();
        while ((str = re.readLine()) != null)
        {
            if (str.length()==1 && str.charAt(0) == '?')
            {
                st = (LisArrayStack)hlp.activate(ContainerHelper.SETEMPTYCOM);
                hlp = st.getHelper();
                continue;
            }
            if (st.size() >= st.capacity())
            {
                st = (LisArrayStack)hlp.activate(ContainerHelper.SETDOUBLECOM);
                hlp = st.getHelper();
            }
            st.push(str);
        } // while
    } //[m][s] main

} //{c} LisArrayStack

