package FI2.Examples;
/**{p}
 * -- FI2.Examples --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Esempi vari
 */

/**{f}
 * NLMisure.java
 * Esercizio suppl. 1.9 -
 * una misura di lunghezza normalizzata
 *
 * @version 1.00 00/10/03
 * @author M.Moro DEI UPD
 */

/**{c}
 * Memorizza una misura di lunghezza con l'unita`
 * in forma normalizzata
 * @see Misure
 */
public class NLMisure extends Misure
{
    // tabella delle unita` ammesse
    private static final String units[] =
      { "mm", "cm", "dm", "m", "dam", "hm", "km" };
    private static final int METERIDX = 
    unit2Pos("m"); 

    
    // costruttori

    /**[c]
     * costruttore base
     * @param v  valore
     * @param u  unita`
     */
    public NLMisure(float v, String u)
    { 
        if (unit2Pos(u) == units.length)
            throw new IllegalArgumentException(
            "Nel costruttore: " + v + " " + u + " illegale");
        val = v;
        unit = u;
        norm();
    } //[c]

    /**[c]
     * costruttore di default
     */
    public NLMisure()
    { 
        val = 0;
        unit = "m";
    } //[c]

    // metodi di supporto

    /**[m]
     * calcola l'indice nell'array relativo all'unita` -
     * @return indice nell'array units, units.length se
     * non trovato
     */
    private static int unit2Pos(String uni) 
    { 
        int pos;
        for (pos=0; pos < units.length; pos++)
            if (uni.equals(units[pos]))
                return pos;
        return pos;
    } //[m]uniit2Pos
    
    // metodi esportati

    /**[m]
     * condizione di normalizzata
     * @return true se il valore e` normalizzato
     */
    public boolean isNorm() { return 
      (1<=Math.abs(val) && Math.abs(val)<10) ||
      (val == 0 && unit.equals(units[METERIDX])); }

    /**[m]
     * normalizzazione -
     * tenta la normalizzazione se possibile e se
     * non e` gia` normalizzata
     * @return true se il valore e` normalizzato
     */
    public boolean norm() 
    { 
        int pos = unit2Pos(unit);
        for (;;)
        {
            if (isNorm())
                // gia` normalizzato
                return true;
            if (Math.abs(val) < 1 && val != 0)
            {
                if (unit.equals(units[0]))
                    // l'unita` non si puo` ridurre
                    return false;
                // il valore si moltiplica 
                val *= 10;
                unit = units[--pos];
                continue;
            }
            
            if (Math.abs(val) >= 10)
            {
                if (unit.equals(units[units.length-1]))
                    // l'unita` non si puo` aumentare
                    return false;
                // il valore si divide
                val /= 10; 
                unit = units[++pos];
                continue;
            }
            
            if (val == 0)
            {
                // caso particolare
                unit = units[METERIDX];
                return true;
            }
        } // for
    }

    public void set(float v, String u)
    { 
        if (unit2Pos(u) == units.length)
            throw new IllegalArgumentException(
            "In set: " + v + " " + u + " illegale");
        val = v;
        unit = u;
        norm();
    } //[m] set
    
    /**[m]
     * somma -
     * ha senso ora consentire unita` diverse
     * purche' i parametri siano di classe piu'
     * specifica
     * @param val2  seconda misura da sommare
     * @return la somma tra this e val2
     */
    public Misure add(Misure val2)
    {
        int pos1 = unit2Pos(unit);
        if (pos1 == units.length)
            throw new IllegalArgumentException(
            "In add: " + val + " " + unit + " illegale");
        int pos2 = unit2Pos(val2.unit);
        if (pos2 == units.length)
            throw new IllegalArgumentException(
            "In add: " + val2.val + " " + val2.unit + " illegale");
        if (pos1 < pos2)
            // pos1 riportato a pos2
            while (pos1 < pos2)
            {
                val /= 10;
                pos1++;
                unit = units[pos1];
            }
        else if (pos1 > pos2)
            // pos2 riportato a pos1
            while (pos1 > pos2)
            {
                val2.val /= 10;
                pos2++;
                val2.unit = units[pos2];
            }
        // pos1 == pos2
        NLMisure sum = new NLMisure(val+val2.val, unit);
        sum.norm();
        return sum;
    }

    /**[m]
     * somma versione static -
     * ha senso ora consentire unita` diverse
     * purche' i parametri siano di classe piu'
     * specifica
     * @param val1  prima misura da sommare
     * @param val2  seconda misura da sommare
     * @return la somma tra val1 e val2
     */
    public static Misure add(Misure val1, Misure val2)
    {
        // si sfrutta l'altra
        return val1.add(val2);
    }

// non richiesto

    /**[m][s]
     * metodo di collaudo
     */
    public static void main(String[] args)
    {
        NLMisure m1 = new NLMisure (10, "cm");
        NLMisure m2 = new NLMisure (10, "dm");
        NLMisure m3 = new NLMisure (4, "km");
        NLMisure m4 = new NLMisure (1, "m");
        NLMisure m5 = new NLMisure (-40, "hm");
        NLMisure m6 = new NLMisure (400, "km");
          // grande
        
        System.out.println("m1 = " + m1);
        System.out.println("m2 = " + m2);
        System.out.println("m3 = " + m3);
        System.out.println("m4 = " + m4);
        System.out.println("m5 = " + m5);
        System.out.println("m6 = " + m6);
        System.out.println("m1+m3 = " + NLMisure.add(m1,m3));
        System.out.println("m1 " + 
          (m1.equals(m2) ? "e`" : "non e`") + " eguale a m2");
        System.out.println("m2 " + 
          (m2.equals(m4) ? "e`" : "non e`") + " eguale a m4");
        m4.opp();
        System.out.println("m4 reso opposto e` = " + m4);
        System.out.println("m3+m5 = " + m3.add(m5));
        System.out.println("m1+m6 = " + m1.add(m6));
    } //[m][s] main

} //{c} NLMisure

