package FI2.Examples;
/**{p}
 * -- FI2.Examples --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Esempi vari
 */

/**{f}
 * ParList.java
 * Esercizio 6.2 -
 * Lista con parentesi
 *
 * @version 1.00 99/11/05
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Util.FI2Sys;
import FI2.Linear.Stack;
import FI2.Linear.LinkedStack;
import java.util.NoSuchElementException;
import FI2.Exceptions.InvalidElementException;
import FI2.Set.ObjectIterator;

/**{c}
 * Nodo della lista con parentesi
 */
class PLNode
{
    Object el;
    PLNode subl, succ;  // sottolista, successore
    
    /**[c]
     * costruttore base
     * @param o  elemento associato
     * @param l  sottolista
     * @param s  successore
     */
    PLNode(Object o, PLNode l, PLNode s)
    {
        el = o;
        subl = l;
        succ = s;
    } //[c] PLNode

    /**[c]
     * costruttore di default
     */
    PLNode()
    {
        this(null, null, null);
    } //[c] PLNode

    /**[c]
     * costruttore ridotto
     * @param o  elemento associato
     */
    PLNode(Object o)
    {
        this(o, null, null);
    } //[c] PLNode

} //{c} PLNode

/**{c}
 * Lista con parentesi -
 * i nodi '(' puntano alla sottolista e al nodo
 * che segue la corrispondente ')'
 */
public class ParList
{
    private PLNode head = null;  // testa della lista
    private PLNode curr = null;  // nodo corrente
    private Stack levels;  // stack per le parentesi

    private static ObjectIterator sIt;  // iteratore per showLevel
    private static int shLevel;  // livello per showLevel

    /**[c]
     * costruttore di default
     */
    public ParList()
    {
        levels = new LinkedStack();
    } //[c] ParList

    /**[m]
     * Aggiunta di un elemento come stringa
     * @param el  elemento da aggiungere
     */
    public void addElement(String el)
    {
        newAtom(el);
    } //[m] addElement

    /**[m]
     * Aggiunta di un elemento come numero
     * @param el  numero da aggiungere
     */
    public void addElement(double el)
    {
        newAtom(new Double(el));
    } //[m] addElement

    /**[m]
     * Aggiunta di un elemento atomico
     * @param el  elemento da aggiungere
     */
    private void newAtom(Object el)
    {
        if (curr == null)
        {
            // lista vuota
            head = curr = new PLNode(el);
        }
        else if (curr.el instanceof Character)
            // dopo una parentesi
            if (((Character)curr.el).charValue() == '(')
            {
                // dopo un'aperta
                curr.subl = new PLNode(el);
                curr = curr.subl;
            } // if (
            else 
            {
                // dopo una chiusa
                PLNode tmp = curr.succ = new PLNode(el);
                curr = (PLNode)levels.pop();
                curr.succ = tmp;
                curr = tmp;
            } // else )
        else
        {
            // dopo nodo atomico
            curr.succ = new PLNode(el);
            curr = curr.succ;
        } // if null || Character
    } //[m] newAtom

    /**[m]
     * Aggiunta di una parentesi aperta
     */
    public void newList()
    {
        if (curr == null)
            // lista vuota
            head = curr = new PLNode(new Character('('));
        else if (curr.el instanceof Character)
            if (((Character)curr.el).charValue() == '(')
            {
                // dopo un'aperta
                curr.subl = new PLNode(new Character('('));
                curr = curr.subl;
            } // if (
            else 
            {
                // dopo una chiusa
                PLNode tmp = curr.succ = 
                  new PLNode(new Character('('));
                curr = (PLNode)levels.pop();
                curr.succ = tmp;
                curr = tmp;
            } // else )
        else
        {
            curr.succ = new PLNode(new Character('('));
            curr = curr.succ;
        }
        levels.push(curr);  // salva nodo parentesi
    } //[m] newList;

    /**[m]
     * Aggiunta di una parentesi chiusa
     * @exception InvalidElementException
     * se parentesi chiusa non bilanciata
     */
    public void endList()
    {
        if (levels.isEmpty())
            throw new InvalidElementException(
            "** Parentesi chiusa non bilanciata: non inserita");
        newAtom(new Character(')'));
    } //[m] newList;

    /**[m]
     * iteratore
     * @return un iteratore in ordine di inserimento
     */
    public ObjectIterator elements()
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per gli elementi
         */
        return new ObjectIterator()
        {
            PLNode currIter = head;  // riferimento da far scorrere

            /**[m]
             * @see FI2.Set.ObjectIterator
             */
            public boolean hasNext()
            {
                return currIter != null;
            } //[m] hasNext
            
            /**[m]
             * @see FI2.Set.ObjectIterator
             */
            public Object nextObject()
            {
                if (currIter == null)
                    throw new NoSuchElementException(
"In ParList.ObjectIterator.nextObject: non ci sono piu` elementi");
                Object tmp = currIter.el;
                if ((tmp instanceof Character) && 
                  (((Character)tmp).charValue() == '('))
                    currIter = currIter.subl;
                else
                    currIter = currIter.succ;
                return tmp;
            } //[m] nextElement
        } //{c} anonima ObjectIterator
        ;
    } //[m] elements

    /**[m]
     * verifica bilanciamento
     */
    public void balanceVerify()
    {
        if (levels.isEmpty() ||
          ( levels.size() == 1 && 
            curr.el instanceof Character &&
            ((Character)curr.el).charValue() == ')'))
            // dopo una parentesi chiusa
            return;
        FI2Sys.out.println("** Parentesi aperta/e non bilanciata/e");
    }

    /**[m]
     * Copia del primo elemento
     * @return clone del primo elemento (o sottolista)
     */
    public ParList first()
    {
        ObjectIterator it = elements();
        ParList ret = new ParList();

        if (!it.hasNext())
            // lista vuota
            return ret;
        Object obj = it.nextObject();
        if (obj instanceof Character)
        {
            // parentesi aperta
            int level = 1;
            while (it.hasNext())
            {
                obj = it.nextObject();
                if (obj instanceof Character)
                    if (((Character)obj).charValue() == ')')
                        if (level == 1)
                            // chiusa la sottolista
                            break;
                        else
                        {
                            level--;
                            ret.endList();
                        }
                    else
                    {
                        // sottolista
                        level++;
                        ret.newList();
                    }
                else 
                    ret.newAtom(obj);
            } // while
        } 
        else
            // primo elemento semplice
            ret.newAtom(obj);
        return ret;
    } //[m] first

    /**[m]
     * Copia di tutto fuorche' il primo elemento
     * @return clone del resto dopo il primo elemento
     */
    public ParList butFirst()
    {
        ObjectIterator it = elements();
        ParList ret = new ParList();

        if (!it.hasNext())
            // lista vuota
            return ret;
        Object obj = it.nextObject();
        if (obj instanceof Character)
        {
            // parentesi aperta
            int level = 1;
            while (it.hasNext())
            {
                obj = it.nextObject();
                if (obj instanceof Character)
                    if (((Character)obj).charValue() == ')')
                        if (level == 1)
                            // chiusa la sottolista
                            break;
                        else
                            level--;
                    else
                        // sottolista
                        level++;
            } // while
        } 
 
        while (it.hasNext())
        {
            obj = it.nextObject();
            if (obj instanceof Character)
                if (((Character)obj).charValue() == ')')
                    ret.endList();
                else
                    ret.newList();
            else 
                ret.newAtom(obj);
        } // while
        return ret;
    } //[m] butFirst

    /**[m][s]
     * visualizza un livello di parentesi
     */
    public static void showLevel()
    {
        if (shLevel > 0)
            FI2Sys.out.println(" ");
        for (int i=0; i<shLevel; i++)
            System.out.print("----");
        while (sIt.hasNext())
        {
            Object obj = sIt.nextObject();

            if (obj instanceof Character)  // parentesi
                if (((Character)obj).charValue() == '(')
                {
                    shLevel++;
                    showLevel();
                    shLevel--;
                    FI2Sys.out.println(" ");
                    for (int i=0; i<shLevel; i++)
                        System.out.print("----");
                }
                else  // chiusa parentesi
                    return;
            else
                System.out.print(obj.toString()+" ");
        } //while
        FI2Sys.out.println(" ");
    } //[m] showLevel

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  nome file (opzionale)
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;
        ParList pl = new ParList();

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
              "Lista con parentesi (ex=termina)");
        }
        else
            in = new FileInputStream(args[0]);

        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        sIn.parseNumbers();

        while(sIn.nextToken() != StreamTokenizer.TT_EOF)
        {
            try
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                    pl.addElement(sIn.nval);
                else if(sIn.ttype == StreamTokenizer.TT_WORD)
                {
                    if (sIn.sval.equals("ex")) break;
                    pl.addElement(sIn.sval);
                }
                else // singolo carattere
                if (((char)sIn.ttype) == '(')
                    pl.newList();
                else // )
                    pl.endList();
            } //try
            catch (InvalidElementException e)
            {
                FI2Sys.out.println(e);
                // ignorato
            } //catch(EOF)
        } //while
        pl.balanceVerify();
        FI2Sys.out.println("Lista costruita");
        ObjectIterator it = pl.elements();
        while (it.hasNext())
            FI2Sys.out.println(it.nextObject());
        sIt = pl.elements();
        shLevel=0;
        showLevel();

        FI2Sys.out.println("Primo elemento (o sottolista)");
        ParList fst = pl.first();
        sIt = fst.elements();
        shLevel=0;
        showLevel();

        FI2Sys.out.println("Il resto dopo il primo elemento (o sottolista)");
        ParList bfst = pl.butFirst();
        sIt = bfst.elements();
        shLevel=0;
        showLevel();
    } //[m][s] main

} //{c} ParList

