package FI2.Examples;
/**{p}
 * -- FI2.Examples --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Esempi
 */

/**{f}
 * Rect.java
 *
 * realizzazione di una classe che rappresenta un rettangolo
 * con i lati paralleli agli assi
 *
 * @version 2.00 98/09/24
 * @author M.Moro DEI UPD
 */

import FI2.Util.SimpleComparable;

/**{c}
 * Rettangolo sul piano: e` rappresentato dalla coppia di 
 * vertici in basso a sinistra, in alto a destra -
 * implementa sia SimpleComparable (equals) che Cloneable 
 * (clone) per poter definire copie-cloni
 * di un'istanza (esempio di 'ereditarieta` multipla')
 */
public class Rect implements SimpleComparable, Cloneable
{
    protected PointE[] vx;
        // coppia di vertici

    /**[c]
     * costruttore base
     * @param xVal1  ascissa angolo sinistra in basso
     * @param yVal1  ordinata angolo sinistra in basso
     * @param xVal2  ascissa angolo destra in alto
     * @param yVal2  ordinata angolo destra in alto
     */
    public Rect(double xVal1, double yVal1, 
      double xVal2, double yVal2)
    {
        vx = new PointE[2];
        vx[0] = new PointE(xVal1, yVal1);
        vx[1] = new PointE(xVal2, yVal2);
    } //[c] Rect

    /**[m]
     * un vertice come PointE
     * @param index  indice del vertice -
     * 0..3 dal basso a sinistra in senso antiorario
     * @return il vertice richiesto -
     *         null se Index illegale
     */
    public PointE vertex(int index)
    {
        switch(index) {
          case 0:  // in basso a sinistra
            return new PointE(vx[0].xVal(), vx[0].yVal());
          case 1:  // in basso a destra
            return new PointE(vx[1].xVal(), vx[0].yVal());
          case 2:  // in alto a destra
            return new PointE(vx[1].xVal(), vx[1].yVal());
          case 3:  // in alto a sinistra
            return new PointE(vx[0].xVal(), vx[1].yVal());
          default:  // indice illegale
            return null;           
        } // switch(index)
    } //[m] vertex

    /**[m]
     * perimetro
     * @return somma delle misure dei lati
     */
    public double perim()
    {
        return 2 * (vx[1].xVal()-vx[0].xVal()+
          vx[1].yVal()-vx[0].yVal());
    } //[m] perim

    /**[m]
     * diagonale
     * @return distanza tra i due vertici
     */
    public double diag()
    {
        return vx[0].dist(vx[1]);
    } //[m] diag

    /**[m]
     * area
     * @return prodotto dei lati
     */
    public double surf()
    {
        return (vx[1].xVal()-vx[0].xVal()) * 
          (vx[1].yVal()-vx[0].yVal());
    } //[m] surf

    /**[m]
     * rettangolo traslato
     * @param newVx  nuovo vertice in basso a sinistra
     */
    public void trasl(PointE newVx)
    {
        PointE tmp0 = vx[0];
        vx[0] = new PointE(newVx.xVal(), newVx.yVal());
        vx[1] = new PointE(
          vx[1].xVal()+newVx.xVal()-tmp0.xVal(),
          vx[1].yVal()+newVx.yVal()-tmp0.yVal());
    } //[m] trasl

    /**[m]
     * ridefinisce il metodo Object.equals()
     * @param sndR  secondo rettangolo -
     *              si assume che sia in realta` 
     *              almeno un Rect
     * @return true se rappresentano gli stessi vertici
     * @see SimpleComparable
     */
    public boolean equals(Object sndR)
    {
        return vx[0].equals(((Rect)sndR).vertex(0)) &&
          vx[1].equals(((Rect)sndR).vertex(2));
    } //[m] equals

    /**[m]
     * conversione a stringa
     * @return descrizione del rettangolo
     */
    public String toString()
    {
        String str = "{Rect} vertici:";

        for (int i=0; i<4; i++) 
            str += " --" + i + "-- " + vertex(i);
        return str;
    } //[m] toString

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  vx0.x 
     * @param args[1]  vx0.y
     * @param args[2]  vx1.x
     * @param args[3]  vx1.y
     */
    public static void main(String[] args)
    {
        Rect r1;

        if (args.length >= 4)
            // ci sono i parametri di chiamata
            r1 = new
              Rect(
              Double.valueOf(args[0]).doubleValue(),
              Double.valueOf(args[1]).doubleValue(),
              Double.valueOf(args[2]).doubleValue(),
              Double.valueOf(args[3]).doubleValue());
        else
            r1 = new Rect(0.0, 0.0, 4.0, 3.0);

        System.out.println("Il rettangolo r1 e` " + r1);
        System.out.println("Perimetro=" + r1.perim());
        System.out.println("Diagonale=" + r1.diag());
        System.out.println("Area=" + r1.surf());

        System.out.println("r1 " + 
          (r1.equals(new Rect(0.0, 0.0, 1.0, 1.0)) ? 
          "coincide" : "non coincide") +
          " con il rettangolo ((0.0, 0.0) , (1.0,1.0))");

    Rect r2;
        try 
        {
            r2 = (Rect)r1.clone();
        }
        catch(CloneNotSupportedException Ex) 
        {
            System.out.println("Eccezione clone: " + Ex);
            return;
        }
        System.out.println("Il clone e` " + r2);

        r1.trasl(new PointE(-3.0, 2.0));
        System.out.println("Traslato in (-3.0, 2.0) e` " + 
          r1);
        System.out.println("Il clone e` ora " + r2);
            /* la clonazione ha solo copiato i riferimenti ai
            PointE che rappresentano i vertici, sicche' 
            variazioni apportate via r1 su tali oggetti 
            vengono lette anche via r2. Per avere un clone
            completo bisognerebbe ridefinire clone() in modo
            da clonare anche i PointE */
    } //[m][s] main

} //{c} Rect

