package FI2.Examples;
/**{p}
 * -- FI2.Examples --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Esempi vari
 */

/**{f}
 * Rombo.java
 * Esercizio 3.4 -
 * Rombo su piano cartesiano
 *
 * @version 2.00 00/09/22
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Set.ObjectIterator;

/**{c}
 * Realizzazione di un rombo su piano cartesiano -
 * le coordinate sono memorizzate come array di
 * 4 Point
 */
public class Rombo
{
    private Point vx[];  // vertici

    /**[c]
     * costruttore di default - 
     * il quadrato (0,0)(1,0)(1,1)(0,1)
     */
    public Rombo()
    {
        vx = new Point[4];
        vx[0] = new Point(0.0, 0.0);
        vx[1] = new Point(1.0, 0.0);
        vx[2] = new Point(1.0, 1.0);
        vx[3] = new Point(0.0, 0.1);
    } //[c] Rombo

    /**[c]
     * costruttore con 4 vertici
     * @param va  vertice 0
     * @param vb  vertice 1
     * @param vc  vertice 2
     * @param vd  vertice 3
     */
    public Rombo(Point va, Point vb, Point vc, Point vd)
    {
        vx = new Point[4];
        vx[0] = va;
        vx[1] = vb;
        vx[2] = vc;
        vx[3] = vd;
    } //[c] Rombo

    /**[c]
     * costruttore da file
     * @param fn  file pathname
     */
    public Rombo(String fn)
      throws IOException
    {
        vx = new Point[4];
        PointInputStream fin = new PointInputStream(
          new FileInputStream(fn));
        vx[0] = fin.readPoint();
        vx[1] = fin.readPoint();
        vx[2] = fin.readPoint();
        vx[3] = fin.readPoint();
    } //[c] Rombo

    /**[m]
     * conversione a stringa
     * @return descrizione del contenuto
     */
    public String toString()
    {
        return "{Rombo} vertici A=" + vx[0] + 
          " B=" + vx[1] + " C=" + vx[2] + " D=" + vx[3];
        // si possono dare descrizioni anche piu` articolate
    }

    /**[m]
     * Iteratore
     * @see ObjectIterator
     * @return un oggetto di iterazione
     */
    public ObjectIterator vertici()
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per gli elementi -
         * assunto l'oggetto rombo immutabile,
         * non e` necessario eseguire una copia dell'array,
         * basta un indice di scorrimento sull'array dell'oggetto -
         * bisognerebbe pero` restituire una copia 
         * dei singoli vertici per evitare
         * la possibile modifica degli oggetti Point, restituiti
         * per riferimento, da parte dei
         * metodi chiamanti dall'esterno della classe - 
         * per far questo un oggetto Point dovrebbe essere
         * clonabile cioe` implementare l'interfaccia
         * Cloneable (cosa che l'attuale versione di Point 
         * non fa)
         */
        class RomboIter implements ObjectIterator
        {
            private int ix = 0;
              // indice di iterazione

            // costruttore di default predefinito

            /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return ix<4;
            } //[m] hasNext
            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                return vx[ix++] /* .clone() */;
            } //[m] nextObject

        } //{c} RomboIter

        return new RomboIter();
    } //[m] vertici

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  nome file (opzionale)
     */
    public static void main(String[] args)
      throws IOException
    {
        Rombo r;

        if (args.length >= 1)
            // lettura da file
            r = new Rombo(args[0]);
        else
            r = new Rombo (
              new Point(0.0, 1.0),
              new Point(2.0, 3.0),
              new Point(4.0, 5.0),
              new Point(6.0, 7.0));
        System.out.println("Il rombo creato :" + r);
        System.out.println("ovvero:");
        ObjectIterator it = r.vertici();
        int i = 1;
        while (it.hasNext())
            System.out.println("Vertice " + (i++) + ":" + 
              it.nextObject());
    } //[m][s] main

} //{c} Rombo

