package FI2.Graph;
/**{p}
 * -- FI2.Graph --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Grafi
 */

/**{f}
 * SILEdge.java
 * arco per SILGraph -
 *
 * @version 2.00 00/09/28
 * @author M.Moro DEI UPD
 * @see SILGraph
 */

/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

// package jdsl.core.ref;

import FI2.Set.Position;
import FI2.Set.Container;
import FI2.Exceptions.InvalidPositionException;
import FI2.Exceptions.InvalidEdgeException;


/**
    * An edge in a <code>SILGraph.</code>
    *
 * @author Benoit Hudson (bh)
 * @author Mike Boilen (mgb)
    * @version JSDL-Teach release 1.0
    */
public class SILEdge extends SILPosition implements Edge  {
    private Vertex v1_, v2_ ; 
      // if directed, v1 is origin, v2 dest
    private Position p1_, p2_, pgraph_; 
      // positions within inner containers
    private boolean directed_; // true if the edge is directed

    /** 
     * Construct a new edge for use in 
     * a SimpleIncidenceListGraph.
     * @param cont  The graph
     * @param elt   The element associated with this position
     * @param v1    The first endpoint 
     *        (the origin if directed)
     * @param v2    The second endpoint 
     *        (the destination if directed)
     * @param directed  If true, the edge is directed.
     */
    public SILEdge(Container cont, Object elt,
            Vertex v1, Vertex v2,
            boolean directed) {
        super(cont, elt);
        v1_ = v1 ; 
        v2_ = v2 ;
        p1_ = p2_ = null ;
        directed_ = directed;
    }

    /** 
     * Set the position within that endpoint's edge list, 
     * for O(1)-access
     * to this information.<p>
     * 
     * Note that if we are a self-loop, we will have 
     * two positions for the vertex.  
     * To set them both, we have to make a special case.
     * 
     * @param v  The endpoint
     * @param p  The position containing this edge within that
     *  endpoint's edge list
     * @exception RuntimeException 
     * indicates a bug in the Graph implementation
     */
    void setPosition(Vertex v, Position p) {
        if(isSelfLoop()) {
            if(p1_ == null) p1_ = p ;
            else p2_ = p;
        }
        else if(v==v1_) p1_ = p ;
        else if(v==v2_) p2_ = p ;
        else throw new 
          RuntimeException(
          "setPosition passed a non-endpoint vertex");
    }

    /** 
     * Remove the first reference we have to the vertex 
     * (not all references, in the case of a self-loop).
     *
     * @param v  The vertex to lose track of.
     */
    void removePosition(Vertex v) {
        if(v==v1_) {
            v1_ = null ;
            p1_ = null ;
        }
        else if(v==v2_) {
            v1_ = null ;
            p1_ = null ;
        }
    }


    /** 
     * Set the position of the edge within the graph's 
     * edgeset, for O(1)-access to that info.
     * 
     * @param g  The graph which contains this edge
     * @param p  The position within the graph's edgeset
     */
    void setPosition(Graph g, Position p) {
        pgraph_ = p ;
    }

    /** 
     * Get the position of the edge within the vertex's 
     * edgeset.  
     * This should always succeed; 
     * a RuntimeException will be thrown if it
     * fails, which would indicate a bug in the graph.
     * 
     * @param v   The vertex that wants our position.
     * @exception RuntimeException 
     * indicates a bug in the Graph implementation
     * @return    The position within that vertex's edge list.
     */
    Position getPosition(Vertex v) {
        if(isSelfLoop()) {
            if(p1_ != null) return p1_;
            else return p2_;
        }
        else if(v==v1_) return p1_ ;
        else if(v==v2_) return p2_;
        else throw new RuntimeException(
          "setPosition passed a non-endpoint vertex");
    }


    /** 
     * Get the position of this edge within that graph's 
     * edge list.
     * @param g   The graph containing this edge.
     * @exception IPE if the graph isn't our container.
     * @return    The position we have within the graph.
     */
    Position getPosition(Graph g) {
        if(g!=container()) throw new InvalidPositionException(
  "getPosition passed a graph not containing this edge");
        return pgraph_;
    }


    /** 
     * Returns true if the edge is directed.  
     * If so, the origin is v1_.
     * @return True if the edge is directed
     */
    boolean isDirected() {
        return directed_;
    }

    /** 
     * Makes this edge directed, with v1_ as its origin.
     * @param directed  True if the edge will now be directed
     */
    void setDirected(boolean directed) {
        directed_ = directed ;
    }

    /** 
     * Returns true if this edge is a self-loop 
     * (the endpoints are the same point).
     * @return  true if the edge is a self-loop
     */
    boolean isSelfLoop() {
        return v1_==v2_;
    }

    /** 
     * Returns the endpoints of the edge, 
     * in no guaranteed order.
     * @return the endpoints of the edge
     */
    Vertex[] endpoints() {
        Vertex[] retval = new Vertex[2];
        retval[0] = v1_;
        retval[1] = v2_;
        return retval;
    }

    /** 
     * Returns the vertex opposite the vertex 
     * passed in along this edge (i.e. the other vertex).
     * 
     * @param v The vertex we know.
     * @exception RuntimeException 
     * if v is not an end point (indicates
     * a bug in the graph implementation)
     * @return  The vertex opposite v along this edge
     */
    Vertex opposite(Vertex v) {
        if(v==v1_) return v2_ ;
        else if(v==v2_) return v1_;
        else throw new RuntimeException("v not an endpoint");
    }

    /** 
     * Returns the origin of the directed edge.
     * 
     * @exception  IEE if the edge is undirected.
     * @return     the origin of the directed edge.
     */
    Vertex origin() {
        if(!isDirected()) throw new 
          InvalidEdgeException("undirected edge");
        return v1_;
    }

    /** 
     * Returns the destination of the directed edge.
     * 
     * @exception  IEE if the edge is undirected.
     * @return     the destination of the directed edge.
     */
    Vertex destination() {
        if(!isDirected()) throw 
          new InvalidEdgeException("undirected edge");
        return v2_;
    }


    /** 
     * Set the endpoints of the edge; 
     * the position of the edge within its
     * endpoints' edge lists becomes undefined.
     * @param v1  The origin if this edge is directed
     * @param v2  The destination if this edge is directed
     */
    void setEndpoints(Vertex v1, Vertex v2) {
        v1_ = v1; v2_ = v2 ;
        p1_=p2_=null;
    }

    /** 
     * Sets the edge to be directed, 
     * with is origin being v.<p>
     * If its origin is already v, 
     * just set the directed_ flag;
     * otherwise, we also need to swap the points.
     * 
     * @exception RuntimeException 
     * if v is not an endpoint of this
     * edge, indicating bug within the graph implementation
     * 
     * @param v  The vertex which is to be the new origin
     */
    void setDirectionFrom(Vertex v) {
        setDirected(true);
        if(v==v1_) return;
        if(v!=v2_) throw new 
          RuntimeException("v not an endpoint");
        swapPoints();
    }

    /** 
     * Sets the edge to be directed, 
     * with is destination being v.<p>
     * If its destination is already v, 
     * just set the directed_ flag;
     * otherwise, we also need to swap the points.
     * 
     * @exception RuntimeException 
     * if v is not an endpoint of this
     * edge, indicating bug within the graph implementation
     * 
     * @param v  The vertex which is to be the new destination
     */
    void setDirectionTo(Vertex v) {
        setDirected(true);
        if(v==v2_) return;
        if(v!=v1_) throw new 
          RuntimeException("v not an endpoint");
        swapPoints();
    }

    /** 
     * Swap the two endpoints and their associated positions.  
     * In an undirected edge, this has no effet; 
     * in a directed edge, this
     * reverses the direction of the edge.
     */
    void swapPoints() {
        Vertex vtmp = v1_ ;
        v1_ = v2_ ;
        v2_ = vtmp;

        Position ptmp = p1_ ;
        p1_ = p2_ ;
        p2_ = ptmp;
    }

} //{c} SILEdge
