package FI2.Graph;
/**{p}
 * -- FI2.Graph --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Grafi
 */

/**{f}
 * SILPosition.java
 * posizionatore per SILGraph -
 * software ricavato dalla libreria JDSL
 *
 * @version 2.00 00/09/28
 * @author M.Moro DEI UPD
 * @see SILGraph
 */

/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

// package jdsl.core.ref;

import FI2.Set.Position;
import FI2.Set.Container;
import FI2.Exceptions.InvalidPositionException;

/**
    * Position in the <code>SILGraph</code>
    *
 * @author Benoit Hudson (bh)
 * @author Mike Boilen (mgb)
    * @version JSDL-Teach release 1.0
    */
abstract class SILPosition implements Position {
    private Container cont_ ; // if null, position is invalid
    private Object elt_ ;

    /** 
     * Create a new ODPosition, 
     * with as its container cont, and its
     * locator loc.<p>
     *
     * @param cont  The container for the position
     * @param elt   The element to hold at this position
     */
    SILPosition(Container cont, Object elt) {
        cont_ = cont ;
        elt_ = elt ;
    }

    /** 
     * Returns the string representing the element<p>
     * Implemented as "" + element(), which means to call
     * element().toString() unless element() is null, 
     * in which case "null" is returned.
     * @return the string representing the element
     */
    public String toString() {
        return "" + element();
    }

    /** 
     * Returns the element at this position<p>
     * 
     * @exception  InvalidPositionException 
     * if the position has been
     *             invalidated 
     * @return the element at this position
     */
    public Object element() {
        checkValid();
        return elt_ ;
    }

    /** 
     * Returns the container of this position<p>
     * 
     * @exception InvalidPositionException 
     * if the position has been
     *             invalidated 
     * @return the container of this position
     */
    public Container container() { 
        checkValid(); 
        return cont_ ; 
    }


    /**
     * Set the element.
     */
    void setElement(Object elt) {
        elt_ = elt ;
    }

    /** 
     * Invalidate the position; 
     * any subsequent calls will throw an IPE<p>
     */
    void invalidate() {
        cont_ = null;
    }

    /** 
     * Check to see if the position has been invalidated<p>
     * 
     * @exception  InvalidPositionException 
     * if the position has been
     *             invalidated 
     */
    void checkValid() throws InvalidPositionException {
        if(cont_ == null)
            throw new InvalidPositionException(
              "position has been invalidated");
    }
} //{c}{a} SilPosition
