package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * ArrayQueue.java
 * Implementazione con array per una coda FIFO
 *
 * @version 1.00 98/09/19
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Exceptions.QueueEmptyException;
import FI2.Exceptions.QueueFullException;
import FI2.Util.FI2Sys;

/**{c}
 * coda FIFO generica realizzata con vettore
 * in forma circolare
 * @see Queue
 */
public class ArrayQueue implements Queue
{
    public static final int CAPACITY = 1000+1;  
      // dimensione massima di default
      // una casella sempre vuota
    private int capacity;  // capacita`
    private Object Q[];  //vettore di supporto
    private int f = 0;  // testa (front), prima occupata
    private int r = 0;  // coda (rear), prima libera
            // f==r vuol dire coda vuota
            // (r+1)%capacity == f vuol dire coda piena

    /**[c]
     * costruttore di default
     */
    public ArrayQueue()
    {
        this(CAPACITY);
    } //[c] ArrayQueue

    /**[c]
     * costruttore con capacita` massima
     * in numero di elementi
     * @param cap  capacita` massima dell'array
     */
    public ArrayQueue(int cap)
    {
        capacity = cap+1;
        Q = new Object[capacity];
    } //[c] ArrayQueue

    /**[c] !!
     * costruttore con una copia dei riferimenti contenuti
     * nell'array fornito -
     * mantiene l'ordine originario -
     * gli oggetti non vengono clonati - O(n)
     * @param val  array da copiare
     * @param cap  capacita` massima
     * @exception IndexOutOfBoundsException
     *            se cap < val.length
     */
    public ArrayQueue(Object val[], int cap)
    {
        this(cap);
        f = 0;
        r = val.length;
        for (int i=0; i<val.length; i++)
            Q[i] = val[i];
                // questa istruzione puo` sollevare
                // l'eccezione IndexOutOfBoundsException
    } //[c] ArrayQueue

    /**[m]
     * O(1)
     * @see Queue
     */
    public int size()
    {
        return (capacity-f+r) % capacity;
    } //[m] size

    /**[m]
     * O(1)
     * @see Queue
     */
    public boolean isEmpty()
    {
        return f==r;
    } //[m] isEmpty

    /**[m]
     * O(1)
     * @see Queue
     */
    public Object front()
      throws QueueEmptyException
    {
        if (isEmpty())
            throw new QueueEmptyException(
              "In ArrayQueue.front");
        return Q[f];
    } //[m] front

    /**[m]
     * O(1)
     * @see Queue
     */
    public void enqueue(Object obj)
    {
        if (size() == capacity-1)
            throw new QueueFullException(
              "In ArrayQueue.enqueue");
        Q[r] = obj;
        r = ++r % capacity;
    } //[m] enqueue

    /**[m]
     * O(1)
     * @see Queue
     */
    public Object dequeue()
      throws QueueEmptyException
    {
        if (isEmpty())
            throw new QueueEmptyException(
              "In ArrayQueue.dequeue");
        Object temp = Q[f];
        Q[f] = null;
        f = ++f % capacity;
        return temp;
    } //[m] dequeue

    /**[m] !!
     * capacita` corrente - O(1)
     * @return la capacita` del vettore in numero di elementi
     */
    public int capacity()
    {
        return capacity-1;
            // una casella e` sempre vuota
    } //[m] capacity

    /**[m] !!
     * conversione a stringa - O(n)
     * @return stringa con il contenuto
     */
    public String toString()
    {
        String str =
          "Coda su vettore [size=" + size() +
          "] (dalla testa):\n";

        int pos = f;
output:
        for (int j=0; ; j++)
        {
            for (int i=0; i<6; i++)
            {
                // 6 valori per riga
                if (pos == r)
                    // finita la coda
                    break output;
                str += (j*6+i) + ": " +
                  Q[pos].toString() + " | ";
                pos = ++pos % capacity;
            } //for(i)
            if (pos != r)
                str += "\n";
        } //for(j)
        return str;
    } //[m] toString

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "ArrayQueue: Battere almeno 6 interi (fine=Ctrl-Z):");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        ArrayQueue myQueue = new ArrayQueue(30);

        sIn.parseNumbers();
        try
        {
            Integer next;
            int pos = 0;

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sIn.nval);
                    myQueue.enqueue(next);
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while

        }
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch(EOF)
        catch (QueueFullException e)
        {
            FI2Sys.out.println("[" + e + "] troppi numeri!");
            // termina il ciclo
        } //catch(QueueFullException)
        // la lista ha ora i numeri forniti nell'ordine

        FI2Sys.out.println("Coda creata");
        FI2Sys.out.println(myQueue);

        FI2Sys.out.println("size=" + myQueue.size());

        myQueue.dequeue();
        myQueue.dequeue();
        FI2Sys.out.println("La coda dopo due estrazioni e`:");
        FI2Sys.out.println("size=" + myQueue.size());
        FI2Sys.out.println(myQueue);

        // prova della circolarita`
        for (int i=0; i<1000; i++)
        {
            myQueue.enqueue(new Integer(i));
            myQueue.dequeue();
        }
        FI2Sys.out.println("Dopo 1000 enq-deq la coda e`: "
          + myQueue);
    } //[m][s] main

} //{c} ArrayQueue
