package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * ArraySequence.java
 * Implementazione con vettore di un Sequence
 * (sequenza con accesso indicizzato e per posizione)
 *
 * @version 2.00 00/09/20
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Exceptions.BoundaryViolationException;
import FI2.Exceptions.InvalidPositionException;
import FI2.Util.FI2Sys;
import FI2.Set.Position;

/**{c}
 * sequenza generica con accesso sia con indice
 * che per posizione -
 * estende con accesso indicizzato l'implementazione
 * posizionale
 * @see Sequence
 * @see ArrayList
 */
public class ArraySequence 
  extends ArrayList implements Sequence
{
    /**[c]
     * costruttore di default
     */
    public ArraySequence()
    { super(); }

    /**[c] !!
     * costruttore con una copia dei riferimenti contenuti
     * nell'array fornito -
     * effettua un inserimento nell'ordine degli elementi -
     * gli oggetti non vengono clonati 
     * @param val  array da copiare
     * @param cap  capacita` massima
     * @exception IndexOutOfBoundsException
     *            se cap < val.length
     */
    public ArraySequence(Object val[])
    { super(val); }

    /**[m]
     * O(1)
     * @see InspectableVector
     */
    public Object elemAtRank(int r)
      throws BoundaryViolationException
    {
        if (r<0 || r>size()-1)
            throw new BoundaryViolationException(
              "In ArraySequence.elemAtRank: indice=" + r);
        return ((ArrayListPosition)S.elemAtRank(r))
          .getElement();
    } //[m] elemAtRank

    /**[m]
     * O(1)
     * @see Vector
     */
    public Object replaceAtRank(int r, Object e)
      throws BoundaryViolationException
    {
        if (r<0 || r>size()-1)
            throw new BoundaryViolationException(
              "In ArraySequence.replaceAtRank: indice=" 
              + r);
        Object obj = 
          ((ArrayListPosition)S.elemAtRank(r)).getElement();
        ((ArrayListPosition)S.elemAtRank(r)).setElement(e);
        return obj;
    } //[m] replaceAtRank

    /**[m]
     * O(n)
     * @see Vector
     */
    public void insertAtRank(int r, Object e)
      throws BoundaryViolationException
    {
        Position pos = new ArrayListPosition(this, e, r);
        S.insertAtRank(r, pos);
        // size gia` aumentato
        // sistema i posizionatori spostati
        for (int i=r+1; i < size(); i++)
            ((ArrayListPosition)S.elemAtRank(i)).setIndex(i);
    } //[m] insertAtRank

    /**[m]
     * O(n)
     * @see Vector
     */
    public Object removeAtRank(int r)
      throws BoundaryViolationException
    {
        ArrayListPosition pos = 
          (ArrayListPosition)S.removeAtRank(r);
        // size e` stato decrementato
        for (int i=r; i < size(); i++)
            ((ArrayListPosition)S.elemAtRank(i)).setIndex(i);
        Object elem = pos.getElement();
        pos.setInvalid();
          // invalida il posizionatore
        return elem;
    } //[m] removeAtRank
    
    /**[m]
     * O(1)
     * @see InspectableSequence
     */
    public Position atRank(int r) 
      throws BoundaryViolationException
    {
        if (r<0 || r>size())
            throw new BoundaryViolationException(
              "In ArrayRankedSequence.atRank: indice=" + r);
        return (Position)S.elemAtRank(r);
    } //[m] atRank

    /**[m]
     * O(1)
     * @see InspectableSequence
     */
    public int rankOf(Position p) 
      throws InvalidPositionException
    {
        ArrayListPosition here = checkPosition(p);
        return here.index();
    } //[m] rankOf

    /**[m] !!
     * conversione a stringa
     * @return stringa con il contenuto
     */
    public String toString()
    {
        return "Sequence su vettore:\n" + super.toString();
    } //[m] toString


    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "ArraySequence: Battere almeno 6 interi (fine=Ctrl-Z)");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        ArraySequence mySeq = new ArraySequence();

        sIn.parseNumbers();
        try
        {
            Integer next;
            int pos = 0;

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sIn.nval);
                    mySeq.insertAtRank(pos++, next);
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while

        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch(EOF)
        catch (BoundaryViolationException e)
        {
            FI2Sys.out.println("[" + e + "] troppi numeri!");
            // termina il ciclo
        } //catch(BoundaryViolationException)

        FI2Sys.out.println("Sequence creato");
        FI2Sys.out.println(mySeq);

        FI2Sys.out.println("size=" + mySeq.size());
        FI2Sys.out.println("elemento all'indice 4=" +
          ((Integer)mySeq.elemAtRank(4)).intValue());
        Object rem = mySeq.removeAtRank(5);
        FI2Sys.out.println("elemento rimosso in 5:" + rem);
        rem = mySeq.replaceAtRank(3, new Integer(66));
        FI2Sys.out.println("elemento sostituito in 3:" + rem);
        FI2Sys.out.println(
          "La sequenza dopo remove in 5 e replace in 3 e`: " +
          mySeq);

        FI2Sys.out.println("primo elemento =" + 
          mySeq.first().element());
        Position fst = mySeq.first();
        FI2Sys.out.println("secondo elemento =" + 
          mySeq.after(fst).element());
        Position snd = mySeq.after(fst);
        FI2Sys.out.println("terzo elemento =" + 
          mySeq.after(snd).element());
        FI2Sys.out.println("ultimo elemento =" + 
          mySeq.last().element());
        Position thr = mySeq.after(snd);
    } //[m][s] main

} //{c} ArraySequence
