package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * ArrayStack.java
 * Implementazione con vettore di uno stack
 *
 * @version 1.00 98/09/19
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Exceptions.StackEmptyException;
import FI2.Exceptions.StackFullException;
import FI2.Util.FI2Sys;

/**{c}
 * stack generico realizzato come vettore a dimensione fissa
 * @see Stack
 */
public class ArrayStack implements Stack
{
    public static final int CAPACITY = 1000;  
      // dimensione massima di default
    private int capacity;  // capacita`
    private Object S[];  // vettore di supporto
    private int top = -1;  // posizione della cima

    /**[c]
     * costruttore di default
     */
    public ArrayStack()
    {
        this(CAPACITY);
    } //[c] ArrayStack

    /**[c]
     * costruttore con capacita` massima
     * @param cap  capacita` massima dell'array
     */
    public ArrayStack(int cap)
    {
        capacity = cap;
        S = new Object[capacity];
    } //[c] ArrayStack

    /**[c] !!
     * costruttore con una copia dei riferimenti contenuti
     * nell'array fornito -
     * effettua un push nell'ordine degli elementi -
     * gli oggetti non vengono clonati - O(n)
     * @param val  array da copiare
     * @param cap  capacita` massima
     * @exception IndexOutOfBoundsException
     *            se cap < val.length
     */
    public ArrayStack(Object val[], int cap)
    {
        this(cap);
        for (int i=0; i<val.length; i++)
            push(val[i]);
                // questa istruzione puo` sollevare
                // l'eccezione IndexOutOfBoundsException
    } //[c] ArrayStack

    /**[m]
     * O(1)
     * @see Stack
     */
    public int size()
    {
        return top+1;
    } //[m] size

    /**[m]
     * O(1)
     * @see Stack
     */
    public boolean isEmpty()
    {
        return top < 0;
    } //[m] isEmpty

    /**[m]
     * O(1)
     * @see Stack
     */
    public Object top()
      throws StackEmptyException
    {
        if (isEmpty())
            throw new StackEmptyException(
              "In ArrayStack.top");
        return S[top];
    } //[m] top

    /**[m]
     * O(1)
     * @see Stack
     */
    public void push(Object obj)
    {
        if (size() == capacity)
            throw new StackFullException(
              "In ArrayStack.push");
        S[++top] = obj;
    } //[m] push

    /**[m]
     * O(1)
     * @see Stack
     */
    public Object pop()
      throws StackEmptyException
    {
        if (isEmpty())
            throw new StackEmptyException(
              "In ArrayStack.pop");
        Object elem = S[top];
        S[top--] = null;
        return elem;
    } //[m] pop

    /**[m] !!
     * capacita` corrente - O(1)
     * @return la capacita` del vettore
     */
    public int capacity()
    {
        return capacity;
    } //[m] capacity

    /**[m] !!
     * conversione a stringa - O(n)
     * @return stringa con il contenuto
     */
    public String toString()
    {
        String str =
          "Stack su vettore [size=" + size() +
          "] (top all'inizio):\n";

output:
        for (int j=0; ; j++)
        {
            int pos = 0;
            for (int i=0; i<6; i++)
            {
                // 6 valori per riga
                if ((pos = j*6+i) > top)
                    // finita la lista
                    break output;
                str += pos + ": " +
                  S[top-pos].toString() + " | ";
            } //for(i)
            if (pos != top)
                str += "\n";
        } //for(j)
        return str;
    } //[m] toString

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "ArrayStack: Battere almeno 6 interi (fine=Ctrl-Z)");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        ArrayStack myStack = new ArrayStack(30);

        sIn.parseNumbers();
        try
        {
            Integer next;
            int pos = 0;

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sIn.nval);
                    myStack.push(next);
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while

        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch(EOF)
        catch (StackFullException e)
        {
            FI2Sys.out.println("[" + e + "] troppi numeri!");
            // termina il ciclo
        } //catch(StackFullException)
        // lo stack ha eseguito il push dei
        // numeri forniti nell'ordine

        FI2Sys.out.println("Stack creato");
        FI2Sys.out.println(myStack);

        FI2Sys.out.println("size=" + myStack.size());
        FI2Sys.out.println("top=" +
          ((Integer)myStack.top()).intValue());

        myStack.pop();
        myStack.pop();
        FI2Sys.out.println("Lo stack dopo due pop e`:"+
          myStack);
        FI2Sys.out.println(
          "Svuota lo stack [ora genera eccezione]");
        for (; myStack.size()>=0; myStack.pop());
            // genera eccezione di stack vuoto
    } //[m][s] main

} //{c} ArrayStack
