package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * ArrayVector.java
 * Implementazione con vettore espandibile
 * di un Vector
 * (sequenza con accesso indicizzato) -
 * derivata da ArrayRankedSequence della v 1.x 
 *
 * @version 2.00 00/09/20
 * @author M.Moro DEI UPD
 */

import java.io.*;
import java.util.NoSuchElementException;
import FI2.Set.ObjectIterator;
import FI2.Exceptions.BoundaryViolationException;
import FI2.Set.Container;
import FI2.Set.InspectableContainer;
import FI2.Util.FI2Sys;

/**{c}
 * sequenza generica con accesso indicizzato
 * realizzata come vettore espandibile
 * @see Vector
 */
public class ArrayVector implements Vector
{
    protected Object a[];  //vettore di supporto
    private int capacity = 16;  // capacita`
    public static final int CAPACITY = 100;  
    protected int size = 0;  // numero elementi corrente
      // parzialmente estendibile

    /**[c]
     * costruttore di default
     */
    public ArrayVector()
    {
        a = new Object[capacity];
    } //[c] ArrayVector

    /**[c] !!
     * costruttore con una copia dei riferimenti contenuti
     * nell'array fornito -
     * effettua un inserimento nell'ordine degli elementi -
     * gli oggetti non vengono clonati 
     * @param val  array da copiare
     * @param cap  capacita` massima
     */
    public ArrayVector(Object val[])
    {
        while(capacity < val.length)
            capacity <<= 1; // *2
        a = new Object[capacity];
        for (int i=0; i<val.length; i++)
            insertAtRank(i, val[i]);
    } //[c] ArrayVector

    /**[m]
     * O(1)
     * @see InspectableContainer
     */
    public int size()
    {
        return size;
    } //[m] size

    /**[m]
     * O(1)
     * @see InspectableContainer
     */
    public boolean isEmpty()
    {
        return size == 0;
    } //[m] isEmpty

    /**[m]
     * @see InspectableContainer
     */
    public ObjectIterator elements()
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per gli elementi
         */
        class Iter implements ObjectIterator
        {
            private int pos = 0;
              // enumeratore di indici

             /**[m]
             * @see FI2.Set.ObjectIterator
             */
            public boolean hasNext()
            {
                return pos < size;
            } //[m] hasNext

            /**[m]
             * @see FI2.Set.ObjectIterator
             */
            public Object nextObject()
            {
                if (pos >= size)
                    throw new NoSuchElementException(
"In ArrayVector.Iter.nextObject: fine elementi [pos="
                  + pos + "]");
                return a[pos++];
            } //[m] nextObject

        } //{c} Iter

        return new Iter();
    } //[m] elements

    /**[m]
     * @see Container
     */
    public Container newContainer()
    {
        return new ArrayVector();
    } //[m] newContainer

    /**[m]
     * O(1)
     * @see InspectableVector
     */
    public Object elemAtRank(int r)
      throws BoundaryViolationException
    {
        if (r<0 || r>size-1)
            throw new BoundaryViolationException(
              "In ArrayVector.elemAtRank: indice=" + 
              r);
        return a[r];
    } //[m] elemAtRank

    /**[m]
     * O(1)
     * @see Vector
     */
    public Object replaceAtRank(int r, Object e)
      throws BoundaryViolationException
    {
        if (r<0 || r>size-1)
            throw new BoundaryViolationException(
  "In ArrayVector.replaceAtRank: indice=" + 
              r);
        Object temp = a[r];
        a[r] = e;
        return temp;
    } //[m] replaceAtRank

    /**[m]
     * O(n)
     * @see Vector
     */
    public void insertAtRank(int r, Object e)
      throws BoundaryViolationException
    {
        if (r<0 || r>size)
            throw new BoundaryViolationException(
  "In ArrayVector.insertAtRank: indice=" + 
              r);
        if (size == capacity)
        {
            // soggetto ad espansione
            capacity <<= 1; // *2
            Object[] b =
              new Object[capacity];

            for (int i=0; i<size; i++)
                b[i] = a[i];
                    // copia dei riferimenti
                    // si puo` usare anche System.arracopy
            a = b;
                // sostituzione con il nuovo
        }
        for (int i=size-1; i >= r; i--)
            a[i+1] = a[i];
        a[r] = e;
        size++;
    } //[m] insertAtRank

    /**[m]
     * O(n)
     * @see Vector
     */
    public Object removeAtRank(int r)
      throws BoundaryViolationException
    {
        if (r<0 || r>size-1)
            throw new BoundaryViolationException(
   "In ArrayVector.removeAtRank: indice=" + 
              r);
        Object temp = a[r];
        for (int i=r; i<=size-2; i++)
            a[i] = a[i+1];
        size--;
        return temp;
    } //[m] removeAtRank
    
    /**[m] !!
     * capacita` corrente
     * @return la capacita` del vettore
     */
    public int capacity()
    {
        return capacity;
    } //[m] capacity

    /**[m] !!
     * conversione a stringa
     * @return stringa con il contenuto
     */
    public String toString()
    {
        String str =
          "Vector su array [size=" + size() +
          "]\n";

output:
        for (int j=0; ; j++)
        {
            int pos = 0;
            for (int i=0; i<6; i++)
            {
                // 6 valori per riga
                pos = j*6+i;
                if (pos >= size)
                    // finita la lista
                    break output;
                str += pos + ": " +
                  elemAtRank(pos).toString() + " | ";
            } //for(i)
            if (pos < size)
                str += "\n";
        } //for(j)
        return str;
    } //[m] toString

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "ArrayVector: Battere almeno 6 interi (fine=Ctrl-Z)"
              );
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        ArrayVector myVec = new 
          ArrayVector();

        sIn.parseNumbers();
        try
        {
            Integer next;
            int pos = 0;

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sIn.nval);
                    myVec.insertAtRank(pos++, next);
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while

        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch(EOF)
        catch (BoundaryViolationException e)
        {
            FI2Sys.out.println("[" + e + "] troppi numeri!");
            // termina i1l ciclo
        } //catch(BoundaryViolationException)

        FI2Sys.out.println("Vector creato");
        FI2Sys.out.println(myVec);

        FI2Sys.out.println("size=" + myVec.size());
        FI2Sys.out.println("elemento all'indice 4=" +
          ((Integer)myVec.elemAtRank(4)).intValue());

        Object rem = myVec.removeAtRank(5);
        FI2Sys.out.println("elemento rimosso in 5:" + rem);
        rem = myVec.replaceAtRank(3, new Integer(66));
        FI2Sys.out.println("elemento sostituito in 3:" + rem);
        FI2Sys.out.println(
 "Il vettore dopo remove in 5 e replace in 3 e` [size=" +
          myVec.size() + "]:");
        ObjectIterator it = myVec.elements();
        try
        {
            while(true)
                FI2Sys.out.println(
                  it.nextObject().toString());
        }
        catch (NoSuchElementException e)
        {
            FI2Sys.out.println("Fine iterazione: " + e);
        }
    } //[m][s] main

} //{c} ArrayVector
