package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * HeapPriorityQueue.java
 * Implementazione con heap
 * di coda a priorita` -
 * derivata da HeapSimplePriorityQueue della v 1.x 
 *
 * @version 2.00 00/09/24
 * @author M.Moro DEI UPD
 */

import java.util.NoSuchElementException;
import FI2.Set.Item;
import FI2.Set.Position;
import FI2.Set.Container;
import FI2.Set.InspectableContainer;
import FI2.Set.InspectableKeyBasedContainer;
import FI2.Util.Comparator;
import FI2.Exceptions.EmptyContainerException;
import FI2.Exceptions.InvalidKeyException;
import FI2.Exceptions.InvalidPositionException;
import FI2.Set.ObjectIterator;
import FI2.Tree.HeapTree;

/**{c}{a}
 * coda a priorita`
 * realizzata mediante un heap - 
 * lo heap si appoggia su un albero binario
 * specializzato (HeapTree) realizzato 
 * dalla classe che si derivera`
 * o in forma concatenata o con una sequenza su array -
 * la classe e` astratta perche' non implementa
 * costruttori ne' il metodo newContainer()
 * (i due casi sono diversi anche per la determinazione
 * dell'ultima posizione, piu` semplice nel secondo caso)
 * ne' il metodo sort, che nel secondo caso
 * fa uso del meccanismo di BuildHeap
 */
public abstract class HeapPriorityQueue 
  implements PriorityQueue 
{
    protected HeapTree T;  // l'albero binario heap
    protected Comparator comp;  // comparatore interno

    /**[m]
     * @see InspectableContainer
     */
    public int size() { return (T.size() - 1) / 2; }
  
    /**[m]
     * @see InspectableContainer
     */
    public boolean isEmpty() { return T.size() == 1; }
  
    /**[m]
     * @see InspectableContainer
     */
    public ObjectIterator elements()
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per gli elementi
         */
        class Iter implements ObjectIterator
        {
            private ObjectIterator itT;
            private int pos;
              // enumeratore di indici
              // si sfrutta la condizione di completezza
              // dello heap

            /**[c]
             * costruttore di default
             */
            Iter()
            {
                itT = T.elements();
                pos = 0;
            } //[c] Iter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return pos < size();
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                if (pos >= size())
                    throw new NoSuchElementException(
       "In HeapPriorityQueue.Iter.nextObject: " + 
                      "fine elementi [pos=" + pos + "]");
                pos++;
                Item next;
                while ((next = 
                  (Item)itT.nextObject()) == null);
                return next;
            } //[m] nextObject

        } //{c}{l} Iter

        return new Iter();
    } //[m] elements

    /**[m]
     * @see InspectableKeyBasedContainer
     */
    public ObjectIterator keys()  
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per scorrere le chiavi -
         * si appoggia sull'iteratore di sequenza
         */
        class KeysIter implements ObjectIterator
        {
            private ObjectIterator itemIt;
              // iteratore sulla sequenza

            /**[c]
             * costruttore di default
             */
            KeysIter()
            {
                itemIt = elements();
            } //[c] KeysIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return itemIt.hasNext();
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                if (!hasNext())
                    throw new NoSuchElementException(
"HeapPriorityQueue.KeysIter.nextObject(): fine elementi"
                  );
                return ((Item)itemIt.nextObject()).key();
            } //[m] nextObject

        } //{c}{l} KeysIter

        return new KeysIter();
    } //[m] keys

    /**[m]
     * @see InspectablePriorityQueue
     */
    public Object minElement() throws EmptyContainerException 
    {
        if (isEmpty())
            throw new EmptyContainerException(
              "Coda a priorita` vuota!");
        return element(T.root());
    } //[m] minElement
  
    /**[m]
     * @see InspectablePriorityQueue
     */
    public Object minKey() throws EmptyContainerException 
    {
        if (isEmpty())
            throw new EmptyContainerException(
            "Coda a priorita` vuota!");
        try 
        {
            return key(T.root());
        } catch (InvalidPositionException e) 
        { 
            // non dovrebbe accadere
        }
        return null;
    } //[m] minKey
  
    /**[m]
     * @see PriorityQueue
     */
    public void insertItem(Object k, Object e) 
      throws InvalidKeyException 
    {
        if (!comp.isComparable(k))
          throw new InvalidKeyException("Chiave illegale");
        Position z = T.add(new Item(k, e)); 
          // aggiunta in ultima posizione
        // z e` la nuova ultima posizione 
        // ovvero la foglia piu` a sinistra

        // ora bisogna
        // ripristinare la regola di ordinamento heap
        Position u;
        while (!T.isRoot(z)) 
        { 
            // up-heap bubbling
            u = T.parent(z);
            if (comp.isLessThanOrEqualTo(
              key(u), key(z))) 
                // ordinamento corretto ripristinato
                break;
            T.swapElements(u, z);
              // scambio per ripristinare l'ordinamento
            z = u;
        }
    } //[m] insertItem
  
    /**[m]
     * @see PriorityQueue
     */
    public Object removeMin() 
      throws EmptyContainerException 
    {
        Object min = minElement(); 
          // throws EmptyContainerException
        if (size() == 1)
            T.remove();  // un solo nodo
        else
        {
            T.replaceElement(T.root(), T.remove());
              // sostituisce la radice con l'ultimo inserito
            Position r = T.root();
            // ora bisogna ripristinare l'ordinamento heap
            // down-heap bubbling
            while (T.isInternal(T.leftChild(r))) 
            {
                Position s;
                // r ha un figlio sinistro interno
                if (T.isExternal(T.rightChild(r)) ||
                  // r ha un figlio destro esterno
                  comp.isLessThanOrEqualTo(
                  key(T.leftChild(r)),
                  key(T.rightChild(r)))
                  // r ha due figli entrambi interni e 
                  // il sinistro e` <= del destro
                  )
                    s = T.leftChild(r);
                else
                    s = T.rightChild(r);
                // s e` il figlio minore 
                if (comp.isLessThan(
                key(s), key(r))) 
                {
                    // ordine errato, scambio
                    T.swapElements(r, s);
                    r = s;
                }
                else 
                    // l'ordinamento e` ripristinato
                    break;
            } // while(isInternal)
        } //else
        return min;
    } //[m] removeMin
  
    /**[m]
     * chiave associata al posizionatore
     * @param p  posizionatore 
     * @return chiave associata
     * @exception InvalidPositionException  
     * posizionatore illegale
     */
    protected Object key(Position p) 
      throws InvalidPositionException 
    {
        return ((Item) p.element()).key(); 
          // il cast non dovrebbe fallire
    } //[m] key
  
    /**[m]
     * elemento associato al posizionatore
     * @param p  posizionatore 
     * @return elemento associato
     * @exception InvalidPositionException  
     * posizionatore illegale
     */
    protected Object element(Position p) 
      throws InvalidPositionException 
    {
        return ((Item) p.element()).element(); 
          // il cast non dovrebbe fallire
    } //[m] element
  
} //{c}{a} HeapPriorityQueue
