package FI2.Linear;
/**{p}
 * -- FI2.Linear --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Strutture lineari
 */

/**{f}
 * SLList.java
 * Implementazione con lista concatenata semplice
 * di un List (sequenza con posizioni) -
 * derivata da SListPositionalSequence della v 1.x 
 *
 * @version 2.00 00/09/20
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Set.ObjectIterator;
import FI2.Exceptions.*;
  // per InvalidPositionException, BoundaryViolationException, 
  // EmptyContainerException
import java.util.NoSuchElementException;
import FI2.Util.FI2Sys;
import FI2.Set.Container;
import FI2.Set.InspectableContainer;
import FI2.Set.PositionalContainer;
import FI2.Set.InspectablePositionalContainer;
import FI2.Set.Position;
import FI2.Set.PositionIterator;

/**{c}
 * sequenza generica con posizioni
 * realizzata mediante una lista concatenata
 * semplice - 
 * viene definita come estensione di LinkedQueue
 * giusto per far vedere come riutilizzare software gia`
 * scritto e collaudato.
 * @see List
 * @see LinkedQueue
 */
public class SLList extends LinkedQueue
  implements List
{

    /**[c]
     * costruttore di default
     */
    public SLList()
    {
        super();
    } //[c] SLList

    /**[c] !!
     * costruttore con una copia dei riferimenti contenuti
     * nell'array fornito -
     * effettua un inserimento nell'ordine degli elementi -
     * gli oggetti non vengono clonati 
     * @param val  array da copiare
     * @exception IndexOutOfBoundsException
     *            se cap < val.length
     */
    public SLList(Object val[])
    {
        super(val);
            // questa istruzione puo` sollevare
            // l'eccezione IndexOutOfBoundsException
    } //[c] SLList

    /**[m]
     * @see InspectableContainer
     */
    public ObjectIterator elements()
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per gli elementi -
         * sfrutta quello delle posizioni
         * @see PosIter
         */
        class Iter implements ObjectIterator
        {
            private PositionIterator posIt;
              // iteratore di posizioni

            /**[c]
             * costruttore di default
             */
            Iter()
            {
                posIt = positions();
            } //[c] Iter

             /**[m]
             * @see FI2.Set.ObjectIterator
             */
            public boolean hasNext()
            {
                return posIt.hasNext();
            } //[m] hasNext

            /**[m]
             * @see FI2.Set.ObjectIterator
             */
            public Object nextObject()
            {
                return ((SLNodePosition)posIt.nextPosition())
                  .getElement();
            } //[m] nextElement

        } //{c} Iter

        return new Iter();
    } //[m] elements

    /**[m]
     * @see Container
     */
    public Container newContainer()
    {
        return new SLList();
    } //[m] newContainer

    /**[m]
     * @see InspectablePositionalContainer
     */
    public PositionIterator positions()
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore
         * interno per le posizioni
         */
        class PosIter implements PositionIterator
        {
            private Position p;

            /**[c]
             * costruttore di default
             */
            PosIter()
            {
                try
                {
                    p = first();
                      // prosizionatore al primo
                }
                catch (EmptyContainerException e)
                {
                    // posizione volutamente illegale
                    p = new SLNodePosition();
                }
            } //[c] PosIter

             /**[m]
             * @see FI2.Set.ObjectIterator
             */
            public boolean hasNext()
            {
                try
                {
                    p.element();
                      // tenta l'accesso all'elemento
                    // ok, c'e`
                    return true;
                }
                catch (InvalidPositionException e)
                {
                    // no, non c'e`
                    return false;
                }
            } //[m] hasNext

            /**[m]
             * @see FI2.Set.ObjectIterator
             */
            public Object nextObject()
            {
                if (!hasNext())
                    throw new NoSuchElementException(
"In SLList.PosIter.nextObject: " +
                  "fine elementi");
                Position curPos = p;
                try
                {
                    p = after(p);
                      // tenta l'aggiornamento del succ.
                    // ok, c'e`
                }
                catch (BoundaryViolationException e)
                {
                    // no, non c'e`, posizionatore illegale
                    p = new SLNodePosition();
                }
                return curPos;
            } //[m] nextObject

            /**[m]
             * @see FI2.Set.PositionIterator
             */
            public Position nextPosition()
            {
                return (Position) nextObject();
            } //[m] nextPosition

        } //{c} PosIter

        return new PosIter();
    } //[m] positions

    /**[m]
     * verifica la validita` del posizionatore -
     * O(1)
     * @param p  posizionatore da verificare
     * @return il posizionatore come SequencePosition 
     * se legale
     * @exception InvalidPositionException  
     * posizionatore illegale
     */
    protected SLNodePosition checkPosition(Position p)
      throws InvalidPositionException
    {
        // p.container() verifica se il posizionatore
        // e` legale
        if (p.container() != this)
            throw new InvalidPositionException(
              "Posizionatore riferito ad altro contenitore");
        try
        {
            SLNodePosition temp = (SLNodePosition) p;
            return temp;
        }
        catch (ClassCastException e)
        {
            // il posizionatore non e` di tipo 
            // SequencePosition
            throw new InvalidPositionException(
"Posizionatore di tipo non corretto per questo contenitore");
        }
    } //[m] checkPosition

    /**[m]
     * O(1)
     * @see PositionalContainer
     */
    public Object replaceElement(Position p, Object newElem)
      throws InvalidPositionException
    {
        SLNodePosition here = checkPosition(p);
        Object oldElem = here.getElement();
        here.setElement(newElem);
        return oldElem;
    } //[m] replaceElement

    /**[m]
     * O(1)
     * @see PositionalContainer
     */
    public void swapElements(Position p, Position q)
      throws InvalidPositionException
    {
        SLNodePosition first = checkPosition(p);
        SLNodePosition second = checkPosition(q);
        Object oldFirstElem = first.getElement();
        first.setElement(second.getElement());
        second.setElement(oldFirstElem);
    } //[m] swapElements

    /**[m]
     * O(1)
     * @see InspectableList
     */
    public Position first() throws EmptyContainerException
    {
        if (isEmpty())
            throw new EmptyContainerException(
              "In SLList.first");
        return (SLNodePosition)head;
          // head e` ereditato come SLNode ma 
          // in questo contesto si sa che i nodi sono
          // effettivamente oggetti di classe SLNodePosition
    } //[m] first

    /**[m]
     * O(1)
     * @see InspectableList
     */
    public Position last() throws EmptyContainerException
    {
        if (isEmpty())
            throw new EmptyContainerException(
              "In SLList.last");
        return (SLNodePosition)tail; 
          // tail e` ereditato come SLNode ma 
          // in questo contesto si sa che i nodi sono
          // effettivamente oggetti di classe SLNodePosition
    } //[m] last

    /**[m]
     * O(1)
     * @see InspectableList
     */
    public boolean isFirst(Position p) 
      throws InvalidPositionException
    {
        SLNodePosition here = checkPosition(p);
        return here == (SLNodePosition)head;
    } //[m] isFirst

    /**[m]
     * O(1)
     * @see InspectableList
     */
    public boolean isLast(Position p) 
      throws InvalidPositionException
    {
        SLNodePosition here = checkPosition(p);
        return here == (SLNodePosition)tail;
    } //[m] isLast

    /**[m]
     * O(n)
     * @see InspectableList
     */
    public Position before(Position p) 
      throws InvalidPositionException, 
      BoundaryViolationException
    {
        SLNodePosition here = checkPosition(p);
        if (here == head)
            throw new BoundaryViolationException(
              "In SLList.before: " +
              "posizionatore in prima posizione");
        SLNodePosition prev = (SLNodePosition)head;
          // head e` ereditato come SLNode ma 
          // in questo contesto si sa che i nodi sono
          // effettivamente oggetti di classe SLNodePosition
        SLNodePosition next;
        while((next=(SLNodePosition)prev.getNext()) != here)
            prev = next;
        // ora prev e` il precedente
        return prev; 
    } //[m] before

    /**[m]
     * O(1)
     * @see InspectableList
     */
    public Position after(Position p) 
      throws InvalidPositionException, 
      BoundaryViolationException
    {
        SLNodePosition here = checkPosition(p);
        if (here == tail)
            throw new BoundaryViolationException(
              "In SLList.after: " +
              "posizionatore in ultima posizione");
        return (SLNodePosition)here.getNext(); 
    } //[m] after

    /**[m]
     * O(1)
     * @see List
     */
    public Position insertFirst(Object e)
    {
        SLNodePosition node = 
          new SLNodePosition(this, e, 
          (SLNodePosition)head);
        if (size++==0)
            // era vuota
            tail = node;
        head = node;
        return node;
    } //[m] insertFirst

    /**[m]
     * O(1)
     * @see List
     */
    public Position insertLast(Object e)
    {
        // non si puo` usare enqueue di LinkedQueue
        // perche' aggiunge un SLNode e non un
        // SLNodePosition
        SLNodePosition node = 
          new SLNodePosition(this, e, null);
        if (size++==0)
            head = node;
        else
            tail.setNext(node);
        tail = node;
        return node;
    } //[m] insertLast

    /**[m]
     * O(n)
     * @see List
     */
    public Position insertBefore(Position p, Object e)
      throws InvalidPositionException
    {
        SLNodePosition prev;
        try
        {
            prev = (SLNodePosition)before(p);
        }
        catch (BoundaryViolationException ex)
        {
            // e` da inserire in testa
            return insertFirst(ex);
        }
        // ora p e` certamente ok
        // head e tail non debbono essere aggiornati
        SLNodePosition here = (SLNodePosition)p;
        SLNodePosition node = 
          new SLNodePosition(this, e, here);
        prev.setNext(node);
        size++;
        return node;
    } //[m] insertBefore

    /**[m]
     * O(n)
     * @see List
     */
    public Position insertAfter(Position p, Object e)
      throws InvalidPositionException
    {
        SLNodePosition here = checkPosition(p);
        if (here == tail)
            // e` da inserire in coda
            return insertLast(e);
        // ora p e` certamente ok
        // head e tail non debbono essere aggiornati
        SLNodePosition node = 
          new SLNodePosition(this, e, 
          (SLNodePosition)here.getNext());
        here.setNext(node);
        size++;
        return node;
    } //[m] insertAfter

    /**[m]
     * @see List
     */
    public Object remove(Position p)
      throws InvalidPositionException
    {
        SLNodePosition prev;
        Object toReturn;
        try
        {
            prev = (SLNodePosition)before(p);
        }
        catch (BoundaryViolationException e)
        {
            // e` la testa
            toReturn = dequeue();
            ((SLNodePosition)p).setInvalid();
              // invalida il posizionatore
            return toReturn;
        }
        // ora p e` certamente ok
        SLNodePosition here = (SLNodePosition)p;
        prev.setNext(here.getNext());
        size--;
        if (tail == here)
            // rimosso l'ultimo
            tail = prev;
        toReturn = here.getElement();
        here.setInvalid();
          // invalida il posizionatore
        return toReturn;
    } //[m] remove

    /**[m]
     * O(n)
     * @see List
     */
    public Object removeBefore(Position p)
      throws InvalidPositionException, 
      BoundaryViolationException
    {
        return remove(before(p));
    } //[m] removeBefore

    /**[m]
     * O(n)
     * @see List
     */
    public Object removeAfter(Position p)
      throws InvalidPositionException, 
      BoundaryViolationException
    {
        return remove(after(p));
    } //[m] removeAfter

    /**[m]
     * O(1)
     * @see List
     */
    public Object removeFirst() throws EmptyContainerException
    {
        try
        {
            return dequeue();
        }
        catch (QueueEmptyException e)
        {
            throw new EmptyContainerException(
              "In SLList.removeFirst");
        }
        // si poteva anche semplicemente mettere:
        // return remove(first());
    } //[m] removeFirst

    /**[m]
     * O(n)
     * @see List
     */
    public Object removeLast() throws EmptyContainerException
    {
        return remove(last());
    } //[m] removeLast

    /**[m] !!
     * conversione a stringa
     * @return stringa con il contenuto
     */
    public String toString()
    {
        ObjectIterator it = elements();
        String str =
          "List su lista concatenata [size=" +
          size() + "]\n";

output:
        for (int j=0; ; j++)
        {
            int pos = 0;
            for (int i=0; i<6; i++)
            {
                // 6 valori per riga
                pos = j*6+i;
                if (!it.hasNext())
                    // finita la lista
                    break output;
                str += pos + ": " +
                  it.nextObject().toString() + " | ";
            } //for(i)
            if (it.hasNext())
                str += "\n";
        } //for(j)
        return str;
    } //[m] toString

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
              "SLList: " +
              "Battere almeno 6 interi (fine=Ctrl-Z)");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        SLList myList = 
          new SLList();

        sIn.parseNumbers();
        try
        {
            Integer next;
            int pos = 0;

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sIn.nval);
                    myList.insertLast(next);
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while

        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch(EOF)

        FI2Sys.out.println("List creato");
        FI2Sys.out.println(myList);

        FI2Sys.out.println("size=" + myList.size());
        FI2Sys.out.println("primo elemento =" + 
          myList.first().element());
        Position fst = myList.first();
        FI2Sys.out.println("secondo elemento =" + 
          myList.after(fst).element());
        Position snd = myList.after(fst);
        FI2Sys.out.println("terzo elemento =" + 
          myList.after(snd).element());
        FI2Sys.out.println("ultimo elemento =" + 
          myList.last().element());
        Position thr = myList.after(snd);

        Object rem = myList.remove(snd);
        FI2Sys.out.println("secondo elemento rimosso:" + rem);
        myList.swapElements(fst, thr);
        FI2Sys.out.println("dopo swap 1^<->3^: " + myList);
        try
        {
            rem = myList.replaceElement(snd, new Integer(66));
            FI2Sys.out.println("sostituito in 2:" + rem +
              " con 66");
        }
        catch (InvalidPositionException e)
        {
            FI2Sys.out.println("2^ non c'e` piu`:" + e);
        }

        SLList myList2 = 
          new SLList();
        Position p2 = myList2.insertFirst("Stringa 1");
        p2 = myList2.insertAfter(p2, "Stringa 3");
        myList2.insertBefore(p2, "Stringa 2");
        FI2Sys.out.println("Secondo List: ");
        ObjectIterator it = myList2.elements();
        try
        {
            while(true)
                FI2Sys.out.println(
                  it.nextObject().toString());
        }
        catch (NoSuchElementException e)
        {
            FI2Sys.out.println("Fine enumerazione:" + e);
        }

        FI2Sys.out.println("[Ora solleva l'eccezione]: ");
        myList.insertAfter(p2, "Stringa 4");
          // questa solleva l'eccezione 
          // InvalidPositionException
    } //[m][s] main

} //{c} SLList
