package FI2.Set;
/**{p}
 * -- FI2.Set --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Insiemi
 */

/**{f}
 * ArrayBag.java
 * realizzazione di un Bag con vettore espandibile
 * derivata da ArrayBag della v 1.x 
 *
 * @version 2.00 00/09/29
 * @author M.Moro DEI UPD
 */

import FI2.Exceptions.InvalidElementException;
import FI2.Util.FI2Sys;

/**{c}
 * realizzazione di un insieme generico di Object
 * con ripetizione di oggetti e supporto
 * con vettore espandibile -
 * adotta l'approccio di 'specializzazione derivata'
 * da ArraySet -
 * si noti che il metodo equals() ereditato non tiene
 * conto della molteplicita`
 * @see ArraySet
 */
public class ArrayBag extends ArraySet
{
    /**[c]
     * costruttore base, Bag vuoto
     */
    public ArrayBag()
    {
        super();
    } //[c] ArrayBag

    /**[m]
     * @see Container
     */
    public Container newContainer()
    {
        return new ArrayBag();
    } //[m] newContainer

    /**[m]
     * aggiunge un elemento all'insieme
     * anche se vi appartenga gia` -
     * @param obj  oggetto da aggiungere
     */
    public void insertElement(Object obj)
    {
        addElem(obj);
    } //[m] insertElement

    /**[m]
     * restituisce il Bag unione (con ripetizioni) -
     * O(n1+n2)
     * @param snd  secondo Bag
     * @return Bag unione
     */
    public Set union(Set snd)
    {
        ObjectIterator en1 = elements();
            // enumeratore su this
        ObjectIterator en2 = snd.elements();
            // enumeratore su Snd
        Set union = (Set) newContainer();
            // Bag risultato

        while (en1.hasNext())
            union.insertElement(en1.nextObject());
                // copia di this
                // (si poteva definire in alternatica
                // il metodo clone())

        while (en2.hasNext())
            union.insertElement(en2.nextObject());
        return union;
    } //[m] union

    /**[m]
     * conversione a stringa
     * @return descrizione del contenuto
     */
    public String toString()
    {
        return "{Bag} " + super.toString(); 
    } //[m] toString

    /**[m][s]
     * metodo di collaudo
     * @param args[]  non usato
     */
    public static void main(String[] args)
    {
        Set bag1 = new ArrayBag();
        Set bag2 = new ArrayBag();
        Set dom =  new ArraySet();
        FI2Sys.out.println("Collaudo ArrayBag");
        FI2Sys.out.println("\nbag1 inizialmente e`" +
          " [" + (bag1.isEmpty() ? "" : "non ") + "vuoto]" );


        Object val;

        for (int i=1; i<=15; i++)
        {
            val =
              new Integer((int)(Math.random() * 20));
            bag1.insertElement(val);
        } //for
        
        // l'insieme puo` essere disuniforme
        bag1.insertElement("Stringa 1");
        bag1.insertElement("Stringa 2");
        bag1.insertElement("Stringa 1");
          // l'oggetto di questa terza stringa puo` essere 
          // diverso, come contenitore, da quello della 
          // prima ma il contenuto e` lo stesso

        for (int i=1; i<=7; i++)
        {
            val =
              new Integer((int)(Math.random() * 20));
            bag2.insertElement(val);
        } //for

        // l'insieme puo` essere disuniforme
        bag2.insertElement("Stringa 1");
        bag2.insertElement("Stringa 3");

        for (int i=0; i<=21; i++)
        {
            val = new Integer(i);
            dom.insertElement(val);
                // dominio 0..21
        } //for

        // l'insieme puo` essere disuniforme
        dom.insertElement("Stringa 1");
        dom.insertElement("Stringa 2");
        dom.insertElement("Stringa 3");

        Set sTmp;
        FI2Sys.out.println("\nbag1: " + bag1);
        FI2Sys.out.println("\nbag2: " + bag2);
        FI2Sys.out.println("\nbag1 inizialmente e`" +
          (bag1.isEqual(bag2) ? "uguale " : "diverso ") 
          + "da bag2" );
        FI2Sys.out.println("\n~bag1: " +
          bag1.complement(dom));
        FI2Sys.out.println("\n~bag2: " +
          bag2.complement(dom));
        FI2Sys.out.println("\nbag1 + bag2: " +
          (sTmp = bag1.union(bag2)) +
          " [" + (sTmp.isEmpty() ? "" : "non ") + "vuoto]" );
        FI2Sys.out.println("\nbag1 & bag2: " +
          (sTmp=bag1.intersect(bag2)) +
          " [" + (sTmp.isEmpty() ? "" : "non ") + "vuoto]" );
        FI2Sys.out.println("\nbag1 - bag2: " +
          (sTmp = bag1.subtract(bag2)) +
          " [" + (sTmp.isEmpty() ? "" : "non ") + "vuoto]" );
        try
        {
            ((GenericSet)bag1).insertElement(
              new Integer(2000), dom);
            // per provocare l'eccezione
        }
        catch (InvalidElementException ex)
        {
            // cattura
            FI2Sys.out.println(ex + "<<-- !!!");
        } //catch

        FI2Sys.out.println("\nIl dominio era: " + dom);
        FI2Sys.out.println("\nbag1 prima della rimozione: " + 
          bag1);
        bag1.removeElement("Stringa 1");
        FI2Sys.out.println("\nbag1 dopo la rimozione: " + 
          bag1);
    } //[m][s] main

} //{c} ArrayBag

