package FI2.Set;
/**{p}
 * -- FI2.Set --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Insiemi
 */

/**{f}
 * LocatorKeyBasedContainer.java
 * contenitore-insieme
 * organizzato con l'uso di chiavi
 * e con riferimenti (Locator)
 *
 * @version 1.00 00/09/25
 * @author M.Moro DEI UPD
 */

import FI2.Exceptions.InvalidKeyException;
import FI2.Exceptions.InvalidLocatorException;
import FI2.Exceptions.UncontainedLocatorException;

/**{i}
 * contenitore-insieme con metodi
 * read-write e basato sull'uso
 * delle chiavi per l'accesso
 * e dei riferimenti (Locator)
 */
public interface LocatorKeyBasedContainer
  extends InspectableLocatorKeyBasedContainer, Container
{
    /**[m][a]
     * inserisce un Item fondendo chiave e oggetto
     * forniti e restituisce il riferimento (Locator)
     * @param key  la chiave
     * @param element  l'oggetto
     * @return riferimento all'item inserito
     * @exception InvalidKeyException 
     * se chiave illegale
     */
    Locator insert(Object key, Object element)
      throws InvalidKeyException;
  
    /**[m][a]
     * rimuove l'Item indicato dal riferimento
     * fornito
     * @param loc  il riferimento
     * @exception InvalidLocatorException 
     * riferimento non valido
     * @exception UncontainedLocatorException 
     * l'item indicato non e` contenuto in un
     * contenitore
     */
    void remove(Locator loc)
      throws InvalidLocatorException, 
      UncontainedLocatorException;
  
    /**[m][a]
     * sostituisce l'oggetto nell'item indicato dal
     * riferimento (Locator)
     * @param loc  il riferimento
     * @param newElement  il nuovo oggetto
     * @return il vecchio oggetto nell'Item modificato
     * @exception InvalidLocatorException 
     * riferimento non valido
     */
    Object replaceElement (Locator loc, Object newElement) 
      throws InvalidLocatorException;

    /**[m][a]
     * sostituisce la chiave nell'item indicato dal
     * riferimento (Locator)
     * @param loc  il riferimento
     * @param newKey  la nuova chiave
     * @return il vecchio oggetto nell'Item modificato
     * @exception InvalidLocatorException 
     * riferimento non valido
     * @exception InvalidKeyException 
     * se chiave illegale
     */
    Object replaceKey(Locator loc, Object newKey)
      throws InvalidLocatorException,
      InvalidKeyException;

} //{i} LocatorKeyBasedContainer
