package FI2.Set;
/**{p}
 * -- FI2.Set --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Insiemi
 */

/**{f}
 * Set.java
 * interfaccia per un insieme generico
 *
 * @version 1.00 98/09/18
 * @author M.Moro DEI UPD
 */

import FI2.Exceptions.InvalidElementException;
import FI2.Util.SimpleComparable;

/**{i}
 * interfaccia per un insieme generico di Object -
 * non puo` contenere ripetizioni dello stesso oggetto -
 */
public interface Set extends Container
{
    /**[m][a]
     * aggiunge un elemento all'insieme
     * controllando che non vi appartenga gia` -
     * qualche realizzazione puo` sollevare
     * un'eccezione di tipo "full" -
     * @param obj  oggetto da aggiungere
     * @exception InvalidElementException
     * se l'elemento e` gia` presente
     */
    void insertElement(Object obj)
      throws InvalidElementException;

    /**[m][a]
     * rimuove un elemento dall'insieme
     * se presente -
     * @param obj  oggetto da rimuovere
     * @return true se rimosso; 
     *         false altrimenti (non presente)
     */
    boolean removeElement(Object obj);

    /**[m][a]
     * determina se un oggetto appartiene all'insieme -
     * per la comparazione si assume che gli elementi
     * implementino l'interfaccia SimpleComparable
     * @param obj  oggetto da cercare
     * @return true se l'oggetto e` un elemento
     *         dell'insieme;
     *         false altrimenti
     */
    boolean contains(Object obj);

    /**[m]
     * verifica se i due set contengono gli stessi 
     * elementi - 
     * per la comparazione si assume che gli elementi
     * implementino l'interfaccia SimpleComparable
     * @param snd  secondo insieme da confrontare
     * @return true se contengono gli stessi elementi;
     *         false altrimenti
     * @see SimpleComparable
     */
    boolean isEqual(Set snd);

    /**[m][a]
     * restituisce un nuovo set che e` l'unione 
     * di this e snd
     * @param snd  secondo set
     * @return Set unione
     */
    Set union(Set snd);

    /**[m][a]
     * restituisce un nuovo set che e` l'intersezione
     * di this e snd
     * @param snd  secondo set
     * @return set intersezione
     */
    Set intersect(Set snd);

    /**[m][a]
     * restituisce un nuovo set che e` la differenza
     * this - snd
     * @param snd  secondo set
     * @return set differenza (this - Snd)
     */
    Set subtract(Set snd);

    /**[m][a] !!
     * restituisce il set complemento
     * rispetto al dominio fornito
     * @param Dom  set dominio (senza ripetizioni)
     * @return set complemento (Dom - this)
     */
    Set complement(Set dom);

} //{i} Set
