package FI2.String;
/**{p}
 * -- FI2.String --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Stringhe
 */

/**{f}
 * Cl0Lex.java
 * analizzatore lessicale di Cl0
 *
 * @version 2.00 00/09/27
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Util.*;
import FI2.Set.ObjectIterator;

/**{c}
 * scandisce l'ingresso restituendo il prossimo token
 * in base alla definizione lessicale di Cl0 -
 *
 * classi di caratteri
 * alpha     [@_A-Za-z]
 * digit     [0-9]
 * delim     [ \t\r\n\f\v]
 * oper       [;=()*+\-/,^]
 * anybutnewl [^\n]
 * anyprintbutapex [^']
 *
 *  espressioni regolari
 * separatori       r1 = {delim}+
 * commento         r2 = "!"{anybutnewl}*"\n"
 * operatore        r3 = {oper}
 * numero           r4 = {digit}+
 * identificatore   r5 = {alpha}({alpha}|{digit})*
 * stringa          r6 = "'"({anyprintbutapex}*"''"?)*"'"
 */
public class Cl0Lex
{
    static final int A_S = 0;  // iniziale
    static final int C_S = A_S+1;  // commento
    static final int N_S = C_S+1;  // numero naturale
    static final int I_S = N_S+1;  // identificatore e keyword
    static final int S1_S = I_S+1;  // stringa
    static final int S2_S = S1_S+1;  // stringa dopo apice

    static final int INKEYWIDX = -1;   // indice semantico keyword 'in'
    static final int OUTKEYWIDX = -2;  // indice semantico keyword 'out'
    static final int ENDKEYWIDX = -3;  // indice semantico keyword 'end'

    private PushbackReader fIn;  // input
    private Cl0SymTable tab;  // tabella dei simboli
    private int row = 1;  // riga di definizione
    private int pos = 0;  // posizione nella riga
    private int state;  // stato corrente
    private int strIdx = 0;  // indice progressivo stringhe
    private boolean eofFlag = false;  // raggiunto eof

    /**[c]
     * costruttore base
     * @param in  reader di input
     */
    public Cl0Lex(PushbackReader in)
    {
        // inserimento keyword nella tabella
        Cl0TokenDesc cl0Keyws[] = new Cl0TokenDesc[3];
        cl0Keyws[0] = new Cl0TokenDesc("in", Cl0TokenDesc.KINTOK,
              0, 0, new Integer(INKEYWIDX)); 
        cl0Keyws[1] = new Cl0TokenDesc("out", Cl0TokenDesc.KOUTTOK,
              0, 0, new Integer(OUTKEYWIDX)); 
        cl0Keyws[2] = new Cl0TokenDesc("end", Cl0TokenDesc.KENDTOK,
              0, 0, new Integer(ENDKEYWIDX)); 
          // indici semantici negativi per i keyword
        fIn = in;
        tab = new Cl0SymTable(cl0Keyws);
    } //[c] Cl0Lex

    /**[m]
     * acquisizione di un token
     * @return token letto
     */
    public Cl0TokenDesc scan()
    {
        StringBuffer str = new StringBuffer();
        int ch = ' ';
        Cl0TokenDesc tk;

        state = A_S;

        for (; ; )
        {
            if (eofFlag)
                // eof gia` raggiunto
                ch = -1;
            else
            {
                try
                {
                    str.append((char)(ch = fIn.read()));
                }
                catch (IOException e)
                {
                    System.err.println("** "+e+": abortito **");
                    System.exit(0);
                }
            } // if eofFlag

            // realizzazione DFA
            switch(state)
            {
              case A_S:  // stato iniziale
                switch (ch)
                {
                  // lista operatori monocarattere
                  case ';':
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.SEMICTOK,
                      row, pos, new Character((char)ch));
                    pos++;
                    return tk;
                  case '=':
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.ASSTOK,
                      row, pos, new Character((char)ch));
                    pos++;
                    return tk;
                  case '(':
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.LEFPTOK,
                      row, pos, new Character((char)ch));
                    pos++;
                    return tk;
                  case ')':
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.RIGPTOK,
                      row, pos, new Character((char)ch));
                    pos++;
                    return tk;
                  case '+':
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.PLUSTOK,
                      row, pos, new Character((char)ch));
                    pos++;
                    return tk;
                  case '-':
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.MINUSTOK,
                      row, pos, new Character((char)ch));
                    pos++;
                    return tk;
                  case '*':
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.MULTOK,
                      row, pos, new Character((char)ch));
                    pos++;
                    return tk;
                  case '/':
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.DIVTOK,
                      row, pos, new Character((char)ch));
                    pos++;
                    return tk;
                  case ',':
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.COMMATOK,
                      row, pos, new Character((char)ch));
                    pos++;
                    return tk;
                  case '^':
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.REFTOK,
                      row, pos, new Character((char)ch));
                    pos++;
                    return tk;
                  
                  case -1:  // EOF
                    eofFlag = true;                    
                    return new Cl0TokenDesc("<EOF>", 
                      Cl0TokenDesc.EOFTOK, row, pos, null);

                  // lessemi multicarattere
                  case '!':  // apertura commento
                    state = C_S;
                    break;

                  case '\'':  // apertura stringa
                    state = S1_S;
                    // il lessema non contiene i delimitatori
                    str.setLength(0);  // rimuove apice
                    break;

                  case '@':  // identificatore
                  case '_':
                    state = I_S;
                    break;

                  default:  // sep, nat, iden alphadig, err
                    if (Character.isWhitespace((char)ch))
                    {
                        // separatore
                        str.setLength(0);  // rimuove separatore
                        if (ch == '\n')
                        {
                            row++;
                            pos = 0;
                        }
                        else
                            pos++;
                        break;
                    }

                    if (Character.isDigit((char)ch))
                    {
                        // naturale
                        state = N_S;
                        break;
                    }

                    if (Character.isLetter((char)ch))
                    {
                        // identificatore
                        state = I_S;
                        break;
                    }

                    // illegale
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.LEXERR,
                      row, pos, null);
                    pos++;
                    return tk;
                } //switch(ch) per A_S
                break;

              case C_S:  // in commento
                switch (ch)
                {
                  case '\n':  // fine commento
                    str.setLength(0);  // rimuove commento
                    state = A_S;
                    row++;
                    pos = 0;
                    break;
                  case -1:  // EOF inatteso
                    eofFlag = true;                    
                    return new Cl0TokenDesc("<EOF>", 
                      Cl0TokenDesc.EOFTOK, row, pos, null);
                  default: // carattere in commento, nop
                    ;
                } //switch(ch) per C_S
                break;
              
              case N_S:  // in numero
                if (! Character.isDigit((char)ch))
                {
                    // fine numero
                    str.setLength(str.length()-1);
                    if (ch != -1)
                    {
                        try
                        {
                            fIn.unread(ch);
                        }
                        catch (IOException e)
                        {
                            System.err.println("** "+e+": abortito **");
                            System.exit(0);
                        }
                    } // if ! EOF
                    else 
                        eofFlag = true;                    
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.NATTOK,
                      row, pos, Integer.valueOf(str.toString()));
                    pos += str.length();
                    return tk;
                } // if digit
                break;

              case I_S:  // in identificatore
                if (! Character.isLetterOrDigit((char)ch) && ch != '@' && ch != '_')
                {
                    // fine identificatore
                    str.setLength(str.length()-1);
                    if (ch != -1)
                    {
                        try
                        {
                            fIn.unread(ch);
                        }
                        catch (IOException e)
                        {
                            System.err.println("** "+e+": abortito **");
                            System.exit(0);
                        }
                    } // if !EOF
                    else
                        eofFlag = true;                    
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.IDENTOK,
                      row, pos, new Integer(strIdx++));  // valore sem. provvisorio
                    pos += str.length();
                    return tk;
                } // if ! alfadigit
                break;

              case S1_S:  // in stringa
                switch (ch)
                {
                  case '\'':  // forse fine stringa
                    state = S2_S;
                    str.setLength(str.length()-1);  // un apice rimosso
                    break;
                  case '\n':  // a capo in stringa
                    System.err.println("!! avviso: stringa non terminata nella riga");
                    str.setLength(str.length()-1);
                    try
                    {
                        fIn.unread(ch);
                    }
                    catch (IOException e)
                    {
                        System.err.println("** "+e+": abortito **");
                        System.exit(0);
                    }
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.STRTOK,
                      row, pos, new Integer(strIdx++));
                    pos += str.length();
                    return tk;
                  case -1:  // EOF inatteso
                    System.err.println("\n!! avviso: EOF inatteso in stringa");
                    str.setLength(str.length()-1);
                    eofFlag = true;     
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.STRTOK,
                      row, pos, new Integer(strIdx++));
                    pos += str.length();
                    return tk;
                  default:
                    // char nella stringa
                } //switch(ch) per S1_S
                break;
              
              case S2_S:  // dopo ' in stringa
                switch (ch)
                {
                  case '\'':  // apice protetto
                    state = S1_S;  // l'apice rimane
                    break;
                  case -1: // l'apice chiudeva
                  default:
                    str.setLength(str.length()-1);
                    if (ch == -1)
                        eofFlag = true;                    
                    else
                    {
                        try
                        {
                            fIn.unread(ch);
                        }
                        catch (IOException e)
                        {
                            System.err.println("** "+e+": abortito **");
                            System.exit(0);
                        }
                    } // if
                    tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.STRTOK,
                      row, pos, new Integer(strIdx++));
                    pos += str.length();
                    return tk;
                } //switch(ch) per S2_S
                break;

              default:
                tk = new Cl0TokenDesc(str.toString(), Cl0TokenDesc.LEXERR,
                  row, pos, null);
                pos++;
                return tk;
            } //switch(state)
        } //for
    } //[m] scan

    /**[m]
     * acquisizione di un token esteso
     * con accesso alla tabella dei simboli
     * @return token letto
     */
    public Cl0TokenDesc lexan()
    {
        Cl0TokenDesc tk = scan();
        if (tk.type() == Cl0TokenDesc.IDENTOK)
        {
            // potrebbe anche essere una parola chiave
            tk = tab.LookUp(tk);
            // la chiamata restituisce tk originario con l'indice
            // semantico aggiornato se il simbolo e` nuovo,
            // quello nella tabella se il simbolo e` gia` presente:
            // in questo secondo caso la riga viene aggiunta
            // nella lista di righe
        }
        return tk;
    } //[m] lexan

    /**[m]
     * iteratore dei simboli nella tabella
     * @return enumeratore dei simboli
     */
    public ObjectIterator symbols()
    {  return tab.elements(); }

    /**[m][s]
     * metodo di collaudo
     */
    public static void main(String[] args)
    {
        Cl0Lex mylex = new Cl0Lex(new PushbackReader(new BufferedReader(new
          InputStreamReader(System.in))));

        FI2Sys.out.println(
          "Inserire il testo in Cl0 (end o Ctrl-Z per terminare):");
        for (; ; )
        {
            Cl0TokenDesc td = mylex.lexan();
            if (td.type() == Cl0TokenDesc.EOFTOK)
            {
                FI2Sys.out.println("\n-- Letto token ->" + td.toString(0) + "<-");
                break;
            }
            FI2Sys.out.println("-- Letto token ->" + td.toString(0) + "<-");
            if (td.type() == Cl0TokenDesc.KENDTOK)
                break;
        }
        ObjectIterator enTab = mylex.symbols();

        FI2Sys.out.println(
          "\nContenuto della tabella dei simboli");
        for (int i=1; enTab.hasNext(); i++)
        {
            Cl0TokenDesc td = (Cl0TokenDesc)enTab.nextObject();
            FI2Sys.out.println("-- Simbolo "+i+" ->"+
              td.toString(0)+"<-");

            ObjectIterator enRows = td.rows();
            while (enRows.hasNext())
                FI2Sys.out.println("---- in riga " + enRows.nextObject());
        }
    } //[m][s] main

} //{c} Cl0Lex
