package FI2.String;
/**{p}
 * -- FI2.String --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Stringhe
 */

/**{f}
 * Cl0SymTable.java
 * tabella dei simboli per Cl0
 *
 * @version 1.00 99/12/13
 * @author M.Moro DEI UPD
 */

import FI2.Dictionary.Dictionary;
import FI2.Dictionary.SeparateChainingHashTable;
import FI2.Dictionary.UnOrderedSLList;
import FI2.Set.ObjectIterator;
import FI2.Util.Equality;
import FI2.Util.StringComparator;
import FI2.Util.FI2Sys;

/**{c}
 * tabella dei simboli per il linguaggio Cl0 
 * con il supporto di un Hashtable
 */
public class Cl0SymTable
{
    private Dictionary ht;
    private int semIdx = 0;  
      // indice semantico progressivo per i simboli:
      // positivo per un identificatore,
      // negativo per un keyword

    /**[c]
     * costruttore di default
     */
    public Cl0SymTable() { this(null); }

    /**[c]
     * costruttore base
     * @param keywords  lista delle parole chiave
     */
    public Cl0SymTable(Cl0TokenDesc keywords[])
    {
        Equality eq = new StringComparator();
        ht = new SeparateChainingHashTable(eq, 
          new UnOrderedSLList(eq));
        if (keywords != null)
            for (int i=0; i<keywords.length; i++)
                ht.insertItem(keywords[i].toString(), keywords[i]);
    } //[c] Cl0SymTable

    /**[m]
     * ricerca-inserimento -
     * la chiave e` il lessema -
     * cerca il simbolo passato come token:
     * se trovato, aggiunge, alla lista di righe
     * del descrittore del simbolo nella tabella,
     * la riga indicata nel parametro, e restituisce
     * tale descrittore;
     * se non trovato, assegna un nuovo indice semantico
     * al descrittore parametro e lo inserisce nella tabella
     * @return tk se non trovato, il descrittore in tabella
     * se trovato
     */
    public Cl0TokenDesc LookUp(Cl0TokenDesc tk)
    {
        Object ret = ht.findElement(tk.toString());
        if (ret == Dictionary.NO_SUCH_KEY)
        {
            // non trovato, inserimento
            tk.setAttr(new Integer(semIdx++));
            ht.insertItem(tk.toString(), tk);
            return tk;  // token inserito
        }
        else 
        {
            // trovato
            ((Cl0TokenDesc)ret).addRow(tk.row());
            return (Cl0TokenDesc)ret;
        } // if ret
    } //[m] LookUp

    /**[m]
     * iteratore dei descrittori di simbolo
     * @return iteratore dei descrittori
     */
    public ObjectIterator elements()
    {  return ht.elements();  }

    /**[m][s]
     * metodo di collaudo
     */
    public static void main(String[] args)
    {
        Cl0TokenDesc kws[] = new Cl0TokenDesc[3];
        for (int i=0; i<3; i++)
            kws[i] = new Cl0TokenDesc("keyw"+i, Cl0TokenDesc.KINTOK,
              -1, -1, new Integer(-i-1));
                // indici semantici negativi per i keyword
        Cl0SymTable mySt = new Cl0SymTable(kws);
        mySt.LookUp(new Cl0TokenDesc("pippo", Cl0TokenDesc.IDENTOK, 1, 1, null));
        mySt.LookUp(new Cl0TokenDesc("pappo", Cl0TokenDesc.IDENTOK, 2, 2, null));
        mySt.LookUp(new Cl0TokenDesc("peppo", Cl0TokenDesc.IDENTOK, 3, 3, null));
        mySt.LookUp(new Cl0TokenDesc("pippo", Cl0TokenDesc.IDENTOK, 4, 4, null));
        mySt.LookUp(new Cl0TokenDesc("keyw1", Cl0TokenDesc.IDENTOK, 5, 5, null));

        ObjectIterator en = mySt.elements();
        FI2Sys.out.println("Contenuto della tabella dei simboli");

        for (int i=1; en.hasNext(); i++)
        {
            Cl0TokenDesc td = (Cl0TokenDesc)en.nextObject();
            FI2Sys.out.println("Simbolo "+i+" ->"+
              td.toString(0)+"<-");
            ObjectIterator enRows = td.rows();
            while (enRows.hasNext())
                FI2Sys.out.println("    Presente in riga "+
                  ((Integer)enRows.nextObject()));
        } // for
    } //[m][s] main

} //{c} Cl0SymTable
