package FI2.Temi;
/**{p}
 * -- FI2.Temi --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Temi d'esame
 */

/**{f}
 * ArrayPQueue.java
 * Implementazione con array di PQueue
 * Scritto del 6/9/99
 *
 * @version 1.00
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Exceptions.QueueEmptyException;
import FI2.Exceptions.BoundaryViolationException;
import FI2.Exceptions.QueueFullException;
import FI2.Examples.Linear.ArrayQueue;
import FI2.Linear.Queue; // per javadoc
import FI2.Util.FI2Sys;

/**{c}
 * coda FIFO realizzata con vettore
 * in forma circolare, con inserimento in
 * posizione relativa
 * @see Queue
 * @see PQueue
 */
public class ArrayPQueue extends ArrayQueue implements PQueue
{
    /**[c]
     * costruttore di default
     */
    public ArrayPQueue()
    {
        super();
    } //[c] ArrayPQueue

    /**[c]
     * costruttore con capacita` massima
     * in numero di elementi
     * @param cap  capacita` massima dell'array
     */
    public ArrayPQueue(int cap)
    {
        super(cap);
    } //[c] ArrayPQueue

    /**[m]
     * @see PQueue
     */
    public Object rear()
      throws QueueEmptyException
    {
        if (isEmpty())
            throw new QueueEmptyException(
              "In ArrayPQueue.rear");
        return Q[(r+capacity-1)%capacity];
    } //[m] rear

    /**[m]
     * @see PQueue
     */
    public void insert(Object element, int pos)
      throws BoundaryViolationException
    {
      if (pos <= 0)
        {
            // dal fondo
            if (-pos > size())
                throw new BoundaryViolationException(
                  "In ArrayQueue.insert pos="+pos);
            if (size() == capacity-1)
                throw new QueueFullException(
                  "In ArrayQueue.insert");
            int write = r, read = (r+capacity-1)%capacity;
              // imposta indice di scrittura e di lettura
            for (; pos < 0; write= (--write+capacity) % capacity, 
              read= (--read+capacity) % capacity, pos++)
                Q[write] = Q[read];
            Q[write] = element;
            r = ++r % capacity;
        }
        else
        {
            // dalla testa
            if (pos > size()+1)
                throw new BoundaryViolationException(
                  "In ArrayQueue.insert pos="+pos);
            if (size() == capacity-1)
                throw new QueueFullException(
                  "In ArrayQueue.insert");
            f = (f+capacity-1)%capacity;
            int write = f, read = (f+1)%capacity;
              // imposta indice di scrittura e di lettura
            for (; pos > 1; write= ++write % capacity, 
              read= ++read % capacity, pos--)
                Q[write] = Q[read];
            Q[write] = element;
        }
    } //[m] insert

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "ArrayPQueue: Battere una lettera-tipo(b,i,l,f,d) con valore-posizione oppure un comando(v,r,s)(fine=Ctrl-Z):");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        ArrayPQueue myQueue = new ArrayPQueue(30);

        sIn.parseNumbers();
        do
        {
            try
            {
                while(sIn.nextToken() != StreamTokenizer.TT_EOF)
                {
                    if (sIn.ttype == StreamTokenizer.TT_WORD)
                    {
                        switch (sIn.sval.charAt(0))
                        {
                          case 'b':
                            if (sIn.nextToken() != StreamTokenizer.TT_WORD)
                            {
                                FI2Sys.out.println(sIn.nval +
                                  " boolean illegale: non inserito");
                                break;
                            }
                            else if (
                              !sIn.sval.equals("true") && 
                              !sIn.sval.equals("false")
                            )
                            {
                                FI2Sys.out.println(sIn.sval +
                                  " boolean illegale: non inserito");
                                break;
                            }
                            Boolean bVal = new Boolean(sIn.sval);
                            if (sIn.nextToken() != StreamTokenizer.TT_NUMBER)
                                FI2Sys.out.println(sIn.sval +
                                  " posizione illegale: non inserito");
                            else
                            {
                                myQueue.insert(bVal, (int)sIn.nval);
                                FI2Sys.out.println("Inserito " + bVal + " con pos=" +
                                  (int)sIn.nval);
                            }
                            break;
    
                          case 'i':
                            if (sIn.nextToken() != StreamTokenizer.TT_NUMBER)
                            {
                                FI2Sys.out.println(sIn.sval +
                                  " intero illegale: non inserito");
                                break;
                            }
                            Integer iVal = new Integer((int)sIn.nval);
                            if (sIn.nextToken() != StreamTokenizer.TT_NUMBER)
                                FI2Sys.out.println(sIn.sval +
                                  " posizione illegale: non inserito");
                            else
                            {
                                myQueue.insert(iVal, (int)sIn.nval);
                                FI2Sys.out.println("Inserito " + iVal + " con pos=" +
                                  (int)sIn.nval);
                            }
                            break;

                          case 'v':
                            // visualizzazione coda
                            FI2Sys.out.println("Coda attuale:");
                            FI2Sys.out.println(myQueue);
                            break;

                          case 'c':
                            // visualizzazione elemento di coda
                            FI2Sys.out.println("L'ultimo elemento della coda:");
                            FI2Sys.out.println(myQueue.rear());
                            break;

                          case 'r':
                            // rimozione testa
                            FI2Sys.out.println("Rimosso " + myQueue.dequeue());
                                // la visualizzazione non era richiesta
                            break;

                          case 's':
                            // svuotamento
                            FI2Sys.out.println("Svuotamento:");
                            while (myQueue.size() > 0)
                                FI2Sys.out.println("Rimosso " + myQueue.dequeue());
                                // la visualizzazione non era richiesta
                            break;
                          default:
                          // comando illegale
                            FI2Sys.out.println(sIn.sval +
                              " illegale: non eseguito");
                        } // switch
                    } //if WORD
                    else  
                        FI2Sys.out.println(sIn.nval +
                          " non comando: non eseguito");
                } //while
            } // try
            catch (EOFException e)
            {
                // termina il ciclo
                break;
            } //catch(EOF)
            catch (QueueFullException e)
            {
                FI2Sys.out.println("[" + e + "] troppi numeri!");
            } //catch(QueueFullException)
            catch (QueueEmptyException e)
            {
                FI2Sys.out.println("[" + e + "] non ci sono numeri!");
            } //catch(QueueEmptyException)
            catch (BoundaryViolationException e)
            {
                FI2Sys.out.println("[" + e + "] non ci sono abbastanza numeri!");
            } //catch(BoundaryViolationException)
        } while(sIn.ttype != StreamTokenizer.TT_EOF);
    } //[m] main

} //{c} ArrayPQueue
