package FI2.Temi;
/**{p}
 * -- FI2.Temi --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Temi d'esame
 */

/**{f}
 * RBuf.java
 * Buffer con inversione d'ordine
 * Scritto del 20/9/99
 *
 * @version 1.00
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Linear.*;
import FI2.Exceptions.QueueEmptyException;
import FI2.Exceptions.QueueFullException;
import FI2.Exceptions.StackFullException;
import FI2.Util.FI2Sys;

/**{c}
 * Buffer con stack e coda interni
 */
public class RBuf
{
    private static final int BUFNUM = 3;
      // ampiezza stack con array e massimo numero di elementi presenti
    private static final int QUEUESIZE = 5;
      // ampiezza coda con array
    private Stack S;
    private Queue Q;
      // stack e coda

    /**[c]
     * costruttore di default
     */
    public RBuf()
    {
        S = new ArrayStack(BUFNUM);
        Q = new ArrayQueue(QUEUESIZE);
    } //[c] RBuf

    /**[c]
     * costruttore 2
     * @param dummy  per l'overload
     */
    public RBuf(int dummy)
    {
        S = new LinkedStack();
        Q = new LinkedQueue();
    } //[c] RBuf

    /**[m]
     * inserimento su stack ed eventuale suo
     * svuotamento su coda
     * @param el  oggetto da inserire
     * @exception QueueFullException  coda piena (solo se su array)
     */
    public void insert(Object el)
      throws QueueFullException, StackFullException
    {
    Object tmp = null;

        S.push(el);
        if (S.size() >= BUFNUM)
            try
            {
                while (! S.isEmpty())
                {
                    tmp = S.pop();
                    Q.enqueue(tmp);
                }
            	
            }
            catch (QueueFullException e)
            {
                // un elemento e' stato estratto da stack ma non
                // inserito in coda: rimesso su stack
                S.push(tmp);
                throw e;
            }
    } //[m] insert

    /**[m]
     * estrazione dalla coda
     * @return oggetto estratto
     * @exception QueueEmptyException  coda vuota
     */
    public Object remove()
      throws QueueEmptyException
    {
        Object ret = Q.dequeue();
        // a causa di troppi inserimenti,
        // lo stack potrebbe essere pieno:
        // si rimuove un solo elemento
        if (S.size() >= BUFNUM)
            Q.enqueue(S.pop());
        return ret;
    } //[m] remove

// <<<<<<<<<<< main NON richiesto!
    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "RBuf: Battere un comando (i,r) con valore se i (insert)(fine=Ctrl-Z):");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        RBuf myBuf = new RBuf();

        sIn.parseNumbers();
        do
        {
            try
            {
                while(sIn.nextToken() != StreamTokenizer.TT_EOF)
                {
                    if (sIn.ttype == StreamTokenizer.TT_WORD)
                    {
                        switch (sIn.sval.charAt(0))
                        {
                          case 'i':
                            // inserimento
                            if (sIn.nextToken() != StreamTokenizer.TT_WORD)
                            {
                                FI2Sys.out.println(sIn.nval +
                                  " stringa illegale: non inserito");
                                break;
                            }
                            myBuf.insert(sIn.sval);
                            break;
    
                          case 'r':
                            // rimozione
                            FI2Sys.out.println("Rimozione:");
                            FI2Sys.out.println(myBuf.remove());
                            break;

                          default:
                          // comando illegale
                            FI2Sys.out.println(sIn.sval +
                              " illegale: non eseguito");
                        } // switch
                    } //if WORD
                    else  
                        FI2Sys.out.println(sIn.nval +
                          " non comando: non eseguito");
                } //while
            } // try
            catch (EOFException e)
            {
                // termina il ciclo
                break;
            } //catch(EOF)
            catch (StackFullException e)
            {
                FI2Sys.out.println("[" + e + "] troppi elementi in stack e coda! Non inserito");
            } //catch(QueueFullException)
            catch (QueueFullException e)
            {
                FI2Sys.out.println("[" + e + "] troppi elementi in coda!");
            } //catch(QueueFullException)
            catch (QueueEmptyException e)
            {
                FI2Sys.out.println("[" + e + "] non ci sono elementi!");
            } //catch(QueueEmptyException)
        } while(sIn.ttype != StreamTokenizer.TT_EOF);
    } //[m] main

} //{c} RBuf
