package FI2.Temi;
/**{p}
 * -- FI2.Temi --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Temi d'esame
 */

/**{f}
 * SBSPQ.java
 * Implementazione con SBSTree  
 * di coda a priorita` -
 * prova scritta del 17/9/2001
 *
 * @version 1.00 01/01/12
 * @author M.Moro DEI UPD
 */

import java.io.*;
import java.util.NoSuchElementException;
import FI2.Set.Item;
import FI2.Set.Position;
import FI2.Set.Container;
import FI2.Util.IntegerComparator;
import FI2.Util.Comparator;
import FI2.Set.ObjectIterator;
import FI2.Linear.InspectablePriorityQueue;
import FI2.Linear.PriorityQueue;
import FI2.Linear.List;
import FI2.Linear.ArrayList;
import FI2.Dictionary.OrderedDictionary;
import FI2.Exceptions.EmptyContainerException;
import FI2.Exceptions.InvalidKeyException;

/**{c}
 * coda a priorita` realizzata mediante un 
 * albero binario di ricerca
 */
public class SBSPQ extends SBSTree
  implements PriorityQueue 
{
    protected Comparator comp;
      // comparatore, per newContainer

    /**[c]
     * costruttore base con comparatore per i confronti
     * @param c  il comparatore
     */
    public SBSPQ(Comparator c) 
    { 
        super(c);
        comp = c;
    } //[c] SBSPQ

    /**[m]
     * ATTENZIONE: elements() ritorna un iteratore per Item e non
     * per gli elementi associati, che sono ottenibili
     * applicando element()
     * @see InspectableContainer
     */

    /**[m]
     * @see Container
     */
    public Container newContainer() 
    { return new SBSPQ(comp); }
    
    // metodo di supporto
    /**[m]
     * determina la posizione del nodo minimo
     * @return posizione del nodo con chiave minima
     * @exception EmptyContainerException
     * se coda vuota
     */
    private Position minPos() 
      throws EmptyContainerException 
    {
        if (isEmpty())
            throw new EmptyContainerException(
            "La coda e` vuota!");
        Position curr = T.root();
        Position next;
        while (T.isInternal(next = T.leftChild(curr)))
            curr = next;
        return curr;
    } //[m] minPos
        
    /**[m]
     * O(1)
     * @see InspectablePriorityQueue
     */
    public Object minElement () 
      throws EmptyContainerException 
    {
        return ((Item)minPos().element()).element();
    } //[m] minElement

    /**[m]
     * O(1)
     * @see InspectablePriorityQueue
     */
    public Object minKey () 
      throws EmptyContainerException 
    {
        return ((Item)minPos().element()).key();
    } //[m] minKey

    /**[m]
     * O(n)
     * @see PriorityQueue
     */
     public void insertItem (Object k, Object e) 
       throws InvalidKeyException 
    {
        super.insertItem(k, e);
    } //[m] insertItem

    /**[m]
     * O(1)
     * @see PriorityQueue
     */
    public Object removeMin () 
      throws EmptyContainerException 
    {
        if (isEmpty())
            throw new EmptyContainerException(
            "La coda e` vuota!");
      return removeElement(minKey());
    } //[m] removeMin

    /**[m] !!
     * conversione a stringa
     * @return stringa con il contenuto
     */
    public String toString()
    {
        return "SBSPQ [size=" + size() +
          "]:\n" + super.toString();
    } //[m] toString()

    /**[m][s] !!
     * ordinamento di una sequenza di elementi-chiave
     * mediante coda a priorita` -
     * gli elementi della coda debbono essere anche
     * interpetabili come chiavi ordinabili
     * usando il comparatore fornito
     * @param s  sequenza da ordinare
     * @param c  comparatore per i confronti
     */
    public static List insSort(List uns, Comparator comp)
    {
        Object e;
        PriorityQueue pq = 
          new SBSPQ(comp);
        List ret = (List)uns.newContainer();

        while(!uns.isEmpty())
        {
            e = uns.removeFirst();
            pq.insertItem(e, e);
        } //while

        while(!pq.isEmpty())
        {
            e = pq.removeMin();
            ret.insertLast(e);
        } //while
        return ret;
    } //[m][s] insSort

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  numero dati
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        Comparator comp = new IntegerComparator();
        int num = Integer.parseInt(args[0]);
        List uns = new ArrayList();
        
        for (int i=0; i<num; i++)
            uns.insertLast(new Integer(
              (int)(Math.random()*200.0-100.0)));
        
        System.out.println("La lista originaria era:");
        System.out.println(uns.toString());
        List ret = insSort(uns, comp);
        System.out.println("La lista ordinata e':");
        System.out.println(ret.toString());
    } //[m][s] main

} //{c} SBSPQ
