package FI2.Temi;
/**{p}
 * -- FI2.Temi --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Temi d'esame
 */

/**{f}
 * StackTab.java
 * 2^ prova intermedia 2000/2001 -
 * Tema a -
 * Dizionario non ordinato su stack
 *
 * @version 1.00 01/01/08
 * @author M.Moro DEI UPD
 */

import java.io.*;
import java.util.NoSuchElementException;
import FI2.Set.Item;
import FI2.Set.InspectableContainer;
import FI2.Set.InspectableKeyBasedContainer;
import FI2.Set.ObjectIterator;
import FI2.Util.Equality;
import FI2.Util.FI2Sys;
import FI2.Linear.Vector;
import FI2.Linear.Stack;
import FI2.Linear.ArrayVector;
import FI2.Exceptions.StackFullException;
import FI2.Exceptions.StackEmptyException;
import FI2.Exceptions.InvalidElementException;

/**{c}
 * Dizionario read-only non ordinato basato su stack
 * realizzato con array
 * @see Stack
 */
public class StackTab extends ArrayStack implements 
InspectableTable
{

    private Equality comp;  // comparatore

    /**[c]
     * costruttore base
     * @param cap  capacita` massima dell'array
     * @param eq  comparatore
     */
    public StackTab(int cap, Equality eq)
    {
        super(cap);
        comp = eq;
    } //[c] StackTab
    
    /**[m]
     * inserimento di una coppia chiave-dato
     * @param key  chiave
     * @param data  dato associato
     */
    public void push(Object key, Object data)
    {
        try {
            push(new Item(key, data));
        }
        catch(StackFullException e)
        {
            FI2Sys.out.println("Array pieno!");
            System.exit(0);
        }
    } //[m] push

    /**[m]
     * estrazione di una coppia chiave-dato
     * @param dummy  parametro non usato
     * @return un Item con chiave-dato
     */
    public Item pop(int dummy)
    {
        try {
            return (Item)pop();
        }
        catch(StackEmptyException e)
        {
            FI2Sys.out.println("Array vuoto!");
            System.exit(0);
            return null;
        }
    } //[m] pop

    /**[m]
     * @see InspectableContainer
     */
    public ObjectIterator elements()  
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per scorrere i dati -
         * ritorna un iteratore inutile
         */
        class ElemsIter implements ObjectIterator
        {
             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return false;
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                throw new NoSuchElementException("Iteratore nullo!");
            } //[m] nextElement

        } //{c}{l} ElemsIter

        return new ElemsIter();
    } //[m] elements

    /**[m]
     * @see InspectableKeyBasedContainer
     */
    public ObjectIterator keys()  
    {
        Vector sup = new ArrayVector();
          // vettore di supporto
        for (int i=0; i<size(); i++)
            sup.insertAtRank(sup.size(), ((Item)S[i]).key());
              // copia delle chiavi
        return sup.elements();
    } //[m] keys

    /**[m]
     * @see InspectableTable
     */
    public Object find(Object key)
    {
        int i;
        
        for(i=0; i<size(); i++)
            if (comp.isEqualTo(((Item)S[i]).key(), key))
                return ((Item)S[i]).element();
        // non trovato
        return NOT_FOUND;
    }
    
    /**[m]
     * @see InspectableTable
     */
    public Object firstKey()
    {
        try
        {
            return ((Item)(top())).key(); 
        }
        catch(StackEmptyException e)
        {
            return NOT_FOUND;
        }
    }  

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        /**{c}{l}
         * classe locale che rappresenta il comparatore 
         * interno tra Float
         */
        class FloatEqual implements Equality
        {
            /**[m]
             * @see Equality
             */
            public boolean isEqualTo(Object x1, Object x2)
              throws InvalidElementException 
            {
                if ((x1==null) || (x2==null)) throw
                    new InvalidElementException(
                   "Comparatore applicato ad oggetto nullo");
                return ((Float)x1).floatValue() ==
                  ((Float)x2).floatValue();  
                // si puo' usare anche Float.equals  
            } //[m] isEqualTo
        
            /**[m]
             * @see Equality
             */
            public boolean isComparable(Object x)
            {
                if (x==null)
                    return false;
                return true;
            } //[m] isComparable

        } //{c}{l} FloatEqual
        
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "StackTab: Battere coppie f.p.-dato (fine=Ctrl-Z)");
        }
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(new
          InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        StackTab mySt = new StackTab(100, new FloatEqual());

        sIn.parseNumbers();
        try
        {
            Float next;
            int pos = 0;

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Float((float)sIn.nval);
                    if (sIn.nextToken() == StreamTokenizer.TT_EOF)
                        // fine file, esce
                        break;
                    mySt.push(next, sIn.sval);
                    FI2Sys.out.println("Prima chiave: "+
                      mySt.firstKey());
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while

        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch(EOF)
        catch (StackFullException e)
        {
            FI2Sys.out.println("[" + e + "] troppi numeri!");
            // termina il ciclo
        } //catch(StackFullException)
        // lo stack ha eseguito il push dei
        // numeri forniti nell'ordine

        // non richiesti
        FI2Sys.out.println("StackTab creato");
        FI2Sys.out.println(mySt);

        FI2Sys.out.println("size=" + mySt.size());
        FI2Sys.out.println("firstKey=" +
          mySt.firstKey());
        FI2Sys.out.println("top=" +
          ((Float)(((Item)mySt.top()).key())).floatValue());
        // fine non richiesti
        
        FI2Sys.out.println("estratto:" +
          mySt.pop(0));
        ObjectIterator iter = mySt.keys();
        while(iter.hasNext())
            FI2Sys.out.println("Chiave "+(Float)iter.nextObject());
            
        Object ret;
        if ((ret = mySt.find(new Float(3.5))) == NOT_FOUND)
            FI2Sys.out.println("3.5 non presente!");
        else
            FI2Sys.out.println("3.5 e` chiave dell'elemento:"+ret);
    } //[m][s] main

} //{c} StackTab
