package FI2.Temi;
/**{p}
 * -- FI2.Temi --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Temi d'esame
 */

/**{f}
 * TrieDict.java
 * Dizionario con trie *
 * tema febbraio 2000 2
 *
 * @version 1.00 00/02/06
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Tree.*;
import FI2.Dictionary.*;
import FI2.Set.*;
import FI2.Util.FI2Sys;


/**{c}
 * Utilizza la tecnica dell'albero figlio-fratello
 * per rappresentare un trie che funge da dizionario
 * di stringhe
 */
public class TrieDict
{
    private ExpandBinaryTree dt;
      // albero binario di supporto 
    
    /**[c]
     * costruttore base
     */
    public TrieDict()
    {
        dt = new LinkedBinaryTree();
        dt.expandExternal(dt.root());
          // per avere subito due nodi sentinella
        dt.replaceElement(dt.root(), new TrieItem(false, '?', 0));
    } //[c] TrieDict

    /**[m]
     * routine di ricerca ricorsiva
     * @param v  nodo di partenza
     * @param s  stringa da cercare
     * @return posizione del piu` lungo prefisso
     */
    private Position find(Position v, String s)
    {
        char first = s.charAt(0);
        Position curr = dt.leftChild(v);  // primo figlio
        while (curr.element() != null)
        {
            if (((TrieItem)curr.element()).c == first)
                // un passo
                if (s.length() == 1)
                    return curr;  // stringa trovata
                else
                    return find(curr, s.substring(1)); 
                      // passo successivo
            else
                curr = dt.rightChild(curr);
                  // prossimo figlio
        } // while
        return v;
    } //[m] find

    /**[m]
     * @see Dictionary
     */
    public String findElement (String s) 
    {
        Position v = find(dt.root(), s);
          // cerca pi lungo prefisso memorizzato
        TrieItem ti = ((TrieItem)v.element());
        if (ti.end && ti.len == s.length())
            // trovata
            return s;
        return (String)Dictionary.NO_SUCH_KEY;
    } //[m] findElement

    /**[m]
     * @see Dictionary
     */
    public void insertItem (String s) 
    {
        Position v = find(dt.root(), s);
          // cerca pi lungo prefisso memorizzato
        TrieItem ti = ((TrieItem)v.element());
        if (ti.len == s.length())
        {
            // trovata, gi presente
            if (!ti.end)
                ti.end = true;
            return;
        }
        // v rappresenta il percorso pi lungo comune
        // cerca l'ultimo figlio
        Position leftc = dt.leftChild(v);
        if (leftc.element() == null)
        {
            // c'e` da aggiungere un primo figlio
            for (int i=ti.len+1; i <= s.length(); i++)
            {
                dt.expandExternal(leftc);
                dt.replaceElement(leftc, new TrieItem((s.length() == i),
                  s.charAt(i-1), i));
                    // aggiunge nodo, finale se stringa finita
                leftc = dt.leftChild(leftc);
            }
            return;
        }
        // c'e` almeno un figlio
        Position rightc = leftc;
        do
            rightc = dt.rightChild(rightc);
        while (rightc.element() != null);
        // trovato ultimo figlio
        dt.expandExternal(rightc);
        dt.replaceElement(rightc, new TrieItem((s.length() == ti.len+1),
          s.charAt(ti.len), ti.len+1));
        // forse c'e` da aggiungere a sinistra
        leftc = dt.leftChild(rightc);
        for (int i=ti.len+2; i <= s.length(); i++)
        {
            dt.expandExternal(leftc);
            dt.replaceElement(leftc, new TrieItem((s.length() == i),
              s.charAt(i-1), i));
                // aggiunge nodo, finale se stringa finita
            leftc = dt.leftChild(leftc);
         }
    } //[m] insertItem

    private void getElement(ArraySet e, Position r, String p)
    {
        Position curr = dt.leftChild(r);
          // primo figlio a sinistra
        while (curr.element() != null)
        {
            TrieItem ti = ((TrieItem)curr.element());
            String s = p+ti.c;
            if (ti.end)
                // stringa appartenente
                e.insertElement(s);
            getElement(e, curr, s);
            curr = dt.rightChild(curr);
        }
    }

    /**[m]
     * @see Dictionary
     */
    public ObjectIterator elements()
    {
        ArraySet tmp = new ArraySet();

        getElement(tmp, dt.root(), "");

        return tmp.elements();

    } //[m] elements

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  nome file (opzionale)
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;
        String str;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "TrieDict, battere una stringa per riga");
        }
        else
            in = new FileInputStream(args[0]);

        BufferedReader re = new BufferedReader(new
          InputStreamReader(in));
        TrieDict td = new TrieDict();
        while ((str = re.readLine()) != null)
        {
            if (str.equals(""))
                continue;
            td.insertItem(str);
            FI2Sys.out.println("\nVisita parentetica");
            BinTreeBasicAlgo.parentheticRepresentation(td.dt,
              td.dt.root());
        }
        FI2Sys.out.println("\nStringhe memorizzate");
        ObjectIterator en = td.elements();
        while (en.hasNext())
        {
            FI2Sys.out.println((String)en.nextObject());
        }
    } //[m][s] main

} //{c} TrieDict

