package FI2.Tree;
/**{p}
 * -- FI2.Tree --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Alberi
 */

/**{f}
 * BTNode.java
 * Nodo-posizionatore per un albero binario
 *
 * @version 2.00 00/09/23
 * @author M.Moro DEI UPD
 */

import FI2.Exceptions.InvalidPositionException;
import FI2.Set.Position;
import FI2.Set.Container;


/**{c}
 * nodo di albero binario
 * come posizionatore -
 * i metodi non pubblici assumono il nodo corretto
 * e non fanno controlli
 * @see Position
 */
class BTNode implements Position
{
    private Container cont;  // contenitore associato
    private Object element;  // elemento associato
    private BTNode left;  // figlio di sinistra
    private BTNode right;  // figlio di sinistra
    private BTNode parent;  // nodo padre


    /**[c]
     * costruttore di default, posizionatore illegale
     */
    public BTNode()
    {
        this(null, null, null, null, null);
    } //[c] BTNode

    /**[c]
     * costruttore con oggetto associato -
     * non utilizza setElement, setParent, ecc. per bypassare
     * il controllo su container
     * @param o  l'oggetto associato
     * @param p  nodo padre
     * @param l  figlio di sinistra
     * @param r  figlio di destra
     * @param c  albero contenitore
     */
    protected BTNode(Object o, BTNode p, 
      BTNode l, BTNode r, Container c)
    {
        element = o;
        parent = p;
        left = l;
        right = r;
        cont = c;
    } //[c] BTNode

    /**[m]
     * per completezza, esistendo il costruttore nullo,
     * un posizionatore puo` effettivamente essere illegale
     * @see Position
     */
    public Object element() throws InvalidPositionException
    {
        if (cont==null)
            throw new InvalidPositionException(
              "In BTNode.element");
        return element;
    } //[m] element

    /**[m]
     * per completezza, esistendo il costruttore nullo,
     * un posizionatore puo` effettivamente essere illegale
     * @see Position
     */
    public Container container() 
      throws InvalidPositionException
    {
        if (cont==null)
            throw new InvalidPositionException(
              "On BTNode.container");
        return cont;
    } //[m] container

    /**[m]
     * ritorna l'elemento 'indicato' dal posizionatore
     * assunto corretto
     * @return elemento associato
     */
    protected Object getElement() 
    { return element; }

    /**[m]
     * metodo di modifica dell'oggetto
     * @param newElem l'oggetto da assegnare
     */
    protected void setElement(Object newElem)
    { element = newElem; }

    /**[m]
     * metodo di modifica del contenitore
     * @param newCont il contenitore da assegnare
     */
    protected void setContainer(Container newCont)
    { cont = newCont; }

    /**[m]
     * rende non piu` valido il posizionatore
     */
    protected void setInvalid()
    { cont = null; }

    /**[m]
     * figlio di sinistra -
     * si assume il posizionatore legale
     * @return il nodo figlio di sinistra
     */
    protected BTNode getLeft()
    {
        return left;
    } //[m] getLeft

    /**[m]
     * aggiornamento del figlio di sinistra -
     * si assume il posizionatore legale
     * @param v  il il nuovo nodo figlio di sinistra
     */
    protected void setLeft(Position v)
    {
        left = (BTNode)v;
    } //[m] setLeft

    /**[m]
     * figlio di destra -
     * si assume il posizionatore legale
     * @return il nodo figlio di destra
     */
    protected BTNode getRight()
    {
        return right;
    } //[m] getRight

    /**[m]
     * aggiornamento del figlio di destra -
     * si assume il posizionatore legale
     * @param v  il il nuovo nodo figlio di destra
     */
    protected void setRight(Position v)
    {
        right = (BTNode)v;
    } //[m] setRight

    /**[m]
     * nodo padre -
     * si assume il posizionatore legale
     * @return il nodo padre
     */
    protected BTNode getParent()
    {
        return parent;
    } //[m] getParent

    /**[m]
     * aggiornamento del nodo padre -
     * si assume il posizionatore legale
     * @param p  il il nuovo nodo padre
     */
    protected void setParent(BTNode p)
    {
        parent = p;
    } //[m] setParent

    /**[m] !!
     * conversione a stringa -
     * produce la stringa associata all'elemento
     * @return stringa contenuto del nodo
     */
    public String toString()
    {
        return "BTNode cont.size=" + cont.size() +
          " elem=" + element;
    } //[m] toString

} //{c} BTNode

