package FI2.Tree;
/**{p}
 * -- FI2.Tree --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Alberi
 */

/**{f}
 * BinTreeBasicAlgo.java
 * metodi base sugli alberi binari
 *
 * @version 3.00 00/09/22
 * @author M.Moro DEI UPD
 */

import FI2.Set.Position;
import FI2.Util.FI2Sys;

/**{c}
 * alcuni metodi (statici) base su alberi binari
 * @see InspectableBinaryTree
 */
public class BinTreeBasicAlgo extends TreeBasicAlgo
{
    /**[m][s]
     * altezza di un sottoalbero binario 
     * valutata ricorsivamente
     * @param T  albero binario
     * @param v  posizionatore al nodo radice del sottoalbero
     */
    public static int height2(InspectableBinaryTree T, Position v)
    {
        if (T.isExternal(v))
            return 0;

        int h = 0;
        h = Math.max(h, height2(T, T.leftChild(v)));
        h = Math.max(h, height2(T, T.rightChild(v)));
        return 1+h;
    } //[m][s] height2

    /**[m][s]
     * visita ricorsiva in preordine del sottoalbero 
     * binario con radice v
     * @param T  albero binario
     * @param v  posizionatore al nodo radice del sottoalbero
     */
    public static void 
      preorderPrint(InspectableBinaryTree T, Position v)
    {
        if (v.element() != null)
            FI2Sys.out.println(v.element());
              // out la radice
        else
            FI2Sys.out.println("Nodo nullo");
        if (T.isInternal(v))
        {
            preorderPrint(T, T.leftChild(v));
            preorderPrint(T, T.rightChild(v));
        }
    } //[m][s] preorderPrint

    /**[m][s]
     * visita ricorsiva in preordine del sottoalbero 
     * binario con radice v e 
     * aggiunta delle parentesi di livello
     * @param T  albero binario
     * @param v  posizionatore al nodo radice del sottoalbero
     */
    public static void 
      parentheticRepresentation(InspectableBinaryTree T, Position v)
    {
        if (v.element() != null)
            FI2Sys.out.println(v.element());
              // out nodo
        else
            FI2Sys.out.println("Nodo nullo");
        if (T.isInternal(v))
        {
            FI2Sys.out.println("(");
            parentheticRepresentation(T, T.leftChild(v));
            parentheticRepresentation(T, T.rightChild(v));
            FI2Sys.out.println(")");
        } //if
    } //[m][s] parentheticRepresentation

    /**[m][s] !!
     * visita ricorsiva in preordine del sottoalbero 
     * binario con radice v e 
     * aggiunta delle parentesi di livello
     * @param T  albero binario
     * @param v  posizionatore al nodo radice del sottoalbero
     * @return la stringa prodotta
     */
    public static String toString(InspectableBinaryTree T, Position v)
    {
        String str;

        if (v.element() != null)
            str = " <" + v.element() + ">\n";
              // out nodo
        else
            str = " <Nodo nullo>\n";
        if (T.isInternal(v))
            str += "(\n" + 
            toString(T, T.leftChild(v)) +
            toString(T, T.rightChild(v)) +
            ")\n";
        return str;
    } //[m][s] toString

    /**[m][s]
     * visita ricorsiva in postordine del sottoalbero 
     * binario con radice v
     * @param T  albero binario
     * @param v  posizionatore al nodo radice del sottoalbero
     */
    public static void 
      postorderPrint(InspectableBinaryTree T, Position v)
    {
        if (T.isInternal(v))
        {
            postorderPrint(T, T.leftChild(v));
            postorderPrint(T, T.rightChild(v));
        }
        if (v.element() != null)
            FI2Sys.out.println(v.element());
              // out la radice
        else
            FI2Sys.out.println("Nodo nullo");
    } //[m][s] postorderPrint

    /**[m][s]
     * visita ricorsiva in ordine interno del sottoalbero 
     * binario con radice v
     * @param T  albero binario
     * @param v  posizionatore al nodo radice del sottoalbero
     */
    public static void 
      inorderPrint(InspectableBinaryTree T, Position v)
    {
        if (T.isInternal(v))
            inorderPrint(T, T.leftChild(v));
        if (v.element() != null)
            FI2Sys.out.println(v.element());
              // out la radice
        else
            FI2Sys.out.println("Nodo nullo");
        if (T.isInternal(v))
            inorderPrint(T, T.rightChild(v));
    } //[m][s] inorderPrint

    /**[m][s]
     * visita ricorsiva di Eulero del sottoalbero 
     * binario con radice v
     * @param T  albero binario
     * @param v  posizionatore al nodo radice del sottoalbero
     */
    public static void 
      eulerTour(InspectableBinaryTree T, Position v)
    {
        if (v.element() != null)
            FI2Sys.out.println("[" + v.element());
        else
            FI2Sys.out.println("[NULLO");
          // visita nodo da sinistra
        if (T.isInternal(v))
        {
            FI2Sys.out.println(" ");
            eulerTour(T, T.leftChild(v));
        } //if
        if (v.element() != null)
            FI2Sys.out.println("(" + v.element() + ")");
        else
            FI2Sys.out.println("(NULLO)");
          // visita nodo da sotto
        if (T.isInternal(v))
        {
            FI2Sys.out.println(" ");
            eulerTour(T, T.rightChild(v));
        } //if
        if (v.element() != null)
            FI2Sys.out.println(v.element() + "]");
        else
            FI2Sys.out.println("NULLO]");
          // visita nodo da destra
    } //[m][s] eulerTour

    /**[m][s]
     * viene prodotta una copia del sottoalbero v1 di T1;
     * il nodo esterno v2 di T2 diventa la radice della
     * copia prodotta; T1 non viene modificato
     * @param T1  albero binario sorgente
     * @param T2  albero binario destinazione
     * @param v1  posizionatore ad un nodo radice di 
     * sottoalbero di T1
     * @param v2  posizionatore ad un nodo foglia di T2
     */
    public static void 
      cloneSubTree(InspectableBinaryTree T1, ExpandBinaryTree T2, 
      Position v1, Position v2)
    {
        if (T1.isInternal(v1))
        {
            T2.expandExternal(v2);
              // crea nuovi figli della foglia
            cloneSubTree(T1, T2, T1.leftChild(v1), 
              T2.leftChild(v2));
            cloneSubTree(T1, T2, T1.rightChild(v1), 
              T2.rightChild(v2));
        }
        T2.replaceElement(v2, v1.element());
    } //[m][s] cloneSubTree

} //{c} BinTreeBasicAlgo
