package FI2.Tree;
/**{p}
 * -- FI2.Tree --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Alberi
 */

/**{f}
 * Calculator.java
 * Interprete per calcolatore elementare floating point
 *
 * @version 2.00 98/09/20
 * @version 2.01 98/12/02 qualche commento in piu`
 * @author M.Moro DEI UPD
 */

import FI2.Linear.Stack;
import FI2.Linear.*;
import FI2.Set.*;
import FI2.Util.*;
import java.io.*;
import java.util.*;

/**{c}
 * classe di supporto per gestione di token -
 * associa ad ogni lessema (elemento lessicale singolo:
 * simbolo, operatore, sequenza di spazi, ecc.)
 * una codifica di tipo cosi` da consentire una piu`
 * facile gestione da parte del parser 
 * (analizzatore sintattico)
 */
class Token
{
    static final int NULTOK = 0;
    static final int EOFTOK = NULTOK+1;  // eof
    static final int ERRTOK = EOFTOK+1;  // errore lessicale
    static final int OPNDTOK = ERRTOK+1;  // operando
    static final int SUMTOK = OPNDTOK+1;  // operatore +
    static final int DIFFTOK = SUMTOK+1;  // operatore -
    static final int MULTOK = DIFFTOK+1;  // operatore *
    static final int DIVTOK = MULTOK+1;  // operatore /
    static final int LEFTBTOK = DIVTOK+1;  // operatore (
    static final int RIGHTBTOK = LEFTBTOK+1;
      // operatore )
    static final int EOETOK = RIGHTBTOK+1;
      // ; fine espressione 
    static final int SPACETOK = EOETOK+1;
      // separatore

    Object value = null;
        // valore del token
    int type = NULTOK;
        // tipo del token

    /**[c]
     * costruttore per un lessema di un singolo carattere
     * (operatore, spazio)
     * @param c  carattere letto
     */
    Token(char c)
    {
        value = new Character(c);
        switch(c)
        {
          case '+':
            type = SUMTOK;
            break;
          case '-':
            type = DIFFTOK;
            break;
          case '*':
            type = MULTOK;
            break;
          case '/':
            type = DIVTOK;
            break;
          case '(':
            type = LEFTBTOK;
            break;
          case ')':
            type = RIGHTBTOK;
            break;
          case ';':
            type = EOETOK;
            break;
          case ' ':
          case '\n':
          case '\r':
          case '\t':
            type = SPACETOK;
            break;
          default:
            type = ERRTOK;
        } //switch
    } //[m] Token(char c)

    /**[c]
     * costruttore per un lessema costituito da un numero 
     * floating point
     * @param d  valore floating point letto
     */
    Token(double d)
    {
        value = new Double(d);
        type = OPNDTOK;
    } //[m] Token(double d)

    /**[c]
     * costruttore di default per EOF
     */
    Token()
    {
        value = "<EOF>";
        type = EOFTOK;
    }

    /**[c]
     * costruttore per un lessema costituito da una stringa
     * (simbolo)
     * @param s  stringa letta
     */
    Token(String s)
    {
        value = s;
        type = ERRTOK;
    }

    /**[m]
     * conversione a stringa del valore
     * @return stringa di conversione
     */
    public String toString()
    {
        return value.toString();
    }

    /**[m]
     * il valore espresso dal Token
     * @return valore associato
     */
    Object val()
    {
        return value;
    }

} //{c} Token


/**{c}
 * interprete per un calcolatore elementare:
 * costruisce l'albero astratto, lo visita in postordine
 * per ricavare la forma polacca inversa e utilizza una
 * macchina a stack come interprete
 */
public class Calculator
{

    private Token curTok;
    private StreamTokenizer sIn;
    private Stack myStack;

    public Calculator(StreamTokenizer input)
    {
        sIn = input;
        // sistema definizione dei caratteri
        sIn.parseNumbers();
            // tratta i numeri
        sIn.commentChar('#');
            // un commento e` # fino a fine riga
        sIn.ordinaryChar('/');
            // non commento ma operatore diviso
        sIn.ordinaryChar('-');
            // i numeri sono senza segno
        getToken();
            // acquisisce primo token
        myStack = new ArrayStack(100); 
    }

    /**[m]
     * acquisizione di un token -
     * vengono scanditi i caratteri in base alle semplici 
     * regole lessicali, in parte realizzate direttamente
     * dalla funzione nextToken fornita da StreamTokenizer -
     * il token viene salvato nella variabile d'istanza curTok
     */
    private void getToken()
    {
        do
        {
            try
            {
                switch(sIn.nextToken())
                {
                  case StreamTokenizer.TT_EOL:
                    curTok = new Token(' ');
                    break;
                  case StreamTokenizer.TT_EOF:
                    curTok = new Token();
                    break;
                  case StreamTokenizer.TT_NUMBER:
                    curTok = new Token(sIn.nval);
                    break;
                  case StreamTokenizer.TT_WORD:
                    curTok = new Token(sIn.sval);
                  default:
                    // carattere normale
                    curTok = new Token((char)sIn.ttype);
                } //switch
            } //try
            catch (EOFException e)
            {
                // termina il ciclo
            } //catch
            catch (IOException e)
            {
                curTok = new Token("EOF");
                return;
            } //catch
            if (curTok.type == Token.ERRTOK)
                FI2Sys.out.println("Errore lessicale <" +
                  curTok + ">");
        } while(curTok.type == Token.ERRTOK ||
          curTok.type == Token.SPACETOK);
    }

    /**[m]
     * metodo principale che acquisisce un numero 
     * arbitrario di espressioni da calcolare
     */
    public void expressions()
    {
        EvaluateDoubleExpressionTour ev = 
          new EvaluateDoubleExpressionTour();
            // attraversamento dell'albero astratto 
            // per il calcolo
        BracketedExpressionTour br = 
          new BracketedExpressionTour();
            // attraversamento dell'albero per la 
            // rappresentazione a parentesi
        do
        {
            ExpandBinaryTree expr = expression();
              // albero astratto completo dell'espressione
            if (expr == null)
            {
                FI2Sys.out.println();
                FI2Sys.out.println("Espressione nulla!");
            } //if
            else
            {
                FI2Sys.out.println("Albero double creato:\n" + 
                  expr);
                ev.execute(expr);  // calcolo
                br.execute(expr);  // stampa con parentesi
            }
            if (curTok.type == Token.EOETOK)
                // una nuova espressione
                getToken();  // legge il token di avvio
            else if (curTok.type != Token.EOFTOK)
                FI2Sys.out.println("Errore di sintassi <" +
                      curTok + ">");
        } while (curTok.type != Token.EOFTOK);
    }

    /**[m]
     * tratta (ricorsivamente) una espressione, costruendone
     * il (sotto)albero astratto -
     * corrisponde alla produzione della grammatica 
     * BNL estesa:
     * Expr -> Term [ ('+' | '-') Term ]*
     * @return albero binario astratto ricavato
     */
    public ExpandBinaryTree expression()
    {
        ExpandBinaryTree expr = term();
          // primo termine dell'espressione
        while (curTok.type == Token.SUMTOK ||
          curTok.type == Token.DIFFTOK)
        {
            // c'e` l'operatore additivo, segue il secondo
            // termine dell'espressione
            Token curOp = curTok;
            getToken();
                // salta operatore
            ExpandBinaryTree snd = term();
              // secondo termine
            
            // segue la costruzione dell'albero a partire
            // dai due sottoalberi componenti relativi
            // ai due termini (costruzione bottom-up)
            ExpandBinaryTree newExpr = new LinkedBinaryTree();
              // nuovo albero complessivo
            Position root;
            newExpr.expandExternal(root = newExpr.root());
            newExpr.replaceElement(root, 
              curOp.type == Token.SUMTOK ? 
              (Object) new DoubleAdditionOperator() :
              (Object) new DoubleSubtractionOperator());
              // imposta l'operatore sulla radice
            newExpr.link(newExpr.leftChild(root), expr);
            newExpr.link(newExpr.rightChild(root), snd);
              // il nuovo albero ha come radice l'operatore
              // additivo e come figli di primo livello i due
              // sottoalberi relativi ai termini
            expr = newExpr;
              // l'albero ottenuto diventa l'albero
              // del primo termine corrente
        } //while
        return expr;
    } //[m] expression

    /**[m]
     * tratta (ricorsivamente) un termine, costruendone
     * il (sotto)albero astratto -
     * corrisponde alla produzione della grammatica 
     * BNL estesa:
     * Term -> Factor [ ('*' | '/') Factor ]*
     * @return albero binario astratto ricavato
     */
    public ExpandBinaryTree term()
    {
        ExpandBinaryTree trm = factor();
          // primo fattore del termine
        while (curTok.type == Token.MULTOK ||
          curTok.type == Token.DIVTOK)
        {
            // c'e` l'operatore moltiplicativo, 
            // segue il secondo fattore del termine
            Token curOp = curTok;
            getToken();
                // salta operatore
            ExpandBinaryTree snd = factor();
              // secondo fattore

            // segue la costruzione dell'albero a partire
            // dai due sottoalberi componenti relativi
            // ai due fattori (costruzione bottom-up)
            ExpandBinaryTree newTrm = new LinkedBinaryTree();
              // nuovo albero complessivo
            Position root;
            newTrm.expandExternal(root = newTrm.root());
            newTrm.replaceElement(root, 
              curOp.type == Token.MULTOK ? 
              (Object) new DoubleMultiplicationOperator() :
              (Object) new DoubleDivisionOperator());
              // imposta l'operatore sulla radice
            newTrm.link(newTrm.leftChild(root), trm);
            newTrm.link(newTrm.rightChild(root), snd);
              // il nuovo albero ha come radice l'operatore
              // moltiplicativo e come figli di primo livello 
              // i due sottoalberi relativi ai fattori
            trm = newTrm;
              // l'albero ottenuto diventa l'albero
              // del primo fattore corrente
        } //while
        return trm;
    } //[m] term

    /**[m]
     * tratta (ricorsivamente) un fattore, costruendone
     * il (sotto)albero astratto -
     * corrisponde alla produzione della grammatica 
     * BNL estesa:
     * Expr -> Term [ ('+' | '-') Term ]*
     * @return albero binario astratto ricavato
     */
    public ExpandBinaryTree factor()
    {
        while(true)
        {
            switch(curTok.type)
            {
              case Token.DIFFTOK:
                // meno unario
                getToken();
                    // salta operatore
                ExpandBinaryTree snd = factor();
                  // il fattore su cui si applica 
                  // il meno unario
                ExpandBinaryTree fac = new LinkedBinaryTree();
                Position root;
                fac.expandExternal(root = fac.root());
                fac.replaceElement(root, 
                  new DoubleOppositeOperator());
                  // imposta l'operatore sulla radice
                fac.link(fac.rightChild(root), snd);
                  // il nuovo albero ha come radice 
                  // l'operatore unario e come figlio di 
                  // destra di primo livello il 
                  // sottoalbero relativo al fattore su cui
                  // l'operatore agisce 
                  // non ha il figlio di sinistra
                return fac;
              case Token.LEFTBTOK:
                // sottoespressione in parentesi
                getToken();
                    // salta parentesi
                ExpandBinaryTree expr = expression();
                if (curTok.type != Token.RIGHTBTOK)
                    FI2Sys.out.println(
                      "Errore di sintassi <" +
                      curTok + ">");
                else
                    getToken();
                        // salta parentesi
                // le parentesi servono solo per l'ordine
                // di calcolo: l'albero ne e` privo
                return expr;
              case Token.OPNDTOK:
                // operando
                ExpandBinaryTree opnd = new LinkedBinaryTree();
                opnd.replaceElement(opnd.root(), 
                  new DoubleVariableInfo(
                  ((Double)curTok.val()).doubleValue()));
                  // albero con la sola radice con l'operando
                getToken();
                  // salta operando
                return opnd;
                    // foglia
              default:
                FI2Sys.out.println("Errore di sintassi <" +
                  curTok + ">");
                return null;
            } //switch
        } //while
    } //[m] factor

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "Calcolatore double, " + 
  "battere espressioni separate da ; (fine=Ctrl-Z):");
        } //if
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(
          new InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        Calculator calc = new Calculator(sIn);

        calc.expressions();
    } //[m][s] main

} //{c} Calculator
