package FI2.Tree;
/**{p}
 * -- FI2.Tree --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Alberi
 */

/**{f}
 * ExpandBinaryTree.java
 * albero binario con espansioni -
 * derivata da BinaryTree della v 1.x
 *
 * @version 3.00 00/09/22
 * @author M.Moro DEI UPD
 */

import FI2.Set.Position;
import FI2.Set.PositionalContainer;
import FI2.Exceptions.InvalidPositionException;
import FI2.Exceptions.InvalidContainerException;
import FI2.Exceptions.BoundaryViolationException;

/**{i}
 * albero binario  -
 * estende l'albero binario per definire
 * come metodo di modifica l'espansione locale -
 * raggruppa alcuni altri metodi di modifica
 * collegati
 * @see BinaryTree
 */
public interface ExpandBinaryTree 
  extends BinaryTree
{
    /**[m][a]
     * il nodo foglia fornito diviene radice di due nuove
     * foglie dell'albero (vuote)
     * @param v  nodo da espandere
     * @exception InvalidPositionException
     * posizionatore illegale (anche se v non foglia)
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    void expandExternal(Position vExt)
      throws 
      InvalidPositionException, InvalidContainerException;

    /**[m][a]
     * il nodo foglia fornito viene rimosso assieme al padre;
     * il fratello sostituisce il padre (come figlio 
     * sinistro/destro del nonno)
     * @param v  nodo da rimuovere
     * @return l'oggetto associato al padre della
     * foglia rimossa
     * @exception BoundaryViolationException
     * v e` radice dell'albero contenitore
     * @exception InvalidPositionException
     * posizionatore illegale  (anche se v non foglia)
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    Object removeAboveExternal(Position vExt)
      throws BoundaryViolationException, 
      InvalidPositionException, InvalidContainerException;

    /**[m][a]
     * il nodo fornito e tutti i suoi discendenti vengono 
     * rimossi e sostituiti con un nodo foglia vuoto -
     * i nodi a partire da v vengono inseriti in un
     * nuovo albero appositamente creato
     * @param v  radice del sottoalbero da rimuovere
     * @return sottoalbero rimosso (come nuovo albero binario)
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    ExpandBinaryTree cut(Position v)
      throws 
      InvalidPositionException, InvalidContainerException;

    /**[m][a]
     * il nodo foglia vExt viene rimosso e sostituito
     * dal sottoalbero subTree -
     * i nodi di subTree vengono inseriti nel 
     * contenitore this, subTree diventa un albero vuoto -
     * @param vExt  nodo da rimuovere
     * @param subTree  sottoalbero da 'attaccare'
     * @exception IllegalArgumentException
     * subTree == this
     * @exception InvalidPositionException
     * posizionatore illegale
     * (anche con: vExt non foglia, subTree non compatibile 
     * con this o pari a null)
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    void link(Position vExt, ExpandBinaryTree subTree)
      throws IllegalArgumentException,
      InvalidPositionException, InvalidContainerException;

    /**[m][a]
     * scambia il sottoalbero v con l'albero subTree -
     * i nodi di subTree vengono inseriti nel 
     * contenitore this, quelli del sottoalbero in posizione
     * v vengono assegnati ad un nuovo albero appositamente
     * creato, subTree diventa un albero vuoto
     * @param v  sottoalbero da rimuovere
     * @param subTree  albero da 'attaccare'
     * @return sottoalbero rimosso (come nuovo albero binario)
     * @exception IllegalArgumentException
     * subTree == this
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    ExpandBinaryTree replaceSubtree(Position v, 
      ExpandBinaryTree subTree)
      throws IllegalArgumentException,
      InvalidPositionException, InvalidContainerException;

} //{i} ExpandBinaryTree
