package FI2.Tree;
/**{p}
 * -- FI2 --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Alberi
 */

/**{f}
 * Huffman.java
 * Creazione dell'albero per il codice di Huffman
 *
 * @version 3.00 00/09/27
 * @author M.Moro DEI UPD
 */

import java.io.*;
import FI2.Util.FI2Sys;
import FI2.Linear.PriorityQueue;
import FI2.Linear.SeqHeapPriorityQueue;
import FI2.Set.ObjectIterator;
import FI2.Set.Position;
import FI2.Set.Item;
import FI2.Util.SimpleComparable;
import FI2.Util.Comparator;
import FI2.Util.IntegerComparator;
import FI2.Util.CharComparator;
import FI2.Dictionary.Dictionary;
import FI2.Dictionary.UnOrderedSLList;
import FI2.Dictionary.OrderedDictionary;
import FI2.Dictionary.OrderedArraySeq;

/**{c}
 * costruzione dell'albero per il codice di
 * Huffman -
 * L'algoritmo greedy di generazione del codice di Huffman e`
 * qui realizzato nel costruttore della classe Huffman che
 * produce l'albero di codifica. I nodi dell'albero contengono
 * come valore un oggetto Item che include un oggetto che 
 * rappresenta il valore da codificare e un intero
 * che ne rappresenta la frequenza. La coda di priorita` 
 * ha come chiave la frequenza e come elementi i sottoalberi
 * che vengono a mano a mano creati (bottom-up).
 */
public class Huffman
{
    private PriorityQueue freqNodes;
      // priority queue con i nodi dell'albero
      // binario delle frequenze
    private ExpandBinaryTree code;  
      // albero binario delle frequenze
    private OrderedDictionary valCode;
      // dizionario dei valori-codici calcolati

    /**[c]
     * costruttore base -
     * genera l'albero binario di codifica, prelevando
     * le coppie valore-frequenza dai parametri forniti
     * @param val  vettore dei valori
     * @param frq  vettore delle corrispondenti frequenze
     * @param valC  comparatore dei valori
     * @param size  numero coppie
     */
    public Huffman(Object val[], int frq[], Comparator valC, 
      int size)
    {
        Position root;
          // radice di un sottoalbero
        Object key;
          // la relativa chiave
        Dictionary valPos;
          // dizionario delle posizioni dei nodi freq-valore


        freqNodes = 
          new SeqHeapPriorityQueue(new IntegerComparator());
            // priority queue a priorita` crescenti
        valPos = new UnOrderedSLList(valC);
          // per inserimento dei nodi foglia
        for (int i=0; i<size; i++)
        {
            ExpandBinaryTree t1 = new LinkedBinaryTree();
            t1.expandExternal(root = t1.root());
            t1.replaceElement(root, 
              new Item(key=new Integer(frq[i]), val[i]));
              // alberino da un elemento frequenza
              // come chiave-valore
            freqNodes.insertItem(key, t1);
              // la chiave e` la frequenza, il valore
              // e` il sottoalbero corrispondete
            valPos.insertItem(val[i], root);
              // si memorizzano in una dizionario, ordinato
              // in base al valore,
              // le posizioni dei nodi con un valore
              // per facilitarne la ricerca successivamente
        } // for

        // ciclo dell'algortimo greedy
        for(int i=1; i<=size-1; i++)
        {
            // estrae i primi due valori
            // (frequenze piu` basse) 
            int leftKey = ((Integer)freqNodes.minKey())
              .intValue();
            ExpandBinaryTree left = 
              (ExpandBinaryTree)freqNodes.removeMin();
            int rightKey = ((Integer)freqNodes.minKey())
              .intValue();
            ExpandBinaryTree right = 
              (ExpandBinaryTree)freqNodes.removeMin();

            // costruisce albero somma
            ExpandBinaryTree sumTree = new LinkedBinaryTree();
            sumTree.expandExternal(root = sumTree.root());
            sumTree.link(sumTree.leftChild(root), left);
            sumTree.link(sumTree.rightChild(root), right);
              // si poteva anche attaccare right a left
              // usando expandInternal di LinkedBinaryTree
              // e left come risultato somma, ma si preferisce
              // mantenere una maggiore astrazione in forma
              // di generico BinaryTree (che potrebbe essere
              // implementato non come LinkedBinaryTree)
            sumTree.replaceElement(root, 
              new Item(key=
              new Integer(leftKey+rightKey), null));            
              // inserisce il nodo intermedio con la somma
              // rappresentata da una chiave pari alla somma
              // delle frequenze delle radici dei suoi 
              // sottoalberi e valore nullo
            freqNodes.insertItem(key, sumTree);
        } //for greedy

        code = (ExpandBinaryTree)freqNodes.removeMin();
            // l'ultimo nodo e` la radice 
            // dell'albero di codifica

        // costruisce dizionario dei valori-codici
        valCode = new OrderedArraySeq(valC);
          // per inserimento dei valori-codici
        ObjectIterator it = valPos.elements();
        while (it.hasNext())
        {
            String str = "";
            Position par, 
              curPos = (Position)it.nextObject();
            Object curVal = 
              ((Item)curPos.element()).element();
              // padre

            // risale verso la radice e ricostruisce
            // la codifica in ordine inverso
            while (! code.isRoot(curPos))
            {
                par = code.parent(curPos);
                str = (code.leftChild(par) == curPos ?
                  // figlio di sinistra, 0
                  "0" :
                  // figlio di destra, 0
                  "1") + str;
                curPos = par;
            } //while (isRoot)
            valCode.insertItem(curVal, str);
        } //while (hasMoreElements)
    } //[c] Huffman

    /**[m]
     * ricava dall'albero binario di codifica
     * la stringa con la rappresentazione della codifica
     * binaria (sequenza di 0-1) corrispodente al valore
     * fornito -
     * il valore e` assunto implementare l'interfaccia
     * SimpleComparable
     * @param value  valore di cui calcolare la codifica
     * @return la stringa con la codifica binaria
     * @see SimpleComparable
     */
    public String hufCode(Object value)
    {
        Object ret;
        if ((ret = valCode.findElement(value)) == 
          Dictionary.NO_SUCH_KEY)
          // non trovato
            return "Non trovato";
        return (String)ret;
    } //[m] hufCode

    /**[m]
     * visita in preordine con indicazione della codifica
     * 0-1 del ramo
     * @param root  posizione nodo radice
     * @param level  livello del nodo radice
     * @return la stringa che rappresenta la visita
     */
    public String preOrderWalk(Position root, int level)
    {
        String str = "";

        if (code.isInternal(root))
        {
            str +=
"                                                            "
              .substring(0, 2*level) + root.element()
          + "\n";
            if (code.isInternal(code.leftChild(root)))
                str +=
"                                                            "
                  .substring(0, 2*level+1) + "[0]\n";
            str += preOrderWalk(code.leftChild(root), 
              level+1);
            if (code.isInternal(code.leftChild(root)))
                str +=
"                                                            "
                  .substring(0, 2*level+1) + "[1]\n";
            str += preOrderWalk(code.rightChild(root), 
              level+1);
        } // if internal
        return str;
    } //[m] inOrderWalk

    /**[m]
     * conversione a stringa -
     * produce la visita in ordine interno dell'albero
     * @return stringa con l'albero di codifica
     */
    public String toString()
    {
        if (code.isEmpty())
            // heap vuoto
            return "Codice Huffman vuoto!\n";
        return "Codice Huffman in preordine:\n" + 
          preOrderWalk(code.root(), 0);
    }

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "Codice di Huffman: " + 
  "Battere coppie carattere alfabetico, intero (fine=Ctrl-Z):");
        } //if
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(
          new InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        int myFreq[] = new int[100];
        Character myVal[] = new Character[100];
        int pos = 0;

        sIn.parseNumbers();
        try
        {
            Character next;

            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                next = new Character(sIn.sval.charAt(0));
                if (next == null)
                {
                    FI2Sys.out.println(sIn.sval.charAt(0) +
                      " non char normale: non inserito");
                    continue;
                }
                if (sIn.nextToken() == 
                  StreamTokenizer.TT_NUMBER)
                {
                    myVal[pos] = next;
                    myFreq[pos++] = (int)sIn.nval;
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while
        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch
        // la lista ha ora i numeri forniti nell'ordine

        FI2Sys.out.println("Crea codice con size=" + pos);
        Huffman myCode = new Huffman(myVal, myFreq, 
          new CharComparator(), pos);
        FI2Sys.out.println("Albero heap creato");
        FI2Sys.out.println(myCode);
        FI2Sys.out.println("\nCodici ricavati:");
        for (int i=0; i<pos; i++)
            FI2Sys.out.println(myVal[i] + "[fr=" +
              myFreq[i] + "] " +
              myCode.hufCode(myVal[i]));
        FI2Sys.out.println("a " +
          myCode.hufCode(new Character('a')));
        FI2Sys.out.println("? " +
          myCode.hufCode(new Character('?')));
    } //[m][s] main

} //{c} Huffman
