package FI2.Tree;
/**{p}
 * -- FI2.Tree --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Alberi
 */

/**{f}
 * SequenceBinaryTree.java
 * implementazione di albero binario mediante
 * sequenza
 *
 * @version 2.00 00/09/23
 * @author M.Moro DEI UPD
 */

import java.io.*;
import java.util.NoSuchElementException;
import FI2.Set.Position;
import FI2.Set.ObjectIterator;
import FI2.Set.PositionIterator;
import FI2.Set.Container;
import FI2.Set.InspectableContainer;
import FI2.Set.PositionalContainer;
import FI2.Set.InspectablePositionalContainer;
import FI2.Util.FI2Sys;
import FI2.Exceptions.InvalidPositionException;
import FI2.Exceptions.InvalidContainerException;
import FI2.Exceptions.BoundaryViolationException;
import FI2.Linear.Sequence;
import FI2.Linear.ArraySequence;
import FI2.Tree.BinTreeBasicAlgo;


/**{c}
 * albero binario - i nodi sono elementi di un
 * Sequence, contengono un posizionatore ma non
 * i riferimenti espliciti ai figli -
 * la regola base e`: index(root)=1 
 * index(left(n)) = 2*n index(right(n)) = 2*n+1 -
 * l'indice 0 non e` utilizzato -
 * un albero viene costruito per default come 
 * costituito da un unico nodo nullo -
 * un nodo nullo ha l'elemento associato pari a null -
 * poiche' il conteggio include gli eventuali nodi nulli, 
 * un albero 'vuoto di elementi' ha pero` size == 1 e in
 * generale il numero totale di nodi inseriti e` pari
 * nodi = elementi + nodinulli -
 * non e` necessario un metodo insert(): la costruzione
 * procede con l'uso del metodo expandExternal() a 
 * partire dall'albero iniziale: questo metodo aggiunge
 * al nodo due figli nulli -
 * poiche' nella realizzazione mediante sequenza 
 * vi possono essere dei 'buchi' interni (elementi
 * nella sequenza che non corrispondono a nodi
 * dell'albero, nemmeno a nodi nulli) si assume che
 * un nodo sia foglia se gli indici dei nodi left e right
 * sono oltre l'indice dell'ultimo elemento della sequenza
 * oppure se corrispondono a due elementi della
 * sequenza associati ad un oggetto speciale denominato
 * 'sentinella' (ad esempio un albero in cui esiste il nodo
 * di indice 12, che e` di livello 3, ma il cui nodo 
 * di indice 5, che e` di livello 2, non ha figli,
 * ha gli elementi di indici 10 e 11, di livello 3, 
 * sentinelle) -
 * per semplicita` di trattamento, si assume che un nodo 
 * interno abbia ENTRAMBI i riferimenti nonnulli
 * (se necessario, uno dei due puo` essere un nodo nullo)
  * cioe` sent) 
 */
public class SequenceBinaryTree 
  implements ExpandBinaryTree
{
    public static final int ROOTRANK = 1;
      // l'indice non valido e indice della radice
    private static final int LEFTCHILD = 0;
    private static final int RIGHTCHILD = 1;
    private static final int DONE = 2;
      // per gli iteratori

    private Sequence S;  // sequenza di supporto
    private int size;  // numero totale di nodi

    /**{c}{l}
     * classe locale per definire l'oggetto speciale
     * che fa da nodo 'buco'
     */
    private class Sentinel
    {
        public String toString()
        {
            return "Sentinella";
        } //[m] toString
    } //{c}{l} Sentinel

    private Sentinel sent;  
      // sentinella per marcare nodi 'buco'

    /**[c]
     * costruttore di default, albero vuoto
     */
    public SequenceBinaryTree()
    {
        S = new ArraySequence();
        sent = new Sentinel();
        S.insertAtRank(0, sent);
        S.insertAtRank(ROOTRANK, null);
          // indice 0 non utilizzato S[0]=sent
          // S[ROOTRANK] e` null
        size = 1;
    } //[c] SequenceBinaryTree

    /**[m]
     * O(1)
     * @see InspectableContainer
     */
    public int size()
    {
        return size;
    } //[m] size

    /**[m]
     * O(1)
     * @see InspectableContainer
     */
    public boolean isEmpty()
    {
        return size == 0;
    } //[m] isEmpty

    /**[m]
     * O(n)
     * @see InspectableContainer
     */
    public ObjectIterator elements()
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore
         * interno per gli elementi -
         * sfrutta quello delle posizioni
         * @see PosIter
         */
        class Iter implements ObjectIterator
        {
            private PositionIterator posIt;
              // iteratore di posizioni

            /**[c]
             * costruttore di default
             */
            Iter()
            {
                posIt = positions();
            } //[c] Enum

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return posIt.hasNext();
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                return 
                  ((Position)posIt.nextObject()).element();
            } //[m] nextObject

        } //{c} Iter

        return new Iter();
    } //[m] elements

    /**[m]
     * @see Container
     */
    public Container newContainer()
    {
        return new SequenceBinaryTree();
    } //[m] newContainer

    /**[m]
     * @see InspectablePositionalContainer
     * @see Sequence
     */
    public PositionIterator positions()
    {
        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per le posizioni -
         * deve saltare i nodi 'buco' 
         * rappresentati da sent
         */
        class PosIter implements PositionIterator
        {
            private PositionIterator seqIt;
              // iteratore della sequenza
            private Position curPos;
              // posizione corrente

            /**[c]
             * costruttore di default
             */
            PosIter()
            {
                seqIt = S.positions();
                while (seqIt.hasNext() &&
                  (curPos = seqIt.nextPosition()).
                  element() == sent);
                    // salta i buchi
                if (curPos.element() == sent)
                    // non ci sono piu` elementi, ma non 
                    // dovrebbe accadere all'inizio della 
                    // sequenza perche` l'albero ha almeno 
                    // la radice null
                    curPos = null;
            } //[c] PosIter
            

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return curPos != null;
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                return nextPosition();
            } //[m] nextObject
                
            /**[m]
             * @see PositionIterator
             */
            public Position nextPosition()
            {
                if (curPos == null)
                    throw new NoSuchElementException(
  "In SequenceBinaryTree.PosIter.nextPosition: fine elementi");
                Position toReturn = curPos;
                for (; ; )
                {
                    if (!seqIt.hasNext())
                    {
                        // fine elementi
                        curPos = null;
                        break;
                    }
                    if ((curPos = 
                      seqIt.nextPosition()).
                      element() != sent)
                        // elemento non buco
                        break;
                } //for
                return toReturn;
            } //[m] nextPosition

        } //{c}{l} PosIter

        return new PosIter();
    } //[m] positions

    /**[m]
     * @see PositionalContainer
     */
    public Object replaceElement(Position p, Object newElem)
      throws InvalidPositionException,
      InvalidContainerException
    {
        return S.replaceElement(p, newElem);
    } //[m] replaceElement

    /**[m]
     * @see PositionalContainer
     */
    public void swapElements(Position p, Position q)
      throws InvalidPositionException,
      InvalidContainerException
    {
        S.swapElements(p, q);
    } //[m] swap

    /**[m]
     * O(1)
     * @see InspectableTree
     */
    public Position root()
    {
        return S.atRank(ROOTRANK);
    } //[m] root

    /**[m]
     * O(1)
     * @see InspectableTree
     */
    public Position parent(Position v)
      throws BoundaryViolationException, 
      InvalidPositionException, 
      InvalidContainerException
    {
        int r;

        if ((r = S.rankOf(v)) == ROOTRANK)
            throw new BoundaryViolationException(
  "In SequenceBinaryTree.parent: il nodo [" + v + 
              "] e` la radice");
        return S.atRank(getParent(r));
          // index(parent(n)) = int(n/2)
    } //[m] parent

    /**[m]
     * O(nc)
     * @see InspectableTree
     */
    public PositionIterator children(Position v)
      throws InvalidPositionException, 
      InvalidContainerException
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per i figli
         */
        class ChildIter implements PositionIterator
        {
            private int p;  // il nodo padre
            private int order;  // nell'ordine sin-des

            /**[c]
             * costruttore
             * @param v  nodo di cui ricavare i figli
             */
            ChildIter(int v)
            {
                p = v;
                order = isInternal(S.atRank(v)) ? 
                  LEFTCHILD: DONE;
                  // parte con il sinistro se non foglia
            } //[c] ChildIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return order != DONE;
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator
             */
            public Object nextObject()
            {
                return nextPosition();
            } //[m] nextObject
            
            /**[m]
             * @see PositionIterator
             */
            public Position nextPosition()
            {
                if (order == DONE)
                    throw new NoSuchElementException(
"In SequenceBinaryTree.ChildIter.nextPosition: fine figli"
                  );
                Position curPos;
                if (order == LEFTCHILD)
                {
                    curPos = S.atRank(getLeft(p));
                    order = RIGHTCHILD;
                }
                else
                {
                    curPos = S.atRank(getRight(p));
                    order = DONE;
                }
                return curPos;
            } //[m] nextPosition

        } //{c}{l} ChildIter

        int here = S.rankOf(v);
          // esegue anche un check sul posizionatore
        return new ChildIter(here);

    } //[m] children

    /**[m]
     * O(ns)
     * @see InspectableTree
     */
    public PositionIterator siblings(Position v)
      throws BoundaryViolationException, 
      InvalidPositionException, InvalidContainerException
    {

        /**{c}{l}
         * classe locale che rappresenta l'iteratore 
         * interno per i fratelli
         */
        class SiblingIter implements PositionIterator
        {
            private int p;  // il nodo padre
            private int order;  // nell'ordine sin-des

            /**[c]
             * costruttore
             * @param v  nodo di cui ricavare i figli
             */
            SiblingIter(int v)
            {
                p = getParent(v);
                order = (getRight(p) == v) ? 
                  LEFTCHILD: RIGHTCHILD;
                  // l'altro fratello
            } //[c] SiblingIter

             /**[m]
             * @see ObjectIterator
             */
            public boolean hasNext()
            {
                return order != DONE;
            } //[m] hasNext

            /**[m]
             * @see ObjectIterator             
             */
            public Object nextObject()
            {
                return nextPosition();
            }

            /**[m]
             * @see PositionIterator             
             */
            public Position nextPosition()
            {
                if (order == DONE)
                    throw new NoSuchElementException(
"In SequenceBinaryTree.SiblingIter.nextPosition: fine fratelli"
                  );
                Position curPos;
                if (order == LEFTCHILD)
                    curPos = S.atRank(getLeft(p));
                else
                    curPos = S.atRank(getRight(p));
                order = DONE;
                return curPos;
            } //[m] nextPosition

        } //{c}{l} SiblingIter

        int here = S.rankOf(v);
          // esegue anche un check sul posizionatore
        if (here == ROOTRANK)
            throw new BoundaryViolationException(
  "In SequenceBinaryTree.siblings: il nodo [" + v + 
              "] e` la radice");
        return new SiblingIter(here);
    } //[m] siblings

    /**[m]
     * O(1)
     * @see InspectableTree
     */
    public boolean isInternal(Position v)
      throws InvalidPositionException, 
      InvalidContainerException
    {
        int here = S.rankOf(v);
          // esegue anche un check sul posizionatore
        return (getLeft(here) < S.size()) && 
          (S.elemAtRank(getLeft(here)) != sent
          // || S.elemAtRank(getRight(here)) != sent
          );
          // si assume che una foglia abbia entrambi i 
          // riferimenti nulli
    } //[m] isInternal

    /**[m]
     * O(1)
     * @see InspectableTree
     */
    public boolean isExternal(Position v)
      throws InvalidPositionException, 
      InvalidContainerException
    {
        return !isInternal(v);
    } //[m] isExternal

    /**[m]
     * O(1)
     * @see InspectableTree
     */
    public boolean isRoot(Position v)
      throws InvalidPositionException, 
      InvalidContainerException
    {
        return S.rankOf(v) == ROOTRANK;
          // esegue anche un check sul posizionatore
    } //[m] isRoot

    /**[m]
     * O(1)
     * @see InspectableBinaryTree
     */
    public Position leftChild(Position v)
      throws BoundaryViolationException, 
      InvalidPositionException, InvalidContainerException
    {
        if (isExternal(v))
          // foglia
            throw new BoundaryViolationException(
  "In SequenceBinaryTree.leftChild: il nodo [" + v + 
              "] e` una foglia");
        return S.atRank(getLeft(S.rankOf(v)));
          // esegue anche un check sul posizionatore
    } //[m] leftChild

    /**[m]
     * O(1)
     * @see InspectableBinaryTree
     * @see Sequence
     */
    public Position rightChild(Position v)
      throws BoundaryViolationException, 
      InvalidPositionException, InvalidContainerException
    {
        if (isExternal(v))
          // foglia
            throw new BoundaryViolationException(
  "In SequenceBinaryTree.rightChild: il nodo [" + v + 
              "] e` una foglia");
        return S.atRank(getRight(S.rankOf(v)));
          // esegue anche un check sul posizionatore
    } //[m] rightChild

    /**[m]
     * O(1)
     * @see InspectableBinaryTree
     */
    public Position sibling(Position v)
      throws BoundaryViolationException, 
      InvalidPositionException, InvalidContainerException
    {
        int here = S.rankOf(v);
          // esegue anche un check sul posizionatore
        if (here == ROOTRANK)
            throw new BoundaryViolationException(
  "In SequenceBinaryTree.sibling: il nodo [" + v + 
              "] e` la radice");
        int par = getParent(here);
        return (getLeft(par) == here) ? 
          S.atRank(getRight(par)) : 
          S.atRank(getLeft(par));
    } //[m] sibling

    /**[m]
     * O(1)
     * @see ExpandBinaryTree
     */
    public void expandExternal(Position vExt)
      throws InvalidPositionException, 
      InvalidContainerException
    {
        if (isInternal(vExt))
          // non foglia
            throw new InvalidPositionException(
"In SequenceBinaryTree.expandExternal: il nodo [" + vExt + 
              "] non e` una foglia"
          );
        int here = S.rankOf(vExt);
          // esegue anche un check sul posizionatore
        int left = getLeft(here);

        // riempie il buco, se serve
        while (S.size() < left)
            S.insertAtRank(S.size(), sent);
        if (left >= S.size())
        {
            S.insertAtRank(S.size(), null);
            S.insertAtRank(S.size(), null);
        } //if left
        else
        {
            S.replaceAtRank(left, null);
            S.replaceAtRank(left+1, null);
        } // else left
        size +=2;
    } //[m] expandExternal
    
    /**[m]
     * O(1)
     * @see ExpandBinaryTree
     */
    public Object removeAboveExternal(Position vExt)
      throws BoundaryViolationException, 
      InvalidPositionException, InvalidContainerException
    {
        int here = S.rankOf(vExt);
          // esegue anche un check sul posizionatore
        if (here == ROOTRANK)
            throw new BoundaryViolationException(
  "In SequenceBinaryTree.removeAboveExternal: il nodo [" + 
              vExt + "] e` la radice"
          );
        if (isInternal(vExt))
          // non foglia
            throw new InvalidPositionException(
"In SequenceBinaryTree.removeAboveExternal: il nodo [" + 
              vExt + "] non e` una foglia"
          );
        int par = getParent(here);
          // il padre della foglia
        int gpar = getParent(par);
          // il nonno della foglia; 0 se par==root
        int sib = (getLeft(par) == here) ? getRight(par) :
          getLeft(par);
          // il fratello
        Object obj = S.elemAtRank(par);

        // il fratello diventa figlio del nonno
        ((ArraySequence)S).swapPos(S.atRank(par), 
          S.atRank(sib));
          // questa chiamata e` necessaria per mantenere
          // lo stesso significato per il Position che 
          // rappresenta il vecchio figlio di sinistra e che 
          // ora sostituisce il padre. Questo mantenimento 
          // non c'e` se ci si limita ad aggiornare l'elemento 
          // associato al padre con l'istruzione:
          // S.replaceElemAtRank(par, S.elemAtRank(sib));
        S.replaceAtRank(here, sent);
        S.replaceAtRank(sib, sent);
        // elimina tutti i nodi nulli alla fine della sequenza
        while (S.elemAtRank(S.size()-1) == sent && S.size()>2)
            S.removeAtRank(S.size()-1);
        size -=2;
        return obj;
    } //[m] removeAboveExternal

    /**[m]
     * @see ExpandBinaryTree
     */
    public ExpandBinaryTree cut(Position v)
      throws InvalidPositionException, 
      InvalidContainerException
    {
        return replaceSubtree(v, new SequenceBinaryTree());
          // il taglio puo` essere visto come scambio
          // con un sottoalbero vuoto
    } //[m] cut
    
    /**[m]
     * @see ExpandBinaryTree
     */
    public void link(Position vExt, ExpandBinaryTree subTree)
      throws IllegalArgumentException,
      InvalidPositionException, InvalidContainerException
    {
        if (isInternal(vExt))
          // non foglia
            throw new InvalidPositionException(
"In SequenceBinaryTree.link: il nodo [" + 
              vExt + "] non e` una foglia"
          );
        replaceSubtree(vExt, subTree);
          // viene scartato il sottoalbero restituito
       
    } //[m] link

    /**[m]
     * O(n) (a causa di count())
     * @see ExpandBinaryTree
     */
    public ExpandBinaryTree 
      replaceSubtree(Position v, ExpandBinaryTree subTree)
      throws IllegalArgumentException,
      InvalidPositionException, InvalidContainerException
    {
        if (subTree == this)
            throw new IllegalArgumentException(
  "In SequenceBinaryTree.replaceSubTree: " + 
              "il sottoalbero coincide con l'albero");
        SequenceBinaryTree sTree;
        try
        {
            sTree = (SequenceBinaryTree) subTree;
        }
        catch (ClassCastException e)
        {
            // l'albero non e` di tipo SequenceBinaryTree
            throw new InvalidPositionException(
"In SequenceBinaryTree.replaceSubTree: " + 
              "sottoalbero di tipo incompatibile");
        }
        int here = S.rankOf(v);
          // esegue anche un check sul posizionatore
        int par = getParent(here);
          // il padre del nodo
        // costruisce il sottoalbero risultato
        int hereSize = count(here);
          // numero nodi rimossi
        SequenceBinaryTree newTree = new SequenceBinaryTree();
        BinTreeBasicAlgo.cloneSubTree(this, newTree, v, 
          newTree.S.atRank(ROOTRANK));
        removeSubTree(here);
        BinTreeBasicAlgo.cloneSubTree(sTree, this, 
          sTree.S.atRank(ROOTRANK), v);
        // subTree diventa vuoto
        sTree.removeSubTree(ROOTRANK);
        sTree.size = 1;
        size += (sTree.size-hereSize);
        return newTree;
    } //[m] replaceSubTree

    /**[m] !!
     * la posizione nella sequenza corrispondente all'indice 
     * fornito - 
     * serve per trovare facilmente la posizione di 
     * inserimento nel caso questo albero venga usato 
     * come heap
     * @param r  indice nella sequenza
     * @return la relativa posizione
     * @exception BoundaryViolationException  
     * indice fuori range
     */
    public Position atRank(int r) 
      throws BoundaryViolationException
    { return S.atRank(r); }
        
    /**[m] 
     * conta ricorsivamente i nodi di un sottoalbero -
     * il contenitore e` assunto corretto - 
     * O(n)
     * @param here  indice radice sottoalbero
     * @return il conteggio
     */
    private int count(int here)
    {
        int sum = 1;
        int child;
        if ((child = getLeft(here)) < S.size() &&
          S.elemAtRank(child) != sent)
            // se c'e` il figlio di sinistra, 
            // aggiungi conteggio
            sum += count(child);
        if ((child = getRight(here)) < S.size() &&
          S.elemAtRank(child) != sent)
            // se c'e` il figlio di destra, aggiungi conteggio
            sum += count(child);
        return sum;
    } //[m] count

    /**[m] 
     * annulla ricorsivamente i nodi di un sottoalbero -
     * il contenitore e` assunto corretto - 
     * O(n)
     * @param here  indice radice sottoalbero
     */
    private void removeSubTree(int here)
    {
        S.replaceAtRank(here, sent);
        int child;
        if ((child = getLeft(here)) < S.size() &&
          S.elemAtRank(child) != sent)
            // se c'e` il figlio di sinistra, annulla
            removeSubTree(child);
        if ((child = getRight(here)) < S.size() &&
          S.elemAtRank(child) != sent)
            // se c'e` il figlio di destra, annulla
            removeSubTree(child);
        // elimina tutti i nodi nulli alla fine della sequenza
        while (S.elemAtRank(S.size()-1) == sent && S.size()>2)
            S.removeAtRank(S.size()-1);
    } //[m] removeSubTree

    /**[m][s]
     * calcola indice nodo figlio di sinistra -
     * O(1)
     * @param p  indice nodo padre
     * @return indice nodo figlio di sinistra
     */
    private static int getLeft(int p)
    {
        return p<<1;
    } //[m][m] getLeft

    /**[m][s]
     * calcola indice nodo figlio di destra -
     * O(1)
     * @param p  indice nodo padre
     * @return indice nodo figlio di destra
     */
    private static int getRight(int p)
    {
        return (p<<1)+1;
    } //[m][m] getRight

    /**[m][s]
     * calcola indice nodo padre -
     * O(1)
     * @param v  indice nodo figlio
     * @return indice nodo padre
     */
    private static int getParent(int v)
    {
        return v>>1;
    } //[m][s] getParent

    /**[m] !!
     * costruisce ricorsivamente un albero
     * con valori interi
     * @return il precedente oggetto
     * @exception InvalidPositionException
     * posizionatore illegale
     * @exception InvalidContainerException
     * contenitore this illegale
     */
    protected boolean buildInteger(Position v, 
      int nodes, StreamTokenizer sIn)
      throws IOException
    {
        Integer next = new Integer(0);
        boolean ret = true; // ritorno
        FI2Sys.out.println("Crea albero di " + nodes +
          " elementi");
        try
        {
            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    next = new Integer((int)sIn.nval);
                    break;
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero: non inserito");
            } //while
        } //try
        catch (EOFException e)
        {
            // termina il ciclo
            return false;
        } //catch
        if (nodes > 1)
        {
            expandExternal(v);
            int leftSize = nodes>>1;
              // se del caso, ne mette di piu` a sinistra
              // perche' i nodi ancora da sistemare sono
              // nodes-1. Ad esempio con nodes=4, a sinistra
              // ne mette 2 e a destra 1
            if (leftSize > 0)
                 ret = ret && 
                   buildInteger(leftChild(v), leftSize, sIn);
            int rightSize = nodes-1-leftSize;
            if (rightSize > 0)
                 ret = ret && 
                   buildInteger(rightChild(v), rightSize, 
                   sIn);
        } //if
        replaceElement(v, next);
        return ret;
    } //[m] build

    /**[m] !!
     * conversione a stringa
     * @return stringa con il contenuto
     */
    public String toString()
    {
        return BinTreeBasicAlgo.toString(this, root());
    } //[m] toString()

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  opzionale: nome file di input
     * @exception IOException  se file non trovato 
     */
    public static void main(String[] args)
      throws IOException
    {
        InputStream in;

        if (args.length == 0)
        {
            // default, stdin
            in = System.in;
            FI2Sys.out.println(
  "SequenceBinaryTree: " + 
  "Battere numero di interi e interi (fine=Ctrl-Z):");
        } //if
        else
            in = new FileInputStream(args[0]);

        /* StreamTokenizer(InputStream) e` un costruttore
        deprecato in JDK 1.1; viene sostituito inserendo
        uno StreamReader che converte la sequenza di byte in
        sequenza di caratteri */
        Reader re = new BufferedReader(
          new InputStreamReader(in));
        StreamTokenizer sIn = new StreamTokenizer(re);
        SequenceBinaryTree myTree = new SequenceBinaryTree();

        sIn.parseNumbers();
        int numEl = 0;
        try
        {
            while(sIn.nextToken() != StreamTokenizer.TT_EOF)
            {
                if (sIn.ttype == StreamTokenizer.TT_NUMBER)
                {
                    numEl = (int)sIn.nval;
                    break;
                } //if
                else  
                    FI2Sys.out.println(sIn.sval +
                      " non numero");
            } //while
        } //try
        catch (EOFException e)
        {
            // termina il ciclo
        } //catch

        myTree.buildInteger(myTree.root(), numEl, sIn);
        FI2Sys.out.println("Albero creato");
        FI2Sys.out.println("size=" + myTree.size() + 
          " [S.size() =" + myTree.S.size() + "]");
        FI2Sys.out.println(myTree);
        Position root;
        FI2Sys.out.println("\nLa radice e` " + 
          (root=myTree.root()).element());
        FI2Sys.out.println(
          "\nIl figlio sinistro della radice e` " + 
          myTree.leftChild(root).element() + 
            "\n  e suo padre e` " +
          myTree.parent(myTree.leftChild(root)).element());
        FI2Sys.out.println("\nI figli della radice sono:");
        PositionIterator it = myTree.children(root);
        while (it.hasNext())
            FI2Sys.out.println(it.nextPosition().
              element());
        FI2Sys.out.println(
"\nI fratelli del figlio destro della radice sono:");
        it = myTree.siblings(myTree.rightChild(root));
        while (it.hasNext())
            FI2Sys.out.println(it.nextPosition().
              element());
        FI2Sys.out.println(
"\nIl singolo fratello del figlio destro della radice e` " +
          myTree.sibling(myTree.rightChild(root)).element());
        FI2Sys.out.println("\nVisita in preordine");
        BinTreeBasicAlgo.preorderPrint(myTree, myTree.root());
        FI2Sys.out.println("\nVisita parentetica");
        BinTreeBasicAlgo.parentheticRepresentation(myTree, 
          myTree.root());
        FI2Sys.out.println("\nVisita parentetica come generico");
        TreeBasicAlgo.parentheticRepresentation(myTree, 
          myTree.root());
        FI2Sys.out.println("\nVisita in ordine interno");
        BinTreeBasicAlgo.inorderPrint(myTree, myTree.root());
        FI2Sys.out.println("\nTour di Eulero");
        BinTreeBasicAlgo.eulerTour(myTree, myTree.root());
        FI2Sys.out.println("\nVisita in postordine");
        BinTreeBasicAlgo.postorderPrint(myTree, 
          myTree.root());
        FI2Sys.out.println("\nAltezza dell'albero=" + 
          BinTreeBasicAlgo.height1(myTree));
        FI2Sys.out.println(
          "\nProfondita` e altezza dei nodi");
        it = myTree.positions();
        int pos = 0;
        while (it.hasNext())
        {
            Position p = it.nextPosition();
            FI2Sys.out.println("Nodo " + (pos++) + " el=" +
              p.element() + " prof.=" +
              BinTreeBasicAlgo.depth(myTree, p) +
              " alt.=" +
              BinTreeBasicAlgo.height2(myTree, p));
        } //while
        SequenceBinaryTree myTree2 = new SequenceBinaryTree();
        FI2Sys.out.println("\nCopia del sottoalbero destro");
        BinTreeBasicAlgo.cloneSubTree(myTree, myTree2, 
          myTree.rightChild(myTree.root()), myTree2.root());
        FI2Sys.out.println(
          "\nVisita parentetica della copia");
        BinTreeBasicAlgo.parentheticRepresentation(myTree2, 
          myTree2.root());
        SequenceBinaryTree isACut = 
          (SequenceBinaryTree)myTree.cut(
          myTree.leftChild(myTree.root()));
        FI2Sys.out.println(
          "\nVisita parentetica dell'albero dopo il cut");
        BinTreeBasicAlgo.parentheticRepresentation(myTree, 
          myTree.root());
        FI2Sys.out.println(
          "\nVisita parentetica del cut");
        BinTreeBasicAlgo.parentheticRepresentation(isACut, 
          isACut.root());
    } //[m][s] main

} //{c} SequenceBinaryTree

