package FI2.Tree;
/**{p}
 * -- FI2.Tree --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Alberi
 */

/**{f}
 * TreeBasicAlgo.java
 * metodi base sugli alberi
 *
 * @version 3.00 00/09/22
 * @version 3.01 00/12/09 -
 * correzione in parentheticRepresentation
 * @author M.Moro DEI UPD
 */
 
import FI2.Set.Position;
import FI2.Set.PositionIterator;
import FI2.Util.FI2Sys;

/**{c}
 * alcuni metodi (statici) base su alberi generici
 * @see InspectableTree
 */
public class TreeBasicAlgo
{
    /**[m][s]
     * profondita` di un nodo valutata ricorsivamente
     * @param T  albero a cui il nodo appartiene
     * @param v  posizionatore sul nodo
     */
    public static int depth(InspectableTree T, Position v)
    {
        if (T.isRoot(v))
            return 0;
        else
            return 1+depth(T, T.parent(v));
    } //[m][s] depth

    /**[m][s]
     * altezza di un albero -
     * valuta la massima profondita` delle foglie
     * @param T  albero
     */
    public static int height1(InspectableTree T)
    {
        int h = 0;
        PositionIterator nodes_of_T = T.positions();

        while(nodes_of_T.hasNext())
        {
            Position v = nodes_of_T.nextPosition();
            if (T.isExternal(v))
                h = Math.max(h, depth(T, v));
        } //while
        return h;
    } //[m][s] height1

    /**[m][s]
     * altezza di un sottoalbero valutata ricorsivamente
     * @param T  albero
     * @param v  posizionatore al nodo radice del sottoalbero
     */
    public static int height2(InspectableTree T, Position v)
    {
        if (T.isExternal(v))
            return 0;

        int h = 0;
        PositionIterator children_of_v = T.children(v);

        while(children_of_v.hasNext())
            h = Math.max(h, height2(T, 
              children_of_v.nextPosition()));
        return 1+h;
    } //[m][s] height2

    /**[m][s]
     * visita ricorsiva in preordine del sottoalbero 
     * con radice v
     * @param T  albero
     * @param v  posizionatore al nodo radice del sottoalbero
     */
    public static void preorderPrint(InspectableTree T, Position v)
    {
        FI2Sys.out.println(v.element());
          // out la radice
        PositionIterator children_of_v = T.children(v);
        
        while(children_of_v.hasNext())
            preorderPrint(T, children_of_v.nextPosition());
    } //[m][s] preorderPrint

    /**[m][s]
     * visita ricorsiva in preordine del sottoalbero 
     * con radice v e aggiunta delle parentesi di livello
     * @param T  albero
     * @param v  posizionatore al nodo radice del sottoalbero
     */
    public static void 
      parentheticRepresentation(InspectableTree T, Position v)
    {
        if (v.element() != null)
            FI2Sys.out.println(v.element());
              // out nodo
        else
            FI2Sys.out.println("Nodo nullo");
        if (T.isInternal(v))
        {
            FI2Sys.out.println(v.element() + " (");
            PositionIterator children_of_v = T.children(v);
            
            while(children_of_v.hasNext())
            {
                FI2Sys.out.println(" ");
                parentheticRepresentation(T, 
                  children_of_v.nextPosition());
            } //while
            FI2Sys.out.println(")");
        } //else
    } //[m][s] parentheticRepresentation

    /**[m][s]
     * visita ricorsiva in postordine del sottoalbero 
     * con radice v
     * @param T  albero
     * @param v  posizionatore al nodo radice del sottoalbero
     */
    public static void 
      postorderPrint(InspectableTree T, Position v)
    {
        PositionIterator children_of_v = T.children(v);
        
        while(children_of_v.hasNext())
            postorderPrint(T, 
              children_of_v.nextPosition());

        FI2Sys.out.println(v.element());
          // out la radice
    } //[m][s] postorderPrint

} //{c} TreeBasicAlgo
