package FI2.Util;
/**{p}
 * -- FI2.Util --
 * Universita' di Padova -
 * Diploma Universitario in Ingegneria Informatica
 * Corso di Fondamenti di Informatica 2
 * Util
 */

/**{f}
 * BinaryMat.java
 * matrice bidimensionale di valori binari
 *
 * @version 2.00 98/09/23
 * @author M.Moro DEI UPD
 */

import java.io.*;

/**{c}
 * realizzazione compatta di una matrice di valori
 * binari mediante interi (32 bit impaccati)
 */
public class BinaryMat
{
    private int vec[];  // vettore di supporto
    private int columns;  // numero colonne (seconda dimens.)

    /**[c]
     * costruttore base
     * @param rows  numero delle righe
     * @param cols  numero delle colonne
     */
    public BinaryMat(int rows, int cols)
    {
        int size = rows * (columns = cols);
            // imposta colonne
        vec = new int[(size>>5)+(((size & 0x1f) == 0) ? 
          0 : 1)];
            // numero di elementi = ceiling(rows*cols/32)
    } //[c] BinaryMat

    /**[m]
     * accesso in lettura con boolean
     * @param row  indice di riga
     * @param col  indice di colonna
     * @return il bit riferito come boolean
     */
    public boolean getBoolVal(int row, int col)
    {
        if (col < 0 || col >= columns)
            // l'indice della colonna eccede il range
            throw new ArrayIndexOutOfBoundsException(col);
        int bitIndex = row*columns+col;
            // indice del bit mediante funzione di mappa 
        return
          (vec[bitIndex>>5] & 
          (0x1 << (bitIndex & 0x1f))) == 0 ?
          false : true;
    } //[m] getBoolVal

    /**[m]
     * accesso in lettura con intero
     * @param row  indice di riga
     * @param col  indice di colonna
     * @return il bit riferito rappresentato di numeri 0 e 1
     */
    public int getIntVal(int row, int col)
    {
        if (col < 0 || col >= columns)
            // l'indice della colonna eccede il range
            throw new ArrayIndexOutOfBoundsException(col);
        int bitIndex = row*columns+col;
            // indice del bit mediante funzione di mappa 
        return
          (vec[bitIndex>>5] >> (bitIndex & 0x1f)) & 0x1;
    } //[m] getIntVal

    /**[m]
     * accesso in scrittura con booleano
     * @param row  indice di riga
     * @param col  indice di colonna
     * @param val  valore booleano da assegnare
     */
    public void setBoolVal(int row, int col, boolean val)
    {
        if (col < 0 || col >= columns)
            // l'indice della colonna eccede il range
            throw new ArrayIndexOutOfBoundsException(col);
        int bitIndex = row*columns+col;
            // indice del bit mediante funzione di mappa 
        if (val)
            vec[bitIndex>>5] |= (0x1 << (bitIndex & 0x1f));
        else
            vec[bitIndex>>5] &= ~(0x1 << (bitIndex & 0x1f));
    } //[m] setBoolVal

    /**[m]
     * accesso in scrittura con intero
     * @param row  indice di riga
     * @param col  indice di colonna
     * @param val  valore binario da assegnare
     *             rappresentato come intero: 0 se false,
     *             != 0 se true 
     */
    public void setIntVal(int row, int col, int val)
    {
        if (col < 0 || col >= columns)
            // l'indice della colonna eccede il range
            throw new ArrayIndexOutOfBoundsException(col);
        int bitIndex = row*columns+col;
            // indice del bit mediante funzione di mappa 
        if (val!=0)
            vec[bitIndex>>5] |= (0x1 << (bitIndex & 0x1f));
        else
            vec[bitIndex>>5] &= ~(0x1 << (bitIndex & 0x1f));
    } //[m] setIntVal

    /**[m]
     * azzeramento
     */
    public void setZero()
    {
        for (int i=0; i<vec.length; vec[i++]=0);
    } //[m] setZero


    private static final int ROWS = 4;
    private static final int COLS = 34;

    /**[m][s]
     * metodo di collaudo
     * @param args[0]  non usato
     */
    public static void main(String[] args)
    {
        BinaryMat myMat = new BinaryMat(ROWS, COLS);

        for (int i=0; i<ROWS; i++)
            for (int j=0; j<COLS; j++)
                myMat.setBoolVal(i, j,
                  ((j>>2) & 1) == 0 ? false : true);
        for (int i=0; i<ROWS; i++)
            for (int j=0; j<COLS; j++)
                FI2Sys.out.println(
                  "i=" + i + " j=" + j + " mat[i][j]=" +
                  myMat.getBoolVal(i, j) + "[" +
                  myMat.getIntVal(i, j) + "]");
        // alcune assegnazioni
        myMat.setIntVal(0, 3, 77);
        myMat.setIntVal(0, 4, 0);
        myMat.setIntVal(0, 5, 81);
        myMat.setIntVal(1, 0, 82);
        for (int i=0; i<ROWS; i++)
            for (int j=0; j<COLS; j++)
                FI2Sys.out.println(
                  "i=" + i + " j=" + j + " mat[i][j]=" +
                  myMat.getBoolVal(i, j) + "[" +
                  myMat.getIntVal(i, j) + "]");
        FI2Sys.out.println(
          "\nOra solleva l'eccezione di OutOfBounds");
        myMat.setIntVal(0, 34, 81);
            // solleva eccezione
    } //[m][s] main

} //{c} BinaryMat
