// Caricatore.java -- loader per il simulatore di una macchina "Ram"
// v 2.0 - 27 settembre 1998

import java.io.*;
import java.util.*;

class Caricatore {

  private Hashtable simboli;
  private Hashtable codici;
  private Vector sospesi;
  private Istruzione[] memoria;

  Caricatore (int maxIstruzioni) {
    simboli = new Hashtable();
    codici  = new Hashtable(23);
    sospesi = new Vector();
    memoria = new Istruzione[maxIstruzioni];
    // inizializza il dizionario dei codici
    for (int i = 0; i < Istruzione.TAVOLACODICI.length; i++)
      codici.put(Istruzione.TAVOLACODICI[i], new Integer(i));
    }

// legge una riga da un file - elimina le tabulazioni
  String prendi (InputStream in) {
    try {
      StringBuffer str = new StringBuffer(100);
      String fr = System.getProperties().getProperty("line.separator");
      int cfr = fr.charAt(0);
      int c   = in.read();
      while ((c != -1) && (c != cfr)){
        if (c == '\t')
          c = ' ';
        str.append((char)c);
        c = in.read();
        }
      if ((c == cfr) && (fr.length() == 2))
        in.read();
      if ((c == -1) && (str.length() == 0))
        return null;
      else
        return str.toString();
      } catch (IOException e) {
        System.err.println("Errore di I/O nel caricamento del programma");
        return null;
        }
    }

// carica -- restituisce un vettore di istruzioni nelle quali
// e' stato caricato il progrmma contenuto nello stream di ingresso

  Istruzione[] carica(InputStream in) {
    Integer I;
    int i;
    Coppia c;
    int iic = 0;  // location counter simulato
    String str;
    while ((str = prendi(in)) != null) {
    // elimina eventuali commenti    
      if ((i = str.indexOf('#')) != -1)
        str = str.substring(0, i);
      str = str.trim();
    // l'istruzione e' preceduta da un'etichetta?
      if ((i = str.indexOf(':')) != -1) {
        String etichetta = str.substring(0, i).trim();
        if (simboli.containsKey(etichetta))
          System.err.println("Simbolo duplicato: " + etichetta);
        else
          simboli.put(etichetta, new Integer(iic));
        str = str.substring(i+1).trim();
        }
    // ignora le righe vuote
      if (str.length() == 0)
        continue;
    // assembla l'istruzione
      int codice = 0;
      int tipo = Istruzione.DIRETTO;
      int operando = 0;
      String codiceOp;
      if ((i = str.indexOf(' ')) == -1)
        codiceOp = str;
      else {
        codiceOp = str.substring(0, i);
        str = str.substring(i+1).trim();
        if ((i = str.indexOf('*')) != -1) {
          tipo = Istruzione.INDIRETTO;
          str = str.substring(i+1).trim();
          }
        if ((i = str.indexOf('=')) != -1) {
          tipo = Istruzione.IMMEDIATO;
          str = str.substring(i+1).trim();
          }
      // l'operando e' un numero?
        try {
          operando = Integer.valueOf(str).intValue();
          }
      // operando simbolico
        catch (NumberFormatException e) {
          if ((I = (Integer)simboli.get(str)) == null)
            sospesi.addElement(new Coppia(str, new Integer(iic)));
          else
            operando = I.intValue();
          }
        }
      if ((I = (Integer)codici.get(codiceOp)) == null)
        System.err.println("Codice operativo errato: " + codiceOp);
      else
        codice = I.intValue();
      memoria[iic++] = new Istruzione(codice, tipo, operando);
      }
    // risoluzione dei simboli sospesi
    while (!sospesi.isEmpty()) {
      Coppia cs = (Coppia)sospesi.elementAt(0);      
      sospesi.removeElementAt(0);      
      if ((I = (Integer)simboli.get(cs.primo)) == null)
        System.err.println("Simbolo non definito: " + cs.secondo);
      else
        memoria[((Integer)cs.secondo).intValue()].operando =
            I.intValue();
      }
    return memoria;
    }
  }
