// Ram.java -- simulatore di una macchina "Ram"
// v 2.1 - 27 settembre 1998

import java.io.*;
import java.util.*;

class Ram {

  static final int MMAX = 1000; // posizioni di memoria
  static final int IMAX = 100;  // numero di istruzioni
  static final int OMAX = 1000; // numero massimo di cicli

// modi del simulatore
  static final int VAI = 0;
  static final int STOP = 1;
  static final int ERRORE = 2;

// metodo per leggere una riga dall'ingresso standard
  static private String getLine() {
    StringBuffer buf = new StringBuffer(80);
    int c;
    try {
      while ((c = System.in.read()) != -1) {
        char ch = (char) c;
        if (ch == '\n')
          break;
        buf.append(ch);
        }
      } catch (IOException e) {
        System.err.println("Errore di lettura dal nastro di ingresso");
        }
    return (buf.toString());
    }

  // il simultatore
  public static void main (String[] argomenti) {

  // gestione della riga di comando
    String nomeFile;
    boolean profilo;
    switch (argomenti.length) {
    case 1:
      nomeFile = argomenti[0];
      profilo = false;
      break;
    case 2:
      if (argomenti[0].equals("-p") || argomenti[0].equals("-P")) {
        profilo = true;
        nomeFile = argomenti[1];
        }
      else {
        System.err.println("uso: java Ram [-p] nome-file");
        return;
        }
      break;
    default:
      System.err.println("uso: java Ram [-p] nome-file");
      return;
    }

  // profilo e traccia
    PrintWriter fileTraccia = null;
    PrintWriter fileProfilo = null;
    if (profilo)
      try {
        fileTraccia = new PrintWriter(new FileOutputStream(nomeFile +
              ".tra"));
        fileProfilo = new PrintWriter(new FileOutputStream(nomeFile +
              ".pro"));
        } catch (IOException e) {
          System.err.println
              ("Errore nell'apertura dei file traccia e profilo");
          return;
          }

  // caricamento del programma in memoria
    InputStream in;
    try {
      in = new FileInputStream(nomeFile);
      } catch (FileNotFoundException e) {
        System.err.println("Non trovo il file: " + nomeFile);
        return;
        }
    Caricatore programma = new Caricatore(IMAX);
    Istruzione[] i = programma.carica(in);

  // inizializzazione dello stato della macchina
    int iic        = 0;
    int orologio   = 0;
    int[] m        = new int[MMAX];
    Stack s        = new Stack();
    int[] conta    = new int[IMAX];
    int modo       = VAI;

  // inizio della simulazione
    while (modo == VAI) {
      if (profilo) {
        ++conta[iic];                   // conta i cicli
        fileTraccia.print(iic + ": ");  // fetch e incremento di iic
        }
      orologio++;
      Istruzione ric = i[iic++];
      switch (ric.codiceOperativo) {
      case Istruzione.LOAD:
        if (ric.tipo == Istruzione.IMMEDIATO)
          m[0] = ric.operando;
        else if (ric.tipo == Istruzione.DIRETTO)
          m[0] = m[ric.operando];
        else if (ric.tipo == Istruzione.INDIRETTO)
          m[0] = m[m[ric.operando]];
        else
          modo = ERRORE;
        break;
      case Istruzione.STORE:
        if (ric.tipo == Istruzione.IMMEDIATO)
          modo = ERRORE;
        else if (ric.tipo == Istruzione.DIRETTO)
          m[ric.operando] = m[0];
        else if (ric.tipo == Istruzione.INDIRETTO)
          m[m[ric.operando]] = m[0];
        else
          modo = ERRORE;
        break;
      case Istruzione.ADD:
        if (ric.tipo == Istruzione.IMMEDIATO)
          m[0] += ric.operando;
        else if (ric.tipo == Istruzione.DIRETTO)
          m[0] += m[ric.operando];
        else if (ric.tipo == Istruzione.INDIRETTO)
          m[0] += m[m[ric.operando]];
        else
          modo = ERRORE;
        break;
      case Istruzione.SUB:
        if (ric.tipo == Istruzione.IMMEDIATO)
          m[0] -= ric.operando;
        else if (ric.tipo == Istruzione.DIRETTO)
          m[0] -= m[ric.operando];
        else if (ric.tipo == Istruzione.INDIRETTO)
          m[0] -= m[m[ric.operando]];
        else
          modo = ERRORE;
        break;
      case Istruzione.MULT:
        if (ric.tipo == Istruzione.IMMEDIATO)
          m[0] *= ric.operando;
        else if (ric.tipo == Istruzione.DIRETTO)
          m[0] *= m[ric.operando];
        else if (ric.tipo == Istruzione.INDIRETTO)
          m[0] *= m[m[ric.operando]];
        else
          modo = ERRORE;
        break;
      case Istruzione.DIV:
        if (ric.tipo == Istruzione.IMMEDIATO) {
          if (ric.operando == 0) {
            System.err.println("Divisione per zero");
            modo = ERRORE;
            }
          else
            m[0] /= ric.operando;
          }
        else if (ric.tipo == Istruzione.DIRETTO) {
          if (m[ric.operando] == 0) {
            System.err.println("Divisione per zero");
            modo = ERRORE;
            }
          else
            m[0] /= m[ric.operando];
          }
        else if (ric.tipo == Istruzione.INDIRETTO) {
          if (m[m[ric.operando]] == 0) {
            System.err.println("Divisione per zero");
            modo = ERRORE;
            }
          else
            m[0] /= m[m[ric.operando]];
          }
        else
          modo = ERRORE;
        break;
      case Istruzione.READ:
        try {
          if (ric.tipo == Istruzione.IMMEDIATO)
            modo = ERRORE;  
          else if (ric.tipo == Istruzione.DIRETTO)
            m[ric.operando] =
                    Integer.valueOf(getLine().trim()).intValue();
          else if (ric.tipo == Istruzione.INDIRETTO)
            m[m[ric.operando]] =
                    Integer.valueOf(getLine().trim()).intValue();
          else
            modo = ERRORE;
          }
        catch (NullPointerException e1) {
          System.err.println("Fine del nastro di ingresso");  // possibile ?
          modo = ERRORE;
          }
        catch (NumberFormatException e2) {
          System.err.println("Dato non numerico sul nastro di ingresso");
          modo = ERRORE;
          }
        break;
      case Istruzione.WRITE:
        if (ric.tipo == Istruzione.IMMEDIATO)
          System.out.println(ric.operando);
        else if (ric.tipo == Istruzione.DIRETTO)
          System.out.println(m[ric.operando]);
        else if (ric.tipo == Istruzione.INDIRETTO)
          System.out.println(m[m[ric.operando]]);
        else
          modo = ERRORE;
        break;
      case Istruzione.CALL:
        s.push(new Integer(iic));
        iic = ric.operando;
        break;
      case Istruzione.RET:
        iic = ((Integer)s.pop()).intValue();
        break;
      case Istruzione.PUSH:
        if (ric.tipo == Istruzione.IMMEDIATO)
          s.push(new Integer(ric.operando));
        else if (ric.tipo == Istruzione.DIRETTO)
          s.push(new Integer(m[ric.operando]));
        else if (ric.tipo == Istruzione.INDIRETTO)
          s.push(new Integer(m[m[ric.operando]]));
        else
          modo = ERRORE;
        break;
      case Istruzione.POP:
        if (ric.tipo == Istruzione.IMMEDIATO)
          modo = ERRORE;
        else if (ric.tipo == Istruzione.DIRETTO)
          m[ric.operando] = ((Integer)s.pop()).intValue();
        else if (ric.tipo == Istruzione.INDIRETTO)
          m[m[ric.operando]] = ((Integer)s.pop()).intValue();
        else
          modo = ERRORE;
        break;
      case Istruzione.JUMP:
        iic = ric.operando;
        break;
      case Istruzione.JGZ:
        if (m[0] > 0)
          iic = ric.operando;
        break;
      case Istruzione.JZ:
        if (m[0] == 0)
          iic = ric.operando;
        break;
      case Istruzione.HALT:
        modo = STOP;
        break;
      default:
        modo = ERRORE;
        }   // switch (ric.codiceOperativo)
      if (orologio > OMAX) {
        modo = ERRORE;
        System.err.println("Ha eseguito 1.000 cicli");
        }
      if ((modo == VAI) && (i[iic] == null)) {
        modo = ERRORE;
        System.err.println("Manca l'HALT?");
        }
      if (profilo)
        fileTraccia.println(m[0]);
      }     // while (modo == VAI)
    if (modo == ERRORE)
      System.err.println("Errore in esecuzione");
    else
      System.err.println("HALT: " + orologio + " cicli");
    if (profilo) {
      int k = 0;
      while (i[k] != null) {
        String temp = Integer.toString(conta[k]);
        for (int j = temp.length(); j < 3; j++)
          fileProfilo.print(" ");
        fileProfilo.print(temp + ": ");
        temp = Integer.toString(k);
        for (int j = temp.length(); j < 2; j++)
          fileProfilo.print(" ");
        fileProfilo.print(temp + ": ");
        fileProfilo.print(Istruzione.TAVOLACODICI[i[k].codiceOperativo]);
        for (int j =
              Istruzione.TAVOLACODICI[i[k].codiceOperativo].length();
                    j < 6; j++)
          fileProfilo.print(" ");
        switch (i[k].tipo) {
        case Istruzione.DIRETTO:
          fileProfilo.print("  ");
          break;
        case Istruzione.INDIRETTO:
          fileProfilo.print("* ");
          break;
        case Istruzione.IMMEDIATO:
          fileProfilo.print("= ");
          break;
          }
        fileProfilo.println(i[k].operando);
        k++;
        }
      fileTraccia.close();
      fileProfilo.close();
      }
    }
  }
