// Ram.java -- simulatore di una macchina "Ram"
// v 3.0 - 2 ottobre 1998

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.List;

class XRam {

  static final int MMAX = 1000; // posizioni di memoria
  static final int IMAX = 100;  // numero di istruzioni
  static final int OMAX = 1000; // numero massimo di cicli

  // modi del simulatore
  static final int VAI = 0;
  static final int STOP = 1;
  static final int ERRORE = 2;

  // il simultatore
  public XRam (String[] programma, String[] nastro, TextArea out, List err) {

    err.removeAll();
    if ((programma == null) || (programma.length == 0)) {
      err.add("Manca il programma");
      return;
      }

  // profilo e traccia
    boolean profilo = true;
    PrintWriter fileTraccia = null;
    PrintWriter fileProfilo = null;
    if (profilo)
      try {
        fileTraccia = new PrintWriter(new FileOutputStream("Ram.tra"));
        fileProfilo = new PrintWriter(new FileOutputStream("Ram.pro"));
        } catch (IOException e) {
          err.add
              ("Errore nell'apertura dei file traccia e profilo");
          return;
          }

  // caricamento del programma in memoria
    XCaricatore memoriaI = new XCaricatore(IMAX);
    Istruzione[] i = memoriaI.carica(programma, err);

  // inizializzazione dello stato della macchina
    int iic        = 0;
    int orologio   = 0;
    int posizione  = 0;
    int[] m        = new int[MMAX];
    Stack s        = new Stack();
    int[] conta    = new int[IMAX];
    int modo       = VAI;

  // inizio della simulazione
    while (modo == VAI) {
      if (profilo) {
        ++conta[iic];                   // conta i cicli
        fileTraccia.print(iic + ": "); 
        }
      orologio++;
      Istruzione ric = i[iic++];        // fetch e incremento di iic
      switch (ric.codiceOperativo) {
      case Istruzione.LOAD:
        if (ric.tipo == Istruzione.IMMEDIATO)
          m[0] = ric.operando;
        else if (ric.tipo == Istruzione.DIRETTO)
          m[0] = m[ric.operando];
        else if (ric.tipo == Istruzione.INDIRETTO)
          m[0] = m[m[ric.operando]];
        else
          modo = ERRORE;
        break;
      case Istruzione.STORE:
        if (ric.tipo == Istruzione.IMMEDIATO)
          modo = ERRORE;
        else if (ric.tipo == Istruzione.DIRETTO)
          m[ric.operando] = m[0];
        else if (ric.tipo == Istruzione.INDIRETTO)
          m[m[ric.operando]] = m[0];
        else
          modo = ERRORE;
        break;
      case Istruzione.ADD:
        if (ric.tipo == Istruzione.IMMEDIATO)
          m[0] += ric.operando;
        else if (ric.tipo == Istruzione.DIRETTO)
          m[0] += m[ric.operando];
        else if (ric.tipo == Istruzione.INDIRETTO)
          m[0] += m[m[ric.operando]];
        else
          modo = ERRORE;
        break;
      case Istruzione.SUB:
        if (ric.tipo == Istruzione.IMMEDIATO)
          m[0] -= ric.operando;
        else if (ric.tipo == Istruzione.DIRETTO)
          m[0] -= m[ric.operando];
        else if (ric.tipo == Istruzione.INDIRETTO)
          m[0] -= m[m[ric.operando]];
        else
          modo = ERRORE;
        break;
      case Istruzione.MULT:
        if (ric.tipo == Istruzione.IMMEDIATO)
          m[0] *= ric.operando;
        else if (ric.tipo == Istruzione.DIRETTO)
          m[0] *= m[ric.operando];
        else if (ric.tipo == Istruzione.INDIRETTO)
          m[0] *= m[m[ric.operando]];
        else
          modo = ERRORE;
        break;
      case Istruzione.DIV:
        if (ric.tipo == Istruzione.IMMEDIATO) {
          if (ric.operando == 0) {
            err.add("Divisione per zero");
            modo = ERRORE;
            }
          else
            m[0] /= ric.operando;
          }
        else if (ric.tipo == Istruzione.DIRETTO) {
          if (m[ric.operando] == 0) {
            err.add("Divisione per zero");
            modo = ERRORE;
            }
          else
            m[0] /= m[ric.operando];
          }
        else if (ric.tipo == Istruzione.INDIRETTO) {
          if (m[m[ric.operando]] == 0) {
            err.add("Divisione per zero");
            modo = ERRORE;
            }
          else
            m[0] /= m[m[ric.operando]];
          }
        else
          modo = ERRORE;
        break;
      case Istruzione.READ:
        try {
          if (ric.tipo == Istruzione.IMMEDIATO)
            modo = ERRORE;  
          else if (ric.tipo == Istruzione.DIRETTO)
            m[ric.operando] =
                    Integer.valueOf(nastro[posizione++].trim()).intValue();
          else if (ric.tipo == Istruzione.INDIRETTO)
            m[m[ric.operando]] =
                    Integer.valueOf(nastro[posizione++].trim()).intValue();
          else
            modo = ERRORE;
          }
        catch (ArrayIndexOutOfBoundsException e1) {
          err.add("Lettura del nastro di ingresso vuoto");
          modo = ERRORE;
          }
        catch (NullPointerException e1) {
          err.add("Fine del nastro di ingresso");  // possibile ?
          modo = ERRORE;
          }
        catch (NumberFormatException e2) {
          err.add("Dato non numerico sul nastro di ingresso");
          modo = ERRORE;
          }
        break;
      case Istruzione.WRITE:
        if (ric.tipo == Istruzione.IMMEDIATO)
          out.append(Integer.toString(ric.operando)+'\n');
        else if (ric.tipo == Istruzione.DIRETTO)
          out.append(Integer.toString(m[ric.operando])+'\n');
        else if (ric.tipo == Istruzione.INDIRETTO)
          out.append(Integer.toString(m[m[ric.operando]])+'\n');
        else
          modo = ERRORE;
        break;
      case Istruzione.CALL:
        s.push(new Integer(iic));
        iic = ric.operando;
        break;
      case Istruzione.RET:
        iic = ((Integer)s.pop()).intValue();
        break;
      case Istruzione.PUSH:
        if (ric.tipo == Istruzione.IMMEDIATO)
          s.push(new Integer(ric.operando));
        else if (ric.tipo == Istruzione.DIRETTO)
          s.push(new Integer(m[ric.operando]));
        else if (ric.tipo == Istruzione.INDIRETTO)
          s.push(new Integer(m[m[ric.operando]]));
        else
          modo = ERRORE;
        break;
      case Istruzione.POP:
        if (ric.tipo == Istruzione.IMMEDIATO)
          modo = ERRORE;
        else if (ric.tipo == Istruzione.DIRETTO)
          m[ric.operando] = ((Integer)s.pop()).intValue();
        else if (ric.tipo == Istruzione.INDIRETTO)
          m[m[ric.operando]] = ((Integer)s.pop()).intValue();
        else
          modo = ERRORE;
        break;
      case Istruzione.JUMP:
        iic = ric.operando;
        break;
      case Istruzione.JGZ:
        if (m[0] > 0)
          iic = ric.operando;
        break;
      case Istruzione.JZ:
        if (m[0] == 0)
          iic = ric.operando;
        break;
      case Istruzione.HALT:
        modo = STOP;
        break;
      default:
        modo = ERRORE;
        }   // switch (ric.codiceOperativo)
      if (orologio > OMAX) {
        modo = ERRORE;
        err.add("Ha eseguito 1.000 cicli");
        }
      if ((modo == VAI) && (i[iic] == null)) {
        modo = ERRORE;
        err.add("Manca l'HALT?");
        }
      if (profilo)
        fileTraccia.println(m[0]);
      }     // while (modo == VAI)
    if (modo == ERRORE)
      err.add("Errore in esecuzione");
    else
      err.add("HALT: " + orologio + " cicli");
    if (profilo) {
      int k = 0;
      while (i[k] != null) {
        String temp = Integer.toString(conta[k]);
        for (int j = temp.length(); j < 3; j++)
          fileProfilo.print(" ");
        fileProfilo.print(temp + ": ");
        temp = Integer.toString(k);
        for (int j = temp.length(); j < 2; j++)
          fileProfilo.print(" ");
        fileProfilo.print(temp + ": ");
        fileProfilo.print(Istruzione.TAVOLACODICI[i[k].codiceOperativo]);
        for (int j =
              Istruzione.TAVOLACODICI[i[k].codiceOperativo].length();
                    j < 6; j++)
          fileProfilo.print(" ");
        switch (i[k].tipo) {
        case Istruzione.DIRETTO:
          fileProfilo.print("  ");
          break;
        case Istruzione.INDIRETTO:
          fileProfilo.print("* ");
          break;
        case Istruzione.IMMEDIATO:
          fileProfilo.print("= ");
          break;
          }
        fileProfilo.println(i[k].operando);
        k++;
        }
      fileTraccia.close();
      fileProfilo.close();
      }
    }

  public static void main (String[] argv) {
    new FinestraRam();
    }
  }
